@@ -17,19 +17,20 @@ See also BatchCommandDialog and BatchProcessDialog.
 
 
 #include "Audacity.h"
+#include "BatchCommands.h"
 
 #include <wx/defs.h>
 #include <wx/dir.h>
-#include <wx/msgdlg.h>
 #include <wx/filedlg.h>
 #include <wx/textfile.h>
 
+#include "AudacityApp.h"
 #include "Project.h"
-#include "BatchCommands.h"
 #include "commands/CommandManager.h"
 #include "effects/EffectManager.h"
 #include "FileNames.h"
 #include "Internat.h"
+#include "PluginManager.h"
 #include "Prefs.h"
 #include "Shuttle.h"
 #include "export/ExportFLAC.h"
@@ -40,6 +41,8 @@ See also BatchCommandDialog and BatchProcessDialog.
 #include "Theme.h"
 #include "AllThemeResources.h"
 
+#include "Track.h"
+#include "widgets/ErrorDialog.h"
 
 // KLUDGE: All commands should be on the same footing
 // however, for historical reasons we distinguish between
@@ -51,16 +54,30 @@ enum eCommandType { CtEffect, CtMenu, CtSpecial };
 // TIDY-ME: Not currently translated,
 // but there are issues to address if we do.
 // CLEANSPEECH remnant
-static wxString SpecialCommands[] = {
-   wxT("NoAction"),
-   // wxT("Import"),   // non-functioning
-   wxT("ExportMP3_56k_before"),
-   wxT("ExportMP3_56k_after"),
-   wxT("StereoToMono"),
-   wxT("ExportFLAC"),
-   wxT("ExportMP3"),
-   wxT("ExportOgg"),
-   wxT("ExportWAV")
+static const std::pair<const wxChar*, const wxChar*> SpecialCommands[] = {
+   // Use translations of the first members, some other day.
+   // For 2.2.2 we'll get them into the catalog at least.
+
+   { XO("No Action"),            wxT("NoAction") },
+
+   // { wxT("Import"), wxT("Import") },   // non-functioning
+   /* i18n-hint: before is adverb; MP3 names an audio file format */
+   { XO("Export as MP3 56k before"), wxT("ExportMP3_56k_before") },
+
+   /* i18n-hint: after is adverb; MP3 names an audio file format */
+   { XO("Export as MP3 56k after"),  wxT("ExportMP3_56k_after") },
+
+   /* i18n-hint: FLAC names an audio file format */
+   { XO("Export as FLAC"),          wxT("ExportFLAC") },
+
+   /* i18n-hint: MP3 names an audio file format */
+   { XO("Export as MP3"),           wxT("ExportMP3") },
+
+   /* i18n-hint: Ogg names an audio file format */
+   { XO("Export as Ogg"),           wxT("ExportOgg") },
+
+   /* i18n-hint: WAV names an audio file format */
+   { XO("Export as WAV"),           wxT("ExportWAV") },
 };
 // end CLEANSPEECH remnant
 
@@ -68,6 +85,7 @@ static const wxString MP3Conversion = wxT("MP3 Conversion");
 
 BatchCommands::BatchCommands()
 {
+   mMessage = "";
    ResetChain();
 
    wxArrayString names = GetNames();
@@ -256,38 +274,46 @@ void BatchCommands::SetWavToMp3Chain() // a function per default chain?  This is
 }
 
 // Gets all commands that are valid for this mode.
-wxArrayString BatchCommands::GetAllCommands()
+auto BatchCommands::GetAllCommands() -> CommandNameVector
 {
-   wxArrayString commands;
-   wxString command;
-   commands.Clear();
+   CommandNameVector commands;
 
    AudacityProject *project = GetActiveProject();
    if (!project)
       return commands;
 
-   EffectArray * effects;
-   unsigned int i;
-
    // CLEANSPEECH remnant
-   for(i=0;i<sizeof(SpecialCommands)/sizeof(SpecialCommands[0]);i++)
-   {
-      commands.Add( SpecialCommands[i] );
-   }
+   for( const auto &command : SpecialCommands )
+      commands.push_back( {
+         //wxGetTranslation
+                           (command.first),
+         command.second
+      } );
+
    // end CLEANSPEECH remnant
 
-   int additionalEffects=ADVANCED_EFFECT;
-
-   effects = EffectManager::Get().GetEffects(PROCESS_EFFECT | BUILTIN_EFFECT | PLUGIN_EFFECT | additionalEffects);
-   for(i=0; i<effects->GetCount(); i++) {
-      if ((*effects)[i]->SupportsChains()) {
-         command=(*effects)[i]->GetEffectIdentifier();
-         if (!command.IsEmpty()) {
-            commands.Add( command);
-         }
-      }
+   PluginManager & pm = PluginManager::Get();
+   EffectManager & em = EffectManager::Get();
+   const PluginDescriptor *plug = pm.GetFirstPlugin(PluginTypeEffect);
+   while (plug)
+   {
+      auto command = em.GetEffectIdentifier(plug->GetID());
+      if (!command.IsEmpty())
+         commands.push_back( {
+            plug->GetUntranslatedName(), // plug->GetTranslatedName(),
+            command
+         } );
+      plug = pm.GetNextPlugin(PluginTypeEffect);
    }
-   delete effects;
+
+   // Sort commands by their user-visible names.
+   // PRL:  What should happen if first members of pairs are not unique?
+   // Sort stably?
+   std::sort(
+      commands.begin(), commands.end(),
+      [](const CommandName &a, const CommandName &b)
+         { return a.first < b.first; }
+   );
 
    /* This is for later in development: include the menu commands.
          CommandManager * mManager = project->GetCommandManager();
@@ -302,31 +328,58 @@ wxArrayString BatchCommands::GetAllCommands()
 }
 
 
-wxString BatchCommands::GetCurrentParamsFor(wxString command)
+wxString BatchCommands::GetCurrentParamsFor(const wxString & command)
 {
-   Effect * f = EffectManager::Get().GetEffectByIdentifier( command );
-   if( f==NULL )
-      return wxT("");// effect not found.
-   ShuttleCli shuttle;
-   shuttle.mbStoreInClient=false;
-   f->TransferParameters( shuttle );
-   if( shuttle.mParams.IsEmpty() )
-      return wxT("");// effect had no parameters.
+   const PluginID & ID = EffectManager::Get().GetEffectByIdentifier(command);
+   if (ID.empty())
+   {
+      return wxEmptyString;   // effect not found.
+   }
 
-   return shuttle.mParams;
+   return EffectManager::Get().GetEffectParameters(ID);
 }
 
-bool BatchCommands::PromptForParamsFor(wxString command, wxWindow *parent)
+wxString BatchCommands::PromptForParamsFor(const wxString & command, const wxString & params, wxWindow *parent)
 {
-   Effect * f = EffectManager::Get().GetEffectByIdentifier(command);
-   if( f==NULL )
-      return false;
+   const PluginID & ID = EffectManager::Get().GetEffectByIdentifier(command);
+   if (ID.empty())
+   {
+      return wxEmptyString;   // effect not found
+   }
 
-   //mFactory = factory;
-   //mProjectRate = projectRate;
-   f->mParent = parent;
-   //mTracks = list;
-   return f->PromptUser();
+   wxString res = params;
+
+   auto cleanup = EffectManager::Get().SetBatchProcessing(ID);
+
+   if (EffectManager::Get().SetEffectParameters(ID, params))
+   {
+      if (EffectManager::Get().PromptUser(ID, parent))
+      {
+         res = EffectManager::Get().GetEffectParameters(ID);
+      }
+   }
+
+   return res;
+}
+
+wxString BatchCommands::PromptForPresetFor(const wxString & command, const wxString & params, wxWindow *parent)
+{
+   const PluginID & ID = EffectManager::Get().GetEffectByIdentifier(command);
+   if (ID.empty())
+   {
+      return wxEmptyString;   // effect not found.
+   }
+
+   wxString preset = EffectManager::Get().GetPreset(ID, params, parent);
+
+   // Preset will be empty if the user cancelled the dialog, so return the original
+   // parameter value.
+   if (preset.IsEmpty())
+   {
+      return params;
+   }
+
+   return preset;
 }
 
 double BatchCommands::GetEndTime()
@@ -334,13 +387,13 @@ double BatchCommands::GetEndTime()
    AudacityProject *project = GetActiveProject();
    if( project == NULL )
    {
-      //wxMessageBox( wxT("No project to process!") );
+      //AudacityMessageBox( _("No project to process!") );
       return -1.0;
    }
    TrackList * tracks = project->GetTracks();
    if( tracks == NULL )
    {
-      //wxMessageBox( wxT("No tracks to process!") );
+      //AudacityMessageBox( _("No tracks to process!") );
       return -1.0;
    }
 
@@ -353,14 +406,14 @@ bool BatchCommands::IsMono()
    AudacityProject *project = GetActiveProject();
    if( project == NULL )
    {
-      //wxMessageBox( wxT("No project and no Audio to process!") );
+      //AudacityMessageBox( _("No project and no Audio to process!") );
       return false;
    }
 
    TrackList * tracks = project->GetTracks();
    if( tracks == NULL )
    {
-      //wxMessageBox( wxT("No tracks to process!") );
+      //AudacityMessageBox( _("No tracks to process!") );
       return false;
    }
 
@@ -378,9 +431,57 @@ bool BatchCommands::IsMono()
    return mono;
 }
 
-bool BatchCommands::WriteMp3File( const wxString Name, int bitrate )
+wxString BatchCommands::BuildCleanFileName(const wxString &fileName, const wxString &extension)
+{
+   const wxFileName newFileName{ fileName };
+   wxString justName = newFileName.GetName();
+   wxString pathName = newFileName.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR);
+   const auto cleanedString = _("cleaned");
+
+   if (justName.empty()) {
+      wxDateTime now = wxDateTime::Now();
+      int year = now.GetYear();
+      wxDateTime::Month month = now.GetMonth();
+      wxString monthName = now.GetMonthName(month);
+      int dom = now.GetDay();
+      int hour = now.GetHour();
+      int minute = now.GetMinute();
+      int second = now.GetSecond();
+      justName = wxString::Format(wxT("%d-%s-%02d-%02d-%02d-%02d"),
+           year, monthName, dom, hour, minute, second);
+
+//      SetName(cleanedFileName);
+//      bool isStereo;
+//      double endTime = project->mTracks->GetEndTime();
+//      double startTime = 0.0;
+      //OnSelectAll();
+      pathName = FileNames::FindDefaultPath(FileNames::Operation::Export);
+      ::AudacityMessageBox(wxString::Format(_("Export recording to %s\n/%s/%s%s"),
+            pathName, cleanedString, justName, extension),
+         _("Export recording"),
+         wxOK | wxCENTRE);
+      pathName += wxFileName::GetPathSeparator();
+   }
+   wxString cleanedName = pathName;
+   cleanedName += cleanedString;
+   bool flag  = ::wxFileName::FileExists(cleanedName);
+   if (flag == true) {
+      ::AudacityMessageBox(_("Cannot create directory 'cleaned'. \nFile already exists that is not a directory"));
+      return wxString{};
+   }
+   ::wxFileName::Mkdir(cleanedName, 0777, wxPATH_MKDIR_FULL); // make sure it exists
+
+   cleanedName += wxFileName::GetPathSeparator();
+   cleanedName += justName;
+   cleanedName += extension;
+   wxGetApp().AddFileToHistory(cleanedName);
+
+   return cleanedName;
+}
+
+bool BatchCommands::WriteMp3File( const wxString & Name, int bitrate )
 {  //check if current project is mono or stereo
-   int numChannels = 2;
+   unsigned numChannels = 2;
    if (IsMono()) {
       numChannels = 1;
    }
@@ -400,10 +501,14 @@ bool BatchCommands::WriteMp3File( const wxString Name, int bitrate )
    bool rc;
    long prevBitRate = gPrefs->Read(wxT("/FileFormats/MP3Bitrate"), 128);
    gPrefs->Write(wxT("/FileFormats/MP3Bitrate"), bitrate);
+
+   auto cleanup = finally( [&] {
+      gPrefs->Write(wxT("/FileFormats/MP3Bitrate"), prevBitRate);
+      gPrefs->Flush();
+   } );
+
    // Use Mp3Stereo to control if export is to a stereo or mono file
    rc = mExporter.Process(project, numChannels, wxT("MP3"), Name, false, 0.0, endTime);
-   gPrefs->Write(wxT("/FileFormats/MP3Bitrate"), prevBitRate);
-   gPrefs->Flush();
    return rc;
 }
 
@@ -422,14 +527,14 @@ bool BatchCommands::WriteMp3File( const wxString Name, int bitrate )
 // and think again.
 // ======= IMPORTANT ========
 // CLEANSPEECH remnant
-bool BatchCommands::ApplySpecialCommand(int WXUNUSED(iCommand), const wxString command,const wxString params)
+bool BatchCommands::ApplySpecialCommand(int WXUNUSED(iCommand), const wxString & command,const wxString & params)
 {
    if (ReportAndSkip(command, params))
       return true;
 
    AudacityProject *project = GetActiveProject();
 
-   int numChannels = 1;    //used to switch between mono and stereo export
+   unsigned numChannels = 1;    //used to switch between mono and stereo export
    if (IsMono()) {
       numChannels = 1;  //export in mono
    } else {
@@ -447,15 +552,15 @@ bool BatchCommands::ApplySpecialCommand(int WXUNUSED(iCommand), const wxString c
    else extension = wxT(".mp3");
 
    if (mFileName.IsEmpty()) {
-      filename = project->BuildCleanFileName(project->GetFileName(), extension);
+      filename = BuildCleanFileName(project->GetFileName(), extension);
    }
    else {
-      filename = project->BuildCleanFileName(mFileName, extension);
+      filename = BuildCleanFileName(mFileName, extension);
    }
 
    // We have a command index, but we don't use it!
    // TODO: Make this special-batch-command code use the menu item code....
-   // FIXME: No error reporting on write file failure in batch mode.
+   // FIXME: TRAP_ERR No error reporting on write file failure in batch mode.
    if (command == wxT("NoAction")) {
       return true;
    } else if (!mFileName.IsEmpty() && command == wxT("Import")) {
@@ -467,14 +572,6 @@ bool BatchCommands::ApplySpecialCommand(int WXUNUSED(iCommand), const wxString c
    } else if (command == wxT("ExportMP3_56k_after")) {
       filename.Replace(wxT("cleaned/"), wxT("cleaned/MasterAfter_"), false);
       return WriteMp3File(filename, 56);
-   } else if (command == wxT("StereoToMono")) {
-      // StereoToMono is an effect masquerading as a menu item.
-      Effect * f = EffectManager::Get().GetEffectByIdentifier(wxT("StereoToMono"));
-      if (f != NULL) {
-         return ApplyEffectCommand(f, command, params);
-      }
-      wxMessageBox(_("Stereo to Mono Effect not found"));
-      return false;
    } else if (command == wxT("ExportMP3")) {
       return WriteMp3File(filename, 0); // 0 bitrate means use default/current
    } else if (command == wxT("ExportWAV")) {
@@ -493,7 +590,7 @@ bool BatchCommands::ApplySpecialCommand(int WXUNUSED(iCommand), const wxString c
       }
       return mExporter.Process(project, numChannels, wxT("OGG"), filename, false, 0.0, endTime);
 #else
-      wxMessageBox(_("Ogg Vorbis support is not included in this build of Audacity"));
+      AudacityMessageBox(_("Ogg Vorbis support is not included in this build of Audacity"));
       return false;
 #endif
    } else if (command == wxT("ExportFLAC")) {
@@ -505,35 +602,16 @@ bool BatchCommands::ApplySpecialCommand(int WXUNUSED(iCommand), const wxString c
       }
       return mExporter.Process(project, numChannels, wxT("FLAC"), filename, false, 0.0, endTime);
 #else
-      wxMessageBox(_("FLAC support is not included in this build of Audacity"));
+      AudacityMessageBox(_("FLAC support is not included in this build of Audacity"));
       return false;
 #endif
    }
-   wxMessageBox(wxString::Format(_("Command %s not implemented yet"),command.c_str()));
+   AudacityMessageBox(wxString::Format(_("Command %s not implemented yet"),command));
    return false;
 }
 // end CLEANSPEECH remnant
 
-bool BatchCommands::SetCurrentParametersFor( Effect * f, const wxString command, const wxString params)
-{
-   // transfer the parameters to the effect...
-   if( !params.IsEmpty() )
-   {
-      ShuttleCli shuttle;
-      shuttle.mParams = params;
-      shuttle.mbStoreInClient=true;
-      if( !f->TransferParameters( shuttle ))
-      {
-         wxMessageBox(
-            wxString::Format(
-            _("Could not set parameters of effect %s\n to %s."), command.c_str(),params.c_str() ));
-         return false;
-      }
-   }
-   return true;
-}
-
-bool BatchCommands::ApplyEffectCommand(   Effect * f, const wxString command, const wxString params)
+bool BatchCommands::ApplyEffectCommand(const PluginID & ID, const wxString & command, const wxString & params)
 {
    //Possibly end processing here, if in batch-debug
    if( ReportAndSkip(command, params))
@@ -541,53 +619,65 @@ bool BatchCommands::ApplyEffectCommand(   Effect * f, const wxString command, co
 
    AudacityProject *project = GetActiveProject();
 
-   //FIXME: for later versions may want to not select-all in batch mode.
-   //IF nothing selected, THEN select everything
+   // FIXME: for later versions may want to not select-all in batch mode.
+   // IF nothing selected, THEN select everything
    // (most effects require that you have something selected).
    project->SelectAllIfNone();
 
-   // NOW actually apply the effect.
-   return project->OnEffect(ALL_EFFECTS | CONFIGURED_EFFECT , f, params, false);
+   bool res = false;
+
+   auto cleanup = EffectManager::Get().SetBatchProcessing(ID);
+
+   // transfer the parameters to the effect...
+   if (EffectManager::Get().SetEffectParameters(ID, params))
+   {
+      // and apply the effect...
+      res = project->DoEffect(ID, AudacityProject::OnEffectFlags::kConfigured |
+                                  AudacityProject::OnEffectFlags::kSkipState |
+                                  AudacityProject::OnEffectFlags::kDontRepeatLast);
+   }
+
+   return res;
 }
 
-bool BatchCommands::ApplyCommand(const wxString command, const wxString params)
+bool BatchCommands::ApplyCommand(const wxString & command, const wxString & params)
 {
 
    unsigned int i;
    // Test for a special command.
    // CLEANSPEECH remnant
-   for(i=0;i<sizeof(SpecialCommands)/sizeof(SpecialCommands[0]);i++)
-   {
-      if( command == SpecialCommands[i] )
+   for( i = 0; i < sizeof(SpecialCommands)/sizeof(*SpecialCommands); ++i ) {
+      if( command == SpecialCommands[i].second )
          return ApplySpecialCommand( i, command, params );
    }
    // end CLEANSPEECH remnant
 
    // Test for an effect.
-   Effect * f = EffectManager::Get().GetEffectByIdentifier( command );
-   if( f!=NULL )
-      return ApplyEffectCommand( f, command, params );
+   const PluginID & ID = EffectManager::Get().GetEffectByIdentifier( command );
+   if (!ID.empty())
+   {
+      return ApplyEffectCommand(ID, command, params);
+   }
 
-   wxMessageBox(
+   AudacityMessageBox(
       wxString::Format(
-      _("Your batch command of %s was not recognized."), command.c_str() ));
+      _("Your batch command of %s was not recognized."), command ));
+
    return false;
 }
 
 bool BatchCommands::ApplyCommandInBatchMode(const wxString & command, const wxString &params)
 {
    AudacityProject *project = GetActiveProject();
-   bool rc;
 
    // enter batch mode...
    bool prevShowMode = project->GetShowId3Dialog();
+   auto cleanup = finally( [&] {
+      // exit batch mode...
+      project->SetShowId3Dialog(prevShowMode);
+   } );
 
-   rc = ApplyCommand( command, params );
-
-   // exit batch mode...
-   project->SetShowId3Dialog(prevShowMode);
-
-   return rc;
+   return ApplyCommand( command, params );
 }
 
 // ApplyChain returns true on success, false otherwise.
@@ -595,43 +685,46 @@ bool BatchCommands::ApplyCommandInBatchMode(const wxString & command, const wxSt
 bool BatchCommands::ApplyChain(const wxString & filename)
 {
    mFileName = filename;
-   unsigned int i;
-   bool res = true;
+
+   AudacityProject *proj = GetActiveProject();
+   bool res = false;
+   auto cleanup = finally( [&] {
+      if (!res) {
+         if(proj) {
+            // Chain failed or was cancelled; revert to the previous state
+            proj->RollbackState();
+         }
+      }
+   } );
 
    mAbort = false;
 
-   for (i = 0; i < mCommandChain.GetCount(); i++) {
-      if (!ApplyCommandInBatchMode(mCommandChain[i], mParamsChain[i]) || mAbort) {
-         res = false;
+   size_t i = 0;
+   for (; i < mCommandChain.GetCount(); i++) {
+      if (!ApplyCommandInBatchMode(mCommandChain[i], mParamsChain[i]) || mAbort)
          break;
-      }
    }
 
+   res = (i == mCommandChain.GetCount());
+   if (!res)
+      return false;
+
    mFileName.Empty();
-   AudacityProject *proj = GetActiveProject();
 
-   if (!res)
-   {
-      if(proj) {
-         // Chain failed or was cancelled; revert to the previous state
-         UndoManager *um = proj->GetUndoManager();
-         proj->SetStateTo(um->GetCurrentState());
-      }
-      return false;
-   }
-
-   // Chain was successfully applied; save the new project state
+   // Chain was successfully applied; save the NEW project state
    wxString longDesc, shortDesc;
    wxString name = gPrefs->Read(wxT("/Batch/ActiveChain"), wxEmptyString);
    if (name.IsEmpty())
    {
-      longDesc = wxT("Applied batch chain");
-      shortDesc = wxT("Apply chain");
+      /* i18n-hint: active verb in past tense */
+      longDesc = _("Applied batch chain");
+      shortDesc = _("Apply chain");
    }
    else
    {
-      longDesc = wxString::Format(wxT("Applied batch chain '%s'"), name.c_str());
-      shortDesc = wxString::Format(wxT("Apply '%s'"), name.c_str());
+      /* i18n-hint: active verb in past tense */
+      longDesc = wxString::Format(_("Applied batch chain '%s'"), name);
+      shortDesc = wxString::Format(_("Apply '%s'"), name);
    }
 
    if (!proj)
@@ -679,7 +772,7 @@ void BatchCommands::ResetChain()
 
 // ReportAndSkip() is a diagnostic function that avoids actually
 // applying the requested effect if in batch-debug mode.
-bool BatchCommands::ReportAndSkip(const wxString command, const wxString params)
+bool BatchCommands::ReportAndSkip(const wxString & command, const wxString & params)
 {
    int bDebug;
    gPrefs->Read(wxT("/Batch/Debug"), &bDebug, false);
@@ -689,12 +782,12 @@ bool BatchCommands::ReportAndSkip(const wxString command, const wxString params)
    //TODO: Add a cancel button to these, and add the logic so that we can abort.
    if( params != wxT("") )
    {
-      wxMessageBox( wxString::Format(_("Apply %s with parameter(s)\n\n%s"),command.c_str(), params.c_str()),
+      AudacityMessageBox( wxString::Format(_("Apply %s with parameter(s)\n\n%s"),command, params),
          _("Test Mode"));
    }
    else
    {
-      wxMessageBox( wxString::Format(_("Apply %s"),command.c_str()),
+      AudacityMessageBox( wxString::Format(_("Apply %s"),command),
          _("Test Mode"));
    }
    return true;
@@ -707,9 +800,10 @@ wxArrayString BatchCommands::GetNames()
    wxDir::GetAllFiles(FileNames::ChainDir(), &files, wxT("*.txt"), wxDIR_FILES);
    size_t i;
 
+   wxFileName ff;
    for (i = 0; i < files.GetCount(); i++) {
-      wxFileName f(files[i]);
-      names.Add(f.GetName());
+      ff = (files[i]);
+      names.Add(ff.GetName());
    }
 
    return names;
