@@ -15,6 +15,7 @@
 *//*******************************************************************/
 
 #include "Audacity.h"
+#include "BatchProcessDialog.h"
 
 #include <wx/defs.h>
 #include <wx/checkbox.h>
@@ -29,38 +30,42 @@
 #include <wx/radiobut.h>
 #include <wx/button.h>
 #include <wx/imaglist.h>
-#include <wx/msgdlg.h>
 #include <wx/settings.h>
 
+#include "AudacityException.h"
+#include "ShuttleGui.h"
 #include "Prefs.h"
 #include "Project.h"
-#include "BatchProcessDialog.h"
 #include "Internat.h"
 #include "commands/CommandManager.h"
 #include "effects/Effect.h"
 #include "../images/Arrow.xpm"
+#include "../images/Empty9x16.xpm"
 #include "BatchCommands.h"
+#include "Track.h"
 #include "UndoManager.h"
 
 #include "Theme.h"
 #include "AllThemeResources.h"
 
 #include "FileDialog.h"
+#include "FileNames.h"
 #include "import/Import.h"
+#include "widgets/ErrorDialog.h"
 
 #define ChainsListID       7001
 #define CommandsListID     7002
 #define ApplyToProjectID   7003
 #define ApplyToFilesID     7004
 
-BEGIN_EVENT_TABLE(BatchProcessDialog, wxDialog)
+BEGIN_EVENT_TABLE(BatchProcessDialog, wxDialogWrapper)
    EVT_BUTTON(ApplyToProjectID, BatchProcessDialog::OnApplyToProject)
    EVT_BUTTON(ApplyToFilesID, BatchProcessDialog::OnApplyToFiles)
    EVT_BUTTON(wxID_CANCEL, BatchProcessDialog::OnCancel)
 END_EVENT_TABLE()
 
 BatchProcessDialog::BatchProcessDialog(wxWindow * parent):
-   wxDialog(parent, wxID_ANY, _("Apply Chain"),
+   wxDialogWrapper(parent, wxID_ANY, _("Apply Chain"),
             wxDefaultPosition, wxDefaultSize,
             wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
 {
@@ -145,42 +150,72 @@ void BatchProcessDialog::OnApplyToProject(wxCommandEvent & WXUNUSED(event))
                                     wxLIST_NEXT_ALL,
                                     wxLIST_STATE_SELECTED);
    if (item == -1) {
-      wxMessageBox(_("No chain selected"));
+      AudacityMessageBox(_("No chain selected"));
       return;
    }
    wxString name = mChains->GetItemText(item);
 
-   wxDialog d(this, wxID_ANY, GetTitle());
-   ShuttleGui S(&d, eIsCreating);
+   wxDialog * pD = safenew wxDialogWrapper(this, wxID_ANY, GetTitle());
+   pD->SetName(pD->GetTitle());
+   ShuttleGui S(pD, eIsCreating);
 
    S.StartHorizontalLay(wxCENTER, false);
    {
-      S.StartStatic(wxT(""), false);   // deliberately not translated (!)
+      S.StartStatic( {}, false);   // deliberately not translated (!)
       {
          S.SetBorder(20);
          S.AddFixedText(wxString::Format(_("Applying '%s' to current project"),
-                                         name.c_str()));
+                                         name));
       }
       S.EndStatic();
    }
    S.EndHorizontalLay();
 
-   d.Layout();
-   d.Fit();
-   d.CenterOnScreen();
-   d.Move(-1, 0);
-   d.Show();
-   Hide();
+   pD->Layout();
+   pD->Fit();
+   pD->CenterOnScreen();
+   pD->Move(-1, 0);
+   pD->Show();
 
-   wxWindowDisabler wd;
+   // The Hide() on the next line seems to tickle a bug in wx3,
+   // giving rise to our Bug #1221.  The problem is that on Linux 
+   // the 'Hide' converts us from a Modal into a regular dialog,
+   // as far as closing is concerned.  On Linux we can't close with
+   // EndModal() anymore after this.
+   Hide();
 
    gPrefs->Write(wxT("/Batch/ActiveChain"), name);
    gPrefs->Flush();
 
    mBatchCommands.ReadChain(name);
-   if (!mBatchCommands.ApplyChain()) {
+
+   // The disabler must get deleted before the EndModal() call.  Otherwise,
+   // the menus on OSX will remain disabled.
+   bool success;
+   {
+      wxWindowDisabler wd(pD);
+      success = GuardedCall< bool >(
+         [this]{ return mBatchCommands.ApplyChain(); } );
+   }
+
+   if (!success) {
+      Show();
       return;
    }
+
+#if !defined(__WXMAC__)
+   // Under Linux an EndModal() here crashes (Bug #1221).
+   // But sending a close message instead is OK.
+   wxCloseEvent Evt;
+   Evt.SetId( wxID_OK );
+   Evt.SetEventObject( this);
+   ProcessWindowEvent( Evt );
+#else
+   EndModal(wxID_OK);
+#endif
+
+   // Raise myself again, and the parent window with me
+   Show();
 }
 
 void BatchProcessDialog::OnApplyToFiles(wxCommandEvent & WXUNUSED(event))
@@ -189,7 +224,7 @@ void BatchProcessDialog::OnApplyToFiles(wxCommandEvent & WXUNUSED(event))
                                     wxLIST_NEXT_ALL,
                                     wxLIST_STATE_SELECTED);
    if (item == -1) {
-      wxMessageBox(_("No chain selected"));
+      AudacityMessageBox(_("No chain selected"));
       return;
    }
 
@@ -199,24 +234,22 @@ void BatchProcessDialog::OnApplyToFiles(wxCommandEvent & WXUNUSED(event))
 
    AudacityProject *project = GetActiveProject();
    if (!project->GetIsEmpty()) {
-      wxMessageBox(_("Please save and close the current project first."));
+      AudacityMessageBox(_("Please save and close the current project first."));
       return;
    }
 
-   wxString path = gPrefs->Read(wxT("/DefaultOpenPath"), ::wxGetCwd());
    wxString prompt =  _("Select file(s) for batch processing...");
 
    FormatList l;
    wxString filter;
    wxString all;
 
-   l.DeleteContents(true);
-   wxGetApp().mImporter->GetSupportedImportFormats(&l);
-   for (FormatList::compatibility_iterator n = l.GetFirst(); n; n = n->GetNext()) {
-      Format *f = n->GetData();
+   Importer::Get().GetSupportedImportFormats(&l);
+   for (const auto &format : l) {
+      const Format *f = &format;
 
       wxString newfilter = f->formatName + wxT("|");
-      for (size_t i = 0; i < f->formatExtensions.GetCount(); i++) {
+      for (size_t i = 0; i < f->formatExtensions.size(); i++) {
          if (!newfilter.Contains(wxT("*.") + f->formatExtensions[i] + wxT(";")))
             newfilter += wxT("*.") + f->formatExtensions[i] + wxT(";");
          if (!all.Contains(wxT("*.") + f->formatExtensions[i] + wxT(";")))
@@ -246,7 +279,8 @@ void BatchProcessDialog::OnApplyToFiles(wxCommandEvent & WXUNUSED(event))
       }
    }
 
-   FileDialog dlog(this,
+   auto path = FileNames::FindDefaultPath(FileNames::Operation::Open);
+   FileDialogWrapper dlog(this,
                    prompt,
                    path,
                    wxT(""),
@@ -257,27 +291,29 @@ void BatchProcessDialog::OnApplyToFiles(wxCommandEvent & WXUNUSED(event))
    if (dlog.ShowModal() != wxID_OK) {
       return;
    }
-
+   
    wxArrayString files;
    dlog.GetPaths(files);
 
    files.Sort();
 
-   wxDialog d(this, wxID_ANY, GetTitle());
-   ShuttleGui S(&d, eIsCreating);
+   wxDialog * pD = safenew wxDialogWrapper(this, wxID_ANY, GetTitle());
+   pD->SetName(pD->GetTitle());
+   ShuttleGui S(pD, eIsCreating);
 
    S.StartVerticalLay(false);
    {
       S.StartStatic(_("Applying..."), 1);
       {
-         wxImageList *imageList = new wxImageList(9, 16);
-         imageList->Add(wxIcon(empty_9x16_xpm));
+         auto imageList = std::make_unique<wxImageList>(9, 16);
+         imageList->Add(wxIcon(empty9x16_xpm));
          imageList->Add(wxIcon(arrow_xpm));
 
          S.SetStyle(wxSUNKEN_BORDER | wxLC_REPORT | wxLC_HRULES | wxLC_VRULES |
                     wxLC_SINGLE_SEL);
          mList = S.Id(CommandsListID).AddListControlReportMode();
-         mList->AssignImageList(imageList, wxIMAGE_LIST_SMALL);
+         // AssignImageList takes ownership
+         mList->AssignImageList(imageList.release(), wxIMAGE_LIST_SMALL);
          mList->InsertColumn(0, _("File"), wxLIST_FORMAT_LEFT);
       }
       S.EndStatic();
@@ -305,17 +341,17 @@ void BatchProcessDialog::OnApplyToFiles(wxCommandEvent & WXUNUSED(event))
       mList->SetInitialSize(sz);
    }
 
-   d.Layout();
-   d.Fit();
-   d.SetSizeHints(d.GetSize());
-   d.CenterOnScreen();
-   d.Move(-1, 0);
-   d.Show();
+   pD->Layout();
+   pD->Fit();
+   pD->SetSizeHints(pD->GetSize());
+   pD->CenterOnScreen();
+   pD->Move(-1, 0);
+   pD->Show();
    Hide();
 
    mBatchCommands.ReadChain(name);
    for (i = 0; i < (int)files.GetCount(); i++) {
-      wxWindowDisabler wd(&d);
+      wxWindowDisabler wd(pD);
       if (i > 0) {
          //Clear the arrow in previous item.
          mList->SetItemImage(i - 1, 0, 0);
@@ -323,25 +359,60 @@ void BatchProcessDialog::OnApplyToFiles(wxCommandEvent & WXUNUSED(event))
       mList->SetItemImage(i, 1, 1);
       mList->EnsureVisible(i);
 
-      project->OnRemoveTracks();
-      project->Import(files[i]);
-      project->OnSelectAll();
-      if (!mBatchCommands.ApplyChain()) {
-         break;
-      }
+      auto success = GuardedCall< bool >( [&] {
+         project->Import(files[i]);
+         project->ZoomAfterImport(nullptr);
+         project->OnSelectAll(*project);
+         if (!mBatchCommands.ApplyChain())
+            return false;
 
-      if (!d.IsShown() || mAbort) {
+         if (!pD->IsShown() || mAbort)
+            return false;
+
+         return true;
+      } );
+
+      if (!success)
          break;
-      }
+      
       UndoManager *um = project->GetUndoManager();
       um->ClearStates();
+      project->OnSelectAll(*project);
+      project->OnRemoveTracks(*project);
    }
-   project->OnRemoveTracks();
+   project->OnRemoveTracks(*project);
+
+   // Under Linux an EndModal() here crashes (Bug #1221).
+   // But sending a close message instead is OK.
+#if !defined(__WXMAC__)
+   wxCloseEvent Evt;
+   Evt.SetId( wxID_OK );
+   Evt.SetEventObject( this);
+   ProcessWindowEvent( Evt );
+#else
+   EndModal(wxID_OK);
+#endif 
+
+   // Raise myself again, and the parent window with me
+   Show();
 }
 
 void BatchProcessDialog::OnCancel(wxCommandEvent & WXUNUSED(event))
 {
-   EndModal(false);
+#if !defined(__WXMAC__)
+   // It is possible that we could just do EndModal()
+   // here even on Linux.  However, we know the alternative way of
+   // closing works, if we are hidden, so we hide and then do that.
+   Hide();
+   // Under Linux an EndModal() here potentially crashes (Bug #1221).
+   // But sending a close message instead is OK.
+   wxCloseEvent Evt;
+   Evt.SetId( wxID_CANCEL );
+   Evt.SetEventObject( this);
+   ProcessWindowEvent( Evt );
+#else
+   EndModal(wxID_CANCEL);
+#endif
 }
 
 /////////////////////////////////////////////////////////////////////
@@ -363,7 +434,7 @@ enum {
    RenameButtonID
 };
 
-BEGIN_EVENT_TABLE(EditChainsDialog, wxDialog)
+BEGIN_EVENT_TABLE(EditChainsDialog, wxDialogWrapper)
    EVT_LIST_ITEM_SELECTED(ChainsListID, EditChainsDialog::OnChainSelected)
    EVT_LIST_ITEM_SELECTED(CommandsListID, EditChainsDialog::OnListSelected)
    EVT_LIST_BEGIN_LABEL_EDIT(ChainsListID, EditChainsDialog::OnChainsBeginEdit)
@@ -395,7 +466,7 @@ enum {
 
 /// Constructor
 EditChainsDialog::EditChainsDialog(wxWindow * parent):
-   wxDialog(parent, wxID_ANY, _("Edit Chains"),
+   wxDialogWrapper(parent, wxID_ANY, _("Edit Chains"),
             wxDefaultPosition, wxDefaultSize,
             wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
 {
@@ -415,6 +486,8 @@ EditChainsDialog::~EditChainsDialog()
 /// Creates the dialog and its contents.
 void EditChainsDialog::Populate()
 {
+   mCommandNames = BatchCommands::GetAllCommands();
+
    //------------------------- Main section --------------------
    ShuttleGui S(this, eIsCreating);
    PopulateOrExchange(S);
@@ -468,7 +541,8 @@ void EditChainsDialog::PopulateOrExchange(ShuttleGui & S)
          S.SetStyle(wxSUNKEN_BORDER | wxLC_REPORT | wxLC_HRULES | wxLC_SINGLE_SEL |
                     wxLC_EDIT_LABELS);
          mChains = S.Id(ChainsListID).AddListControlReportMode();
-         mChains->InsertColumn(0, wxT("Chain"), wxLIST_FORMAT_LEFT);
+         // i18n-hint: This is the heading for a column in the edit chains dialog
+         mChains->InsertColumn(0, _("Chain"), wxLIST_FORMAT_LEFT);
          S.StartHorizontalLay(wxCENTER, false);
          {
             S.Id(AddButtonID).AddButton(_("&Add"));
@@ -555,11 +629,20 @@ void EditChainsDialog::PopulateList()
 /// Add one item into mList
 void EditChainsDialog::AddItem(const wxString &Action, const wxString &Params)
 {
+   // Translate internal command name to a friendly form
+   auto item = make_iterator_range(mCommandNames).index_if(
+      [&](const CommandName &name){ return Action == name.second; }
+   );
+   auto friendlyName = item >= 0
+      ? // wxGetTranslation
+                           ( mCommandNames[item].first )
+      : Action;
+
    int i = mList->GetItemCount();
 
    mList->InsertItem(i, wxT(""));
    mList->SetItem(i, ItemNumberColumn, wxString::Format(wxT(" %02i"), i + 1));
-   mList->SetItem(i, ActionColumn, Action );
+   mList->SetItem(i, ActionColumn, friendlyName );
    mList->SetItem(i, ParamsColumn, Params );
 }
 
@@ -570,10 +653,10 @@ bool EditChainsDialog::ChangeOK()
       wxString msg;
       int id;
 
-      title.Printf(_("%s changed"), mActiveChain.c_str());
+      title.Printf(_("%s changed"), mActiveChain);
       msg = _("Do you want to save the changes?");
 
-      id = wxMessageBox(msg, title, wxYES_NO | wxCANCEL);
+      id = AudacityMessageBox(msg, title, wxYES_NO | wxCANCEL);
       if (id == wxCANCEL) {
          return false;
       }
@@ -707,9 +790,10 @@ void EditChainsDialog::OnChainsEndEdit(wxListEvent &event)
 void EditChainsDialog::OnAdd(wxCommandEvent & WXUNUSED(event))
 {
    while (true) {
-      wxTextEntryDialog d(this,
+      AudacityTextEntryDialog d(this,
                           _("Enter name of new chain"),
-                          GetTitle());
+                          _("Name of new chain"));
+      d.SetName(d.GetTitle());
       wxString name;
 
       if (d.ShowModal() == wxID_CANCEL) {
@@ -719,7 +803,7 @@ void EditChainsDialog::OnAdd(wxCommandEvent & WXUNUSED(event))
       name = d.GetValue().Strip(wxString::both);
 
       if (name.Length() == 0) {
-         wxMessageBox(_("Name must not be blank"),
+         AudacityMessageBox(_("Name must not be blank"),
                       GetTitle(),
                       wxOK | wxICON_ERROR,
                       this);
@@ -729,7 +813,7 @@ void EditChainsDialog::OnAdd(wxCommandEvent & WXUNUSED(event))
       if (name.Contains(wxFILE_SEP_PATH) ||
           name.Contains(wxFILE_SEP_PATH_UNIX)) {
          /*i18n-hint: The %c will be replaced with 'forbidden characters', like '/' and '\'.*/
-         wxMessageBox(wxString::Format(_("Names may not contain '%c' and '%c'"),
+         AudacityMessageBox(wxString::Format(_("Names may not contain '%c' and '%c'"),
                       wxFILE_SEP_PATH, wxFILE_SEP_PATH_UNIX),
                       GetTitle(),
                       wxOK | wxICON_ERROR,
@@ -758,9 +842,9 @@ void EditChainsDialog::OnRemove(wxCommandEvent & WXUNUSED(event))
    }
 
    wxString name = mChains->GetItemText(item);
-   wxMessageDialog m(this,
+   AudacityMessageDialog m(this,
    /*i18n-hint: %s will be replaced by the name of a file.*/
-                     wxString::Format(_("Are you sure you want to delete %s?"), name.c_str()),
+                     wxString::Format(_("Are you sure you want to delete %s?"), name),
                      GetTitle(),
                      wxYES_NO | wxICON_QUESTION);
    if (m.ShowModal() == wxID_NO) {
