@@ -9,6 +9,7 @@
 *******************************************************************/
 
 #include "Screenshot.h"
+#include "MemoryX.h"
 #include "commands/ScreenshotCommand.h"
 #include "commands/CommandTargets.h"
 #include "commands/CommandDirectory.h"
@@ -34,18 +35,20 @@
 #include "Prefs.h"
 #include "toolbars/ToolManager.h"
 
+#include "Track.h"
+
 class CommandType;
 
 ////////////////////////////////////////////////////////////////////////////////
 
-class ScreenFrame:public wxFrame
+class ScreenFrame final : public wxFrame
 {
  public:
    // constructors and destructors
    ScreenFrame(wxWindow *parent, wxWindowID id);
    virtual ~ScreenFrame();
 
-   virtual bool ProcessEvent(wxEvent & event);
+   bool ProcessEvent(wxEvent & event) override;
 
  private:
    void Populate();
@@ -62,19 +65,7 @@ class ScreenFrame:public wxFrame
    void OnToggleBackgroundWhite(wxCommandEvent & event);
 
    void DoCapture(wxString captureMode);
-   void OnCaptureWindowContents(wxCommandEvent & event);
-   void OnCaptureFullWindow(wxCommandEvent & event);
-   void OnCaptureWindowPlus(wxCommandEvent & event);
-   void OnCaptureFullScreen(wxCommandEvent & event);
-   void OnCaptureToolbars(wxCommandEvent & event);
-   void OnCaptureSelectionBar(wxCommandEvent & event);
-   void OnCaptureTools(wxCommandEvent & event);
-   void OnCaptureTransport(wxCommandEvent & event);
-   void OnCaptureMixer(wxCommandEvent & event);
-   void OnCaptureMeter(wxCommandEvent & event);
-   void OnCaptureEdit(wxCommandEvent & event);
-   void OnCaptureDevice(wxCommandEvent & event);
-   void OnCaptureTranscription(wxCommandEvent & event);
+   void OnCaptureSomething(wxCommandEvent & event);
 
    void TimeZoom(double seconds);
    void OnOneSec(wxCommandEvent & event);
@@ -88,13 +79,7 @@ class ScreenFrame:public wxFrame
    void OnMedTracks(wxCommandEvent & event);
    void OnTallTracks(wxCommandEvent & event);
 
-   void OnCaptureTrackPanel(wxCommandEvent & event);
-   void OnCaptureRuler(wxCommandEvent & event);
-   void OnCaptureTracks(wxCommandEvent & event);
-   void OnCaptureFirstTrack(wxCommandEvent & event);
-   void OnCaptureSecondTrack(wxCommandEvent & event);
-
-   ScreenshotCommand *CreateCommand();
+   std::unique_ptr<ScreenshotCommand> CreateCommand();
 
    wxCheckBox *mDelayCheckBox;
    wxTextCtrl *mDirectoryTextBox;
@@ -102,20 +87,30 @@ class ScreenFrame:public wxFrame
    wxToggleButton *mWhite;
    wxStatusBar *mStatus;
 
-   ScreenshotCommand *mCommand;
+   std::unique_ptr<ScreenshotCommand> mCommand;
    CommandExecutionContext mContext;
 
    DECLARE_EVENT_TABLE()
 };
 
-static ScreenFrame *mFrame = NULL;
+// Static pointer to the unique ScreenFrame window.
+// Formerly it was parentless, therefore this was a Destroy_ptr<ScreenFrame>
+// But now the window is owned, so just use a bare pointer, and null it when
+// the unique window is destroyed.
+using ScreenFramePtr = ScreenFrame*;
+ScreenFramePtr mFrame;
 
 ////////////////////////////////////////////////////////////////////////////////
 
 void OpenScreenshotTools()
 {
    if (!mFrame) {
-      mFrame = new ScreenFrame(NULL, -1);
+      auto parent = wxGetApp().GetTopWindow();
+      if (!parent) {
+         wxASSERT(false);
+         return;
+      }
+      mFrame = ScreenFramePtr{ safenew ScreenFrame(parent, -1) };
    }
    mFrame->Show();
    mFrame->Raise();
@@ -123,35 +118,32 @@ void OpenScreenshotTools()
 
 void CloseScreenshotTools()
 {
-   if (mFrame) {
-      mFrame->Destroy();
-      mFrame = NULL;
-   }
+   mFrame = nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-class ScreenFrameTimer:public wxTimer
+class ScreenFrameTimer final : public wxTimer
 {
  public:
    ScreenFrameTimer(ScreenFrame *frame,
                     wxEvent & event)
    {
       screenFrame = frame;
-      evt = event.Clone();
+      evt.reset(event.Clone());
    }
 
-   virtual void Notify()
+   void Notify() override
    {
+      // Process timer notification just once, then destroy self
       evt->SetEventObject(NULL);
       screenFrame->ProcessEvent(*evt);
-      delete evt;
       delete this;
    }
 
  private:
    ScreenFrame *screenFrame;
-   wxEvent *evt;
+   std::unique_ptr<wxEvent> evt;
 };
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -176,34 +168,45 @@ enum
 
    IdDelayCheckBox,
 
-   IdCaptureToolbars,
+   IdCaptureFirst,
+   // No point delaying the capture of sets of things.
+   IdCaptureMenus= IdCaptureFirst,
+   IdCaptureEffects,
+   IdCapturePreferences,
+
+   // Put all events that need delay between AllDelayed and LastDelayed.
+   IdAllDelayedEvents,
+   IdCaptureToolbars  =IdAllDelayedEvents,
+   IdCaptureWindowContents,
+   IdCaptureFullWindow,
+   IdCaptureWindowPlus,
+   IdCaptureFullScreen,
+  
    IdCaptureSelectionBar,
+   IdCaptureSpectralSelection,
    IdCaptureTools,
    IdCaptureTransport,
    IdCaptureMixer,
    IdCaptureMeter,
+   IdCapturePlayMeter,
+   IdCaptureRecordMeter,
    IdCaptureEdit,
    IdCaptureDevice,
    IdCaptureTranscription,
+   IdCaptureScrub,
 
    IdCaptureTrackPanel,
    IdCaptureRuler,
    IdCaptureTracks,
    IdCaptureFirstTrack,
    IdCaptureSecondTrack,
+   IdCaptureLast = IdCaptureSecondTrack,
+
+   IdLastDelayedEvent,
 
    IdToggleBackgroundBlue,
    IdToggleBackgroundWhite,
 
-   // Put all events that might need delay below:
-   IdAllDelayedEvents,
-
-   IdCaptureWindowContents,
-   IdCaptureFullWindow,
-   IdCaptureWindowPlus,
-   IdCaptureFullScreen,
-
-   IdLastDelayedEvent,
 };
 
 BEGIN_EVENT_TABLE(ScreenFrame, wxFrame)
@@ -215,20 +218,7 @@ BEGIN_EVENT_TABLE(ScreenFrame, wxFrame)
    EVT_BUTTON(IdMainWindowLarge,        ScreenFrame::OnMainWindowLarge)
    EVT_TOGGLEBUTTON(IdToggleBackgroundBlue,   ScreenFrame::OnToggleBackgroundBlue)
    EVT_TOGGLEBUTTON(IdToggleBackgroundWhite,  ScreenFrame::OnToggleBackgroundWhite)
-   EVT_BUTTON(IdCaptureWindowContents,  ScreenFrame::OnCaptureWindowContents)
-   EVT_BUTTON(IdCaptureFullWindow,      ScreenFrame::OnCaptureFullWindow)
-   EVT_BUTTON(IdCaptureWindowPlus,      ScreenFrame::OnCaptureWindowPlus)
-   EVT_BUTTON(IdCaptureFullScreen,      ScreenFrame::OnCaptureFullScreen)
-
-   EVT_BUTTON(IdCaptureToolbars,        ScreenFrame::OnCaptureToolbars)
-   EVT_BUTTON(IdCaptureSelectionBar,    ScreenFrame::OnCaptureSelectionBar)
-   EVT_BUTTON(IdCaptureTools,           ScreenFrame::OnCaptureTools)
-   EVT_BUTTON(IdCaptureTransport,       ScreenFrame::OnCaptureTransport)
-   EVT_BUTTON(IdCaptureMixer,           ScreenFrame::OnCaptureMixer)
-   EVT_BUTTON(IdCaptureMeter,           ScreenFrame::OnCaptureMeter)
-   EVT_BUTTON(IdCaptureEdit,            ScreenFrame::OnCaptureEdit)
-   EVT_BUTTON(IdCaptureDevice,          ScreenFrame::OnCaptureDevice)
-   EVT_BUTTON(IdCaptureTranscription,   ScreenFrame::OnCaptureTranscription)
+   EVT_COMMAND_RANGE(IdCaptureFirst, IdCaptureLast, wxEVT_COMMAND_BUTTON_CLICKED, ScreenFrame::OnCaptureSomething)
 
    EVT_BUTTON(IdOneSec,                 ScreenFrame::OnOneSec)
    EVT_BUTTON(IdTenSec,                 ScreenFrame::OnTenSec)
@@ -240,36 +230,38 @@ BEGIN_EVENT_TABLE(ScreenFrame, wxFrame)
    EVT_BUTTON(IdMedTracks,              ScreenFrame::OnMedTracks)
    EVT_BUTTON(IdTallTracks,             ScreenFrame::OnTallTracks)
 
-   EVT_BUTTON(IdCaptureTrackPanel,      ScreenFrame::OnCaptureTrackPanel)
-   EVT_BUTTON(IdCaptureRuler,           ScreenFrame::OnCaptureRuler)
-   EVT_BUTTON(IdCaptureTracks,          ScreenFrame::OnCaptureTracks)
-   EVT_BUTTON(IdCaptureFirstTrack,      ScreenFrame::OnCaptureFirstTrack)
-   EVT_BUTTON(IdCaptureSecondTrack,     ScreenFrame::OnCaptureSecondTrack)
-
    EVT_BUTTON(IdDirChoose,              ScreenFrame::OnDirChoose)
 END_EVENT_TABLE();
 
 // Must not be called before CreateStatusBar!
-ScreenshotCommand *ScreenFrame::CreateCommand()
+std::unique_ptr<ScreenshotCommand> ScreenFrame::CreateCommand()
 {
    wxASSERT(mStatus != NULL);
-   CommandOutputTarget *output =
-      new CommandOutputTarget(new NullProgressTarget(),
-                              new StatusBarTarget(*mStatus),
-                              new MessageBoxTarget());
+   auto output =
+      std::make_unique<CommandOutputTarget>(std::make_unique<NullProgressTarget>(),
+                              std::make_shared<StatusBarTarget>(*mStatus),
+                              std::make_shared<MessageBoxTarget>());
    CommandType *type = CommandDirectory::Get()->LookUp(wxT("Screenshot"));
    wxASSERT_MSG(type != NULL, wxT("Screenshot command doesn't exist!"));
-   return new ScreenshotCommand(*type, output, this);
+   return std::make_unique<ScreenshotCommand>(*type, std::move(output), this);
 }
 
 ScreenFrame::ScreenFrame(wxWindow * parent, wxWindowID id)
 :  wxFrame(parent, id, _("Screen Capture Frame"),
            wxDefaultPosition, wxDefaultSize,
+
 #if !defined(__WXMSW__)
-           wxFRAME_TOOL_WINDOW|
+
+   #if !defined(__WXMAC__) // bug1358
+           wxFRAME_TOOL_WINDOW |
+   #endif
+
 #else
+
            wxSTAY_ON_TOP|
+
 #endif
+
            wxSYSTEM_MENU|wxCAPTION|wxCLOSE_BOX),
    mContext(&wxGetApp(), GetActiveProject())
 {
@@ -285,12 +277,19 @@ ScreenFrame::ScreenFrame(wxWindow * parent, wxWindowID id)
    // Note that the audio could be playing.
    // The monitoring will switch off temporarily
    // because we've switched monitor mid play.
-   mContext.proj->mToolManager->Reset();
+   // Bug 383 - Resetting the toolbars is not wanted.
+   // Any that are invisible will be amde visible as/when needed.
+   //mContext.GetProject()->GetToolManager()->Reset();
+   Center();
 }
 
 ScreenFrame::~ScreenFrame()
 {
-   delete mCommand;
+   if (this == mFrame)
+      mFrame = nullptr;
+   else
+      // There should only be one!
+      wxASSERT(false);
 }
 
 void ScreenFrame::Populate()
@@ -331,13 +330,14 @@ void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
             S.Id(IdMainWindowLarge).AddButton(_("Resize Large"));
             /* i18n-hint: Bkgnd is short for background and appears on a small button
              * It is OK to just translate this item as if it said 'Blue' */
-            mBlue = new wxToggleButton(p,
+            wxASSERT(p); // To justify safenew
+            mBlue = safenew wxToggleButton(p,
                                        IdToggleBackgroundBlue,
                                        _("Blue Bkgnd"));
             S.AddWindow(mBlue);
             /* i18n-hint: Bkgnd is short for background and appears on a small button
              * It is OK to just translate this item as if it said 'White' */
-            mWhite = new wxToggleButton(p,
+            mWhite = safenew wxToggleButton(p,
                                         IdToggleBackgroundWhite,
                                         _("White Bkgnd"));
             S.AddWindow(mWhite);
@@ -373,7 +373,18 @@ void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
          S.StartHorizontalLay();
          {
             S.Id(IdCaptureToolbars).AddButton(_("All Toolbars"));
+#ifdef EXPERIMENTAL_DOCS_AUTOMATION
+            S.Id(IdCaptureMenus).AddButton(_("All Menus"));
+            S.Id(IdCaptureEffects).AddButton(_("All Effects"));
+            S.Id(IdCapturePreferences).AddButton(_("All Preferences"));
+#endif
+         }
+         S.EndHorizontalLay();
+
+         S.StartHorizontalLay();
+         {
             S.Id(IdCaptureSelectionBar).AddButton(_("SelectionBar"));
+            S.Id(IdCaptureSpectralSelection).AddButton(_("Spectral Selection"));
             S.Id(IdCaptureTools).AddButton(_("Tools"));
             S.Id(IdCaptureTransport).AddButton(_("Transport"));
          }
@@ -383,9 +394,17 @@ void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
          {
             S.Id(IdCaptureMixer).AddButton(_("Mixer"));
             S.Id(IdCaptureMeter).AddButton(_("Meter"));
+            S.Id(IdCapturePlayMeter).AddButton(_("Play Meter"));
+            S.Id(IdCaptureRecordMeter).AddButton(_("Record Meter"));
+         }
+         S.EndHorizontalLay();
+
+         S.StartHorizontalLay();
+         {
             S.Id(IdCaptureEdit).AddButton(_("Edit"));
             S.Id(IdCaptureDevice).AddButton(_("Device"));
             S.Id(IdCaptureTranscription).AddButton(_("Transcription"));
+            S.Id(IdCaptureScrub).AddButton(_("Scrub"));
          }
          S.EndHorizontalLay();
 
@@ -426,7 +445,7 @@ void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
    S.EndPanel();
 
    Layout();
-   Fit();
+   GetSizer()->Fit(this);
    SetMinSize(GetSize());
 
    int top = 0;
@@ -447,22 +466,27 @@ void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
       CentreOnParent();
    }
 
-   SetIcon(mContext.proj->GetIcon());
+   SetIcon(mContext.GetProject()->GetIcon());
 }
 
 bool ScreenFrame::ProcessEvent(wxEvent & e)
 {
    int id = e.GetId();
 
+   // If split into two parts to make for easier breakpoint
+   // when testing timer.
    if (mDelayCheckBox &&
        mDelayCheckBox->GetValue() &&
        e.IsCommandEvent() &&
-       e.GetEventType() == wxEVT_COMMAND_BUTTON_CLICKED &&
-       id >= IdAllDelayedEvents && id <= IdLastDelayedEvent &&
+       e.GetEventType() == wxEVT_COMMAND_BUTTON_CLICKED)
+   {
+      if( id >= IdAllDelayedEvents && id <= IdLastDelayedEvent &&
        e.GetEventObject() != NULL) {
-      ScreenFrameTimer *timer = new ScreenFrameTimer(this, e);
-      timer->Start(5000, true);
-      return true;
+         // safenew because it's a one-shot that deletes itself
+         ScreenFrameTimer *timer = safenew ScreenFrameTimer(this, e);
+         timer->Start(5000, true);
+         return true;
+      }
    }
 
    if (e.IsCommandEvent() && e.GetEventObject() == NULL) {
@@ -473,14 +497,13 @@ bool ScreenFrame::ProcessEvent(wxEvent & e)
 
 void ScreenFrame::OnCloseWindow(wxCloseEvent &  WXUNUSED(event))
 {
-   mFrame = NULL;
    Destroy();
 }
 
 void ScreenFrame::OnUIUpdate(wxUpdateUIEvent &  WXUNUSED(event))
 {
 #ifdef __WXMAC__
-   wxTopLevelWindow *top = mCommand->GetFrontWindow(mContext.proj);
+   wxTopLevelWindow *top = mCommand->GetFrontWindow(GetActiveProject());
    bool needupdate = false;
    bool enable = false;
 
@@ -508,7 +531,7 @@ void ScreenFrame::OnDirChoose(wxCommandEvent & WXUNUSED(event))
 {
    wxString current = mDirectoryTextBox->GetValue();
 
-   wxDirDialog dlog(this,
+   wxDirDialogWrapper dlog(this,
                     _("Choose a location to save screenshot images"),
                     current);
 
@@ -541,9 +564,10 @@ void ScreenFrame::SizeMainWindow(int w, int h)
 {
    int top = 20;
 
-   mContext.proj->Maximize(false);
-   mContext.proj->SetSize(16, 16 + top, w, h);
-   mContext.proj->mToolManager->Reset();
+   mContext.GetProject()->Maximize(false);
+   mContext.GetProject()->SetSize(16, 16 + top, w, h);
+   //Bug383 - Toolbar Resets not wanted.
+   //mContext.GetProject()->GetToolManager()->Reset();
 }
 
 void ScreenFrame::OnMainWindowSmall(wxCommandEvent & WXUNUSED(event))
@@ -563,106 +587,52 @@ void ScreenFrame::DoCapture(wxString captureMode)
 
    mCommand->SetParameter(wxT("CaptureMode"), captureMode);
    if (!mCommand->Apply(mContext))
-      mStatus->SetStatusText(wxT("Capture failed!"));
+      mStatus->SetStatusText(_("Capture failed!"), mainStatusBarField);
    Show();
 }
 
-void ScreenFrame::OnCaptureWindowContents(wxCommandEvent & WXUNUSED(event))
+void ScreenFrame::OnCaptureSomething(wxCommandEvent &  event)
 {
-   DoCapture(wxT("window"));
-}
+   int i = event.GetId() - IdCaptureFirst;
 
-void ScreenFrame::OnCaptureFullWindow(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("fullwindow"));
-}
+   wxArrayString Names;
 
-void ScreenFrame::OnCaptureWindowPlus(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("windowplus"));
-}
+   Names.Add(wxT("menus"));
+   Names.Add(wxT("effects"));
+   Names.Add(wxT("preferences"));
 
-void ScreenFrame::OnCaptureFullScreen(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("fullscreen"));
-}
+   Names.Add(wxT("toolbars"));
+   Names.Add(wxT("window"));
+   Names.Add(wxT("fullwindow"));
+   Names.Add(wxT("windowplus"));
+   Names.Add(wxT("fullscreen"));
+   Names.Add(wxT("selectionbar"));
+   Names.Add(wxT("spectralselection"));
+   Names.Add(wxT("tools"));
+   Names.Add(wxT("transport"));
+   Names.Add(wxT("mixer"));
+   Names.Add(wxT("meter"));
+   Names.Add(wxT("playmeter"));
+   Names.Add(wxT("recordmeter"));
+   Names.Add(wxT("edit"));
+   Names.Add(wxT("device"));
+   Names.Add(wxT("transcription"));
+   Names.Add(wxT("scrub"));
+   Names.Add(wxT("trackpanel"));
+   Names.Add(wxT("ruler"));
+   Names.Add(wxT("tracks"));
+   Names.Add(wxT("firsttrack"));
+   Names.Add(wxT("secondtrack"));
 
-void ScreenFrame::OnCaptureToolbars(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("toolbars"));
-}
-
-void ScreenFrame::OnCaptureSelectionBar(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("selectionbar"));
-}
-
-void ScreenFrame::OnCaptureTools(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("tools"));
-}
-
-void ScreenFrame::OnCaptureTransport(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("transport"));
-}
-
-void ScreenFrame::OnCaptureMixer(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("mixer"));
-}
-
-void ScreenFrame::OnCaptureMeter(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("meter"));
-}
-
-void ScreenFrame::OnCaptureEdit(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("edit"));
-}
-
-void ScreenFrame::OnCaptureDevice(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("device"));
-}
-
-void ScreenFrame::OnCaptureTranscription(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("transcription"));
-}
-
-void ScreenFrame::OnCaptureTrackPanel(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("trackpanel"));
-}
-
-void ScreenFrame::OnCaptureRuler(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("ruler"));
-}
-
-void ScreenFrame::OnCaptureTracks(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("tracks"));
-}
-
-void ScreenFrame::OnCaptureFirstTrack(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("firsttrack"));
-}
-
-void ScreenFrame::OnCaptureSecondTrack(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("secondtrack"));
+   DoCapture(Names[i]);
 }
 
 void ScreenFrame::TimeZoom(double seconds)
 {
    int width, height;
-   mContext.proj->GetClientSize(&width, &height);
-   mContext.proj->mViewInfo.zoom = (0.75 * width) / seconds;
-   mContext.proj->RedrawProject();
+   mContext.GetProject()->GetClientSize(&width, &height);
+   mContext.GetProject()->mViewInfo.SetZoom((0.75 * width) / seconds);
+   mContext.GetProject()->RedrawProject();
 }
 
 void ScreenFrame::OnOneSec(wxCommandEvent & WXUNUSED(event))
@@ -692,7 +662,7 @@ void ScreenFrame::OnOneHour(wxCommandEvent & WXUNUSED(event))
 
 void ScreenFrame::SizeTracks(int h)
 {
-   TrackListIterator iter(mContext.proj->GetTracks());
+   TrackListIterator iter(mContext.GetProject()->GetTracks());
    for (Track * t = iter.First(); t; t = iter.Next()) {
       if (t->GetKind() == Track::Wave) {
          if (t->GetLink()) {
@@ -703,18 +673,18 @@ void ScreenFrame::SizeTracks(int h)
          }
       }
    }
-   mContext.proj->RedrawProject();
+   mContext.GetProject()->RedrawProject();
 }
 
 void ScreenFrame::OnShortTracks(wxCommandEvent & WXUNUSED(event))
 {
-   TrackListIterator iter(mContext.proj->GetTracks());
+   TrackListIterator iter(mContext.GetProject()->GetTracks());
    for (Track * t = iter.First(); t; t = iter.Next()) {
       if (t->GetKind() == Track::Wave) {
          t->SetHeight(t->GetMinimizedHeight());
       }
    }
-   mContext.proj->RedrawProject();
+   mContext.GetProject()->RedrawProject();
 }
 
 void ScreenFrame::OnMedTracks(wxCommandEvent & WXUNUSED(event))
