@@ -36,7 +36,7 @@ default is to disable caching.
   memory, so they are never read from disk in the current session.
 
 * Write-caching: If caching is enabled and the parameter allowDeferredWrite
-  is enabled at the block file constructor, new block files are held in memory
+  is enabled at the block file constructor, NEW block files are held in memory
   and written to disk only when WriteCacheToDisk() is called. This is used
   during recording to prevent disk access. After recording, WriteCacheToDisk()
   will be called on all block files and they will be written to disk. During
@@ -57,19 +57,23 @@ to get its definition, rather than rolling our own.
 
 *//*******************************************************************/
 
+#include "../Audacity.h"
+#include "SimpleBlockFile.h"
+
 #include <wx/wx.h>
 #include <wx/filefn.h>
 #include <wx/ffile.h>
 #include <wx/utils.h>
 #include <wx/log.h>
 
+#include "../FileException.h"
 #include "../Prefs.h"
 
-#include "SimpleBlockFile.h"
 #include "../FileFormats.h"
 
 #include "sndfile.h"
 #include "../Internat.h"
+#include "../MemoryX.h"
 
 
 static wxUint32 SwapUintEndianess(wxUint32 in)
@@ -94,13 +98,18 @@ static wxUint32 SwapUintEndianess(wxUint32 in)
 /// @param sampleLen    The number of samples to be written to this block.
 /// @param format       The format of the given samples.
 /// @param allowDeferredWrite    Allow deferred write-caching
-SimpleBlockFile::SimpleBlockFile(wxFileName baseFileName,
-                                 samplePtr sampleData, sampleCount sampleLen,
+SimpleBlockFile::SimpleBlockFile(wxFileNameWrapper &&baseFileName,
+                                 samplePtr sampleData, size_t sampleLen,
                                  sampleFormat format,
                                  bool allowDeferredWrite /* = false */,
                                  bool bypassCache /* = false */):
-   BlockFile(wxFileName(baseFileName.GetFullPath() + wxT(".au")), sampleLen)
+   BlockFile {
+      (baseFileName.SetExt(wxT("au")), std::move(baseFileName)),
+      sampleLen
+   }
 {
+   mFormat = format;
+
    mCache.active = false;
 
    bool useCache = GetCache() && (!bypassCache);
@@ -108,7 +117,9 @@ SimpleBlockFile::SimpleBlockFile(wxFileName baseFileName,
    if (!(allowDeferredWrite && useCache) && !bypassCache)
    {
       bool bSuccess = WriteSimpleBlockFile(sampleData, sampleLen, format, NULL);
-      wxASSERT(bSuccess); // TODO: Handle failure here by alert to user and undo partial op.
+      if (!bSuccess)
+         throw FileException{
+            FileException::Cause::Write, GetFileName().name };
    }
 
    if (useCache) {
@@ -116,14 +127,15 @@ SimpleBlockFile::SimpleBlockFile(wxFileName baseFileName,
       mCache.active = true;
       mCache.needWrite = true;
       mCache.format = format;
-      mCache.sampleData = new char[sampleLen * SAMPLE_SIZE(format)];
-      memcpy(mCache.sampleData,
-             sampleData, sampleLen * SAMPLE_SIZE(format));
+      const auto sampleDataSize = sampleLen * SAMPLE_SIZE(format);
+      mCache.sampleData.reinit(sampleDataSize);
+      memcpy(mCache.sampleData.get(), sampleData, sampleDataSize);
+      ArrayOf<char> cleanup;
       void* summaryData = BlockFile::CalcSummary(sampleData, sampleLen,
-                                                format);
-      mCache.summaryData = new char[mSummaryInfo.totalSummaryBytes];
-      memcpy(mCache.summaryData, summaryData,
-             (size_t)mSummaryInfo.totalSummaryBytes);
+                                                format, cleanup);
+      mCache.summaryData.reinit(mSummaryInfo.totalSummaryBytes);
+      memcpy(mCache.summaryData.get(), summaryData,
+             mSummaryInfo.totalSummaryBytes);
     }
 }
 
@@ -131,10 +143,13 @@ SimpleBlockFile::SimpleBlockFile(wxFileName baseFileName,
 /// existing block file.  This file must exist and be a valid block file.
 ///
 /// @param existingFile The disk file this SimpleBlockFile should use.
-SimpleBlockFile::SimpleBlockFile(wxFileName existingFile, sampleCount len,
+SimpleBlockFile::SimpleBlockFile(wxFileNameWrapper &&existingFile, size_t len,
                                  float min, float max, float rms):
-   BlockFile(existingFile, len)
+   BlockFile{ std::move(existingFile), len }
 {
+   // Set an invalid format to force GetSpaceUsage() to read it from the file.
+   mFormat = (sampleFormat) 0;
+
    mMin = min;
    mMax = max;
    mRMS = rms;
@@ -144,16 +159,11 @@ SimpleBlockFile::SimpleBlockFile(wxFileName existingFile, sampleCount len,
 
 SimpleBlockFile::~SimpleBlockFile()
 {
-   if (mCache.active)
-   {
-      delete[] mCache.sampleData;
-      delete[] (char *)mCache.summaryData;
-   }
 }
 
 bool SimpleBlockFile::WriteSimpleBlockFile(
     samplePtr sampleData,
-    sampleCount sampleLen,
+    size_t sampleLen,
     sampleFormat format,
     void* summaryData)
 {
@@ -200,14 +210,17 @@ bool SimpleBlockFile::WriteSimpleBlockFile(
    header.channels = 1;
 
    // Write the file
+   ArrayOf<char> cleanup;
    if (!summaryData)
-      summaryData = /*BlockFile::*/CalcSummary(sampleData, sampleLen, format); //mchinen:allowing virtual override of calc summary for ODDecodeBlockFile.
+      summaryData = /*BlockFile::*/CalcSummary(sampleData, sampleLen, format, cleanup);
+      //mchinen:allowing virtual override of calc summary for ODDecodeBlockFile.
+      // PRL: cleanup fixes a possible memory leak!
 
    size_t nBytesToWrite = sizeof(header);
    size_t nBytesWritten = file.Write(&header, nBytesToWrite);
    if (nBytesWritten != nBytesToWrite)
    {
-      wxLogDebug(wxT("Wrote %d bytes, expected %d."), nBytesWritten, nBytesToWrite);
+      wxLogDebug(wxT("Wrote %lld bytes, expected %lld."), (long long) nBytesWritten, (long long) nBytesToWrite);
       return false;
    }
 
@@ -215,7 +228,7 @@ bool SimpleBlockFile::WriteSimpleBlockFile(
    nBytesWritten = file.Write(summaryData, nBytesToWrite);
    if (nBytesWritten != nBytesToWrite)
    {
-      wxLogDebug(wxT("Wrote %d bytes, expected %d."), nBytesWritten, nBytesToWrite);
+      wxLogDebug(wxT("Wrote %lld bytes, expected %lld."), (long long) nBytesWritten, (long long) nBytesToWrite);
       return false;
    }
 
@@ -226,7 +239,7 @@ bool SimpleBlockFile::WriteSimpleBlockFile(
       // memory
       int *int24sampleData = (int*)sampleData;
 
-      for( int i = 0; i < sampleLen; i++ )
+      for( size_t i = 0; i < sampleLen; i++ )
       {
          nBytesToWrite = 3;
          nBytesWritten =
@@ -237,7 +250,7 @@ bool SimpleBlockFile::WriteSimpleBlockFile(
             #endif
          if (nBytesWritten != nBytesToWrite)
          {
-            wxLogDebug(wxT("Wrote %d bytes, expected %d."), nBytesWritten, nBytesToWrite);
+            wxLogDebug(wxT("Wrote %lld bytes, expected %lld."), (long long) nBytesWritten, (long long) nBytesToWrite);
             return false;
          }
       }
@@ -250,14 +263,16 @@ bool SimpleBlockFile::WriteSimpleBlockFile(
       nBytesWritten = file.Write(sampleData, nBytesToWrite);
       if (nBytesWritten != nBytesToWrite)
       {
-         wxLogDebug(wxT("Wrote %d bytes, expected %d."), nBytesWritten, nBytesToWrite);
+         wxLogDebug(wxT("Wrote %lld bytes, expected %lld."), (long long) nBytesWritten, (long long) nBytesToWrite);
          return false;
       }
    }
 
-    return true;
+   return true;
 }
 
+// This function should try to fill the cache, but just return without effect
+// (not throwing) if there is failure.
 void SimpleBlockFile::FillCache()
 {
    if (mCache.active)
@@ -303,18 +318,19 @@ void SimpleBlockFile::FillCache()
    file.Close();
 
    // Read samples into cache
-   mCache.sampleData = new char[mLen * SAMPLE_SIZE(mCache.format)];
-   if (ReadData(mCache.sampleData, mCache.format, 0, mLen) != mLen)
+   mCache.sampleData.reinit(mLen * SAMPLE_SIZE(mCache.format));
+   if (ReadData(mCache.sampleData.get(), mCache.format, 0, mLen,
+                // no exceptions!
+                false) != mLen)
    {
       // Could not read all samples
-      delete mCache.sampleData;
+      mCache.sampleData.reset();
       return;
    }
 
    // Read summary data into cache
-   mCache.summaryData = new char[mSummaryInfo.totalSummaryBytes];
-   if (!ReadSummary(mCache.summaryData))
-      memset(mCache.summaryData, 0, mSummaryInfo.totalSummaryBytes);
+   // Fills with zeroes in case of failure:
+   ReadSummary(mCache.summaryData);
 
    // Cache is active but already on disk
    mCache.active = true;
@@ -327,45 +343,45 @@ void SimpleBlockFile::FillCache()
 ///
 /// @param *data The buffer to write the data to.  It must be at least
 /// mSummaryinfo.totalSummaryBytes long.
-bool SimpleBlockFile::ReadSummary(void *data)
+bool SimpleBlockFile::ReadSummary(ArrayOf<char> &data)
 {
-   if (mCache.active)
-   {
+   data.reinit( mSummaryInfo.totalSummaryBytes );
+   if (mCache.active) {
       //wxLogDebug("SimpleBlockFile::ReadSummary(): Summary is already in cache.");
-      memcpy(data, mCache.summaryData, (size_t)mSummaryInfo.totalSummaryBytes);
+      memcpy(data.get(), mCache.summaryData.get(), mSummaryInfo.totalSummaryBytes);
       return true;
-   } else
+   }
+   else
    {
       //wxLogDebug("SimpleBlockFile::ReadSummary(): Reading summary from disk.");
 
       wxFFile file(mFileName.GetFullPath(), wxT("rb"));
 
-      wxLogNull *silence=0;
-      if(mSilentLog)silence= new wxLogNull();
-
-      if(!file.IsOpened() ){
-
-         memset(data,0,(size_t)mSummaryInfo.totalSummaryBytes);
-
-         if(silence) delete silence;
-         mSilentLog=TRUE;
-
-         return true;
-
+      {
+         Maybe<wxLogNull> silence{};
+         if (mSilentLog)
+            silence.create();
+         // FIXME: TRAP_ERR no report to user of absent summary files?
+         // filled with zero instead.
+         if (!file.IsOpened()){
+            memset(data.get(), 0, mSummaryInfo.totalSummaryBytes);
+            mSilentLog = TRUE;
+            return false;
+         }
       }
-
-      if(silence) delete silence;
-      mSilentLog=FALSE;
+      mSilentLog = FALSE;
 
       // The offset is just past the au header
-      if( !file.Seek(sizeof(auHeader)) )
+      if( !file.Seek(sizeof(auHeader)) ||
+          file.Read(data.get(), mSummaryInfo.totalSummaryBytes) !=
+             mSummaryInfo.totalSummaryBytes ) {
+         memset(data.get(), 0, mSummaryInfo.totalSummaryBytes);
          return false;
+      }
 
-      int read = (int)file.Read(data, (size_t)mSummaryInfo.totalSummaryBytes);
+      FixSummary(data.get());
 
-      FixSummary(data);
-
-      return (read == mSummaryInfo.totalSummaryBytes);
+      return true;
    }
 }
 
@@ -376,95 +392,35 @@ bool SimpleBlockFile::ReadSummary(void *data)
 /// @param format The format the data will be stored in
 /// @param start  The offset in this block file
 /// @param len    The number of samples to read
-int SimpleBlockFile::ReadData(samplePtr data, sampleFormat format,
-                        sampleCount start, sampleCount len)
+size_t SimpleBlockFile::ReadData(samplePtr data, sampleFormat format,
+                        size_t start, size_t len, bool mayThrow) const
 {
    if (mCache.active)
    {
       //wxLogDebug("SimpleBlockFile::ReadData(): Data are already in cache.");
 
-      if (len > mLen - start)
-         len = mLen - start;
+      auto framesRead = std::min(len, std::max(start, mLen) - start);
       CopySamples(
-         (samplePtr)(((char*)mCache.sampleData) +
+         (samplePtr)(mCache.sampleData.get() +
             start * SAMPLE_SIZE(mCache.format)),
-         mCache.format, data, format, len);
-      return len;
-   } else
-   {
-      //wxLogDebug("SimpleBlockFile::ReadData(): Reading data from disk.");
+         mCache.format, data, format, framesRead);
 
-      SF_INFO info;
-      wxLogNull *silence=0;
-      if(mSilentLog)silence= new wxLogNull();
-
-      memset(&info, 0, sizeof(info));
-
-      wxFile f;   // will be closed when it goes out of scope
-      SNDFILE *sf = NULL;
-
-      if (f.Open(mFileName.GetFullPath())) {
-         // Even though there is an sf_open() that takes a filename, use the one that
-         // takes a file descriptor since wxWidgets can open a file with a Unicode name and
-         // libsndfile can't (under Windows).
-         sf = sf_open_fd(f.fd(), SFM_READ, &info, FALSE);
+      if ( framesRead < len ) {
+         if (mayThrow)
+            // Not the best exception class?
+            throw FileException{ FileException::Cause::Read, mFileName };
+         ClearSamples(data, format, framesRead, len - framesRead);
       }
 
-      if (!sf) {
-
-         memset(data,0,SAMPLE_SIZE(format)*len);
-
-         if(silence) delete silence;
-         mSilentLog=TRUE;
-
-         return len;
-      }
-      if(silence) delete silence;
-      mSilentLog=FALSE;
-
-      sf_seek(sf, start, SEEK_SET);
-      samplePtr buffer = NewSamples(len, floatSample);
-
-      int framesRead = 0;
-
-      // If both the src and dest formats are integer formats,
-      // read integers from the file (otherwise we would be
-      // converting to float and back, which is unneccesary)
-      if (format == int16Sample &&
-          sf_subtype_is_integer(info.format)) {
-         framesRead = sf_readf_short(sf, (short *)data, len);
-      }
-      else
-      if (format == int24Sample &&
-          sf_subtype_is_integer(info.format))
-      {
-         framesRead = sf_readf_int(sf, (int *)data, len);
-
-         // libsndfile gave us the 3 byte sample in the 3 most
-         // significant bytes -- we want it in the 3 least
-         // significant bytes.
-         int *intPtr = (int *)data;
-         for( int i = 0; i < framesRead; i++ )
-            intPtr[i] = intPtr[i] >> 8;
-      }
-      else {
-         // Otherwise, let libsndfile handle the conversion and
-         // scaling, and pass us normalized data as floats.  We can
-         // then convert to whatever format we want.
-         framesRead = sf_readf_float(sf, (float *)buffer, len);
-         CopySamples(buffer, floatSample,
-                     (samplePtr)data, format, framesRead);
-      }
-
-      DeleteSamples(buffer);
-
-      sf_close(sf);
-
       return framesRead;
    }
+   else
+      return CommonReadData( mayThrow,
+         mFileName, mSilentLog, nullptr, 0, 0, data, format, start, len);
 }
 
 void SimpleBlockFile::SaveXML(XMLWriter &xmlFile)
+// may throw
 {
    xmlFile.StartTag(wxT("simpleblockfile"));
 
@@ -481,11 +437,11 @@ void SimpleBlockFile::SaveXML(XMLWriter &xmlFile)
 // even if the result is flawed (e.g., refers to nonexistent file),
 // as testing will be done in DirManager::ProjectFSCK().
 /// static
-BlockFile *SimpleBlockFile::BuildFromXML(DirManager &dm, const wxChar **attrs)
+BlockFilePtr SimpleBlockFile::BuildFromXML(DirManager &dm, const wxChar **attrs)
 {
-   wxFileName fileName;
+   wxFileNameWrapper fileName;
    float min = 0.0f, max = 0.0f, rms = 0.0f;
-   sampleCount len = 0;
+   size_t len = 0;
    double dblValue;
    long nValue;
 
@@ -521,36 +477,81 @@ BlockFile *SimpleBlockFile::BuildFromXML(DirManager &dm, const wxChar **attrs)
       }
    }
 
-   return new SimpleBlockFile(fileName, len, min, max, rms);
+   return make_blockfile<SimpleBlockFile>
+      (std::move(fileName), len, min, max, rms);
 }
 
 /// Create a copy of this BlockFile, but using a different disk file.
 ///
-/// @param newFileName The name of the new file to use.
-BlockFile *SimpleBlockFile::Copy(wxFileName newFileName)
+/// @param newFileName The name of the NEW file to use.
+BlockFilePtr SimpleBlockFile::Copy(wxFileNameWrapper &&newFileName)
 {
-   BlockFile *newBlockFile = new SimpleBlockFile(newFileName, mLen,
-                                                 mMin, mMax, mRMS);
+   auto newBlockFile = make_blockfile<SimpleBlockFile>
+      (std::move(newFileName), mLen, mMin, mMax, mRMS);
 
    return newBlockFile;
 }
 
-wxLongLong SimpleBlockFile::GetSpaceUsage()
+auto SimpleBlockFile::GetSpaceUsage() const -> DiskByteCount
 {
    if (mCache.active && mCache.needWrite)
    {
       // We don't know space usage yet
       return 0;
-   } else
-   {
-      wxFFile dataFile(mFileName.GetFullPath());
-      return dataFile.Length();
    }
+
+   // Don't know the format, so it must be read from the file
+   if (mFormat == (sampleFormat) 0)
+   {
+      // Check sample format
+      wxFFile file(mFileName.GetFullPath(), wxT("rb"));
+      if (!file.IsOpened())
+      {
+         // Don't read into cache if file not available
+         return 0;
+      }
+   
+      auHeader header;
+   
+      if (file.Read(&header, sizeof(header)) != sizeof(header))
+      {
+         // Corrupt file
+         return 0;
+      }
+   
+      wxUint32 encoding;
+   
+      if (header.magic == 0x2e736e64)
+         encoding = header.encoding; // correct endianness
+      else
+         encoding = SwapUintEndianess(header.encoding);
+   
+      switch (encoding)
+      {
+      case AU_SAMPLE_FORMAT_16:
+         mFormat = int16Sample;
+         break;
+      case AU_SAMPLE_FORMAT_24:
+         mFormat = int24Sample;
+         break;
+      default:
+         // floatSample is a safe default (we will never loose data)
+         mFormat = floatSample;
+         break;
+      }
+   
+      file.Close();
+   }
+
+   return {
+          sizeof(auHeader) +
+          mSummaryInfo.totalSummaryBytes +
+          (GetLength() * SAMPLE_SIZE_DISK(mFormat))
+   };
 }
 
 void SimpleBlockFile::Recover(){
    wxFFile file(mFileName.GetFullPath(), wxT("wb"));
-   int i;
 
    if( !file.IsOpened() ){
       // Can't do anything else.
@@ -568,10 +569,11 @@ void SimpleBlockFile::Recover(){
    header.channels = 1;
    file.Write(&header, sizeof(header));
 
-   for(i=0;i<mSummaryInfo.totalSummaryBytes;i++)
+   for(decltype(mSummaryInfo.totalSummaryBytes) i = 0;
+       i < mSummaryInfo.totalSummaryBytes; i++)
       file.Write(wxT("\0"),1);
 
-   for(i=0;i<mLen*2;i++)
+   for(decltype(mLen) i = 0; i < mLen * 2; i++)
       file.Write(wxT("\0"),1);
 
 }
@@ -581,8 +583,8 @@ void SimpleBlockFile::WriteCacheToDisk()
    if (!GetNeedWriteCacheToDisk())
       return;
 
-   if (WriteSimpleBlockFile(mCache.sampleData, mLen, mCache.format,
-                            mCache.summaryData))
+   if (WriteSimpleBlockFile(mCache.sampleData.get(), mLen, mCache.format,
+                            mCache.summaryData.get()))
       mCache.needWrite = false;
 }
 
