@@ -39,14 +39,17 @@
 *//*******************************************************************/
 
 #include "../Audacity.h"
+#include "../Experimental.h"
+
+#if !defined(EXPERIMENTAL_NOISE_REDUCTION)
 
 #include "NoiseRemoval.h"
 
-#include "../Envelope.h"
 #include "../WaveTrack.h"
 #include "../Prefs.h"
 #include "../Project.h"
 #include "../FileNames.h"
+#include "../ShuttleGui.h"
 
 #include <math.h>
 
@@ -65,7 +68,6 @@
 #include <wx/dcmemory.h>
 #include <wx/image.h>
 #include <wx/intl.h>
-#include <wx/msgdlg.h>
 #include <wx/sizer.h>
 #include <wx/statbox.h>
 #include <wx/stattext.h>
@@ -73,7 +75,6 @@
 #include <wx/valtext.h>
 
 
-#include "../AudacityApp.h"
 #include "../PlatformCompatibility.h"
 
 EffectNoiseRemoval::EffectNoiseRemoval()
@@ -98,16 +99,41 @@ EffectNoiseRemoval::EffectNoiseRemoval()
    mHasProfile = false;
    mDoProfile = true;
 
-   mNoiseThreshold = new float[mSpectrumSize];
+   mNoiseThreshold.reinit(mSpectrumSize);
 
    Init();
 }
 
 EffectNoiseRemoval::~EffectNoiseRemoval()
 {
-   delete [] mNoiseThreshold;
 }
 
+// IdentInterface implementation
+
+wxString EffectNoiseRemoval::GetSymbol()
+{
+   return XO("Noise Removal");
+}
+
+wxString EffectNoiseRemoval::GetDescription()
+{
+   return _("Removes constant background noise such as fans, tape noise, or hums");
+}
+
+// EffectIdentInterface implementation
+
+EffectType EffectNoiseRemoval::GetType()
+{
+   return EffectTypeProcess;
+}
+
+bool EffectNoiseRemoval::SupportsAutomation()
+{
+   return false;
+}
+
+// Effect implementation
+
 #define MAX_NOISE_LEVEL  30
 bool EffectNoiseRemoval::Init()
 {
@@ -121,13 +147,12 @@ bool EffectNoiseRemoval::Init()
 
 bool EffectNoiseRemoval::CheckWhetherSkipEffect()
 {
-   bool rc = (mLevel == 0);
-   return rc;
+   return (mLevel == 0);
 }
 
-bool EffectNoiseRemoval::PromptUser()
+bool EffectNoiseRemoval::PromptUser(wxWindow *parent)
 {
-   NoiseRemovalDialog dlog(this, mParent);
+   NoiseRemovalDialog dlog(this, parent);
    dlog.mSensitivity = mSensitivity;
    dlog.mGain = -mNoiseGain;
    dlog.mFreq = mFreqSmoothingHz;
@@ -174,14 +199,6 @@ bool EffectNoiseRemoval::PromptUser()
    return gPrefs->Flush();
 }
 
-bool EffectNoiseRemoval::TransferParameters( Shuttle & WXUNUSED(shuttle) )
-{
-   //shuttle.TransferDouble(wxT("Gain"), mNoiseGain, 0.0);
-   //shuttle.TransferDouble(wxT("Freq"), mFreqSmoothingHz, 0.0);
-   //shuttle.TransferDouble(wxT("Time"), mAttackDecayTime, 0.0);
-   return true;
-}
-
 bool EffectNoiseRemoval::Process()
 {
    Initialize();
@@ -191,7 +208,7 @@ bool EffectNoiseRemoval::Process()
    this->CopyInputTracks(); // Set up mOutputTracks.
    bool bGoodResult = true;
 
-   SelectedTrackListOfKindIterator iter(Track::Wave, mOutputTracks);
+   SelectedTrackListOfKindIterator iter(Track::Wave, mOutputTracks.get());
    WaveTrack *track = (WaveTrack *) iter.First();
    int count = 0;
    while (track) {
@@ -201,12 +218,11 @@ bool EffectNoiseRemoval::Process()
       double t1 = mT1 > trackEnd? trackEnd: mT1;
 
       if (t1 > t0) {
-         sampleCount start = track->TimeToLongSamples(t0);
-         sampleCount end = track->TimeToLongSamples(t1);
-         sampleCount len = (sampleCount)(end - start);
+         auto start = track->TimeToLongSamples(t0);
+         auto end = track->TimeToLongSamples(t1);
+         auto len = end - start;
 
          if (!ProcessOne(count, track, start, len)) {
-            Cleanup();
             bGoodResult = false;
             break;
          }
@@ -220,18 +236,16 @@ bool EffectNoiseRemoval::Process()
       mDoProfile = false;
    }
 
-   if (bGoodResult)
-      Cleanup();
    this->ReplaceProcessedTracks(bGoodResult);
    return bGoodResult;
 }
 
 void EffectNoiseRemoval::ApplyFreqSmoothing(float *spec)
 {
-   float *tmp = new float[mSpectrumSize];
-   int i, j, j0, j1;
+   Floats tmp{ mSpectrumSize };
+   int j, j0, j1;
 
-   for(i = 0; i < mSpectrumSize; i++) {
+   for(int i = 0; i < mSpectrumSize; i++) {
       j0 = wxMax(0, i - mFreqSmoothingBins);
       j1 = wxMin(mSpectrumSize-1, i + mFreqSmoothingBins);
       tmp[i] = 0.0;
@@ -241,24 +255,18 @@ void EffectNoiseRemoval::ApplyFreqSmoothing(float *spec)
       tmp[i] /= (j1 - j0 + 1);
    }
 
-   for(i = 0; i < mSpectrumSize; i++)
+   for(size_t i = 0; i < mSpectrumSize; i++)
       spec[i] = tmp[i];
-
-   delete[] tmp;
 }
 
 void EffectNoiseRemoval::Initialize()
 {
-   int i;
-
    mSampleRate = mProjectRate;
    mFreqSmoothingBins = (int)(mFreqSmoothingHz * mWindowSize / mSampleRate);
    mAttackDecayBlocks = 1 +
       (int)(mAttackDecayTime * mSampleRate / (mWindowSize / 2));
-   // Applies to amplitudes, divide by 20:
-   mNoiseAttenFactor = pow(10.0, mNoiseGain/20.0);
-   // Applies to gain factors which apply to amplitudes, divide by 20:
-   mOneBlockAttackDecay = pow(10.0, (mNoiseGain / (20.0 * mAttackDecayBlocks)));
+   mNoiseAttenFactor = DB_TO_LINEAR(mNoiseGain);
+   mOneBlockAttackDecay = DB_TO_LINEAR(mNoiseGain / mAttackDecayBlocks);
    // Applies to power, divide by 10:
    mSensitivityFactor = pow(10.0, mSensitivity/10.0);
    mMinSignalBlocks =
@@ -270,68 +278,54 @@ void EffectNoiseRemoval::Initialize()
    if (mHistoryLen < mMinSignalBlocks)
       mHistoryLen = mMinSignalBlocks;
 
-   mSpectrums = new float*[mHistoryLen];
-   mGains = new float*[mHistoryLen];
-   mRealFFTs = new float*[mHistoryLen];
-   mImagFFTs = new float*[mHistoryLen];
-   for(i = 0; i < mHistoryLen; i++) {
-      mSpectrums[i] = new float[mSpectrumSize];
-      mGains[i] = new float[mSpectrumSize];
-      mRealFFTs[i] = new float[mSpectrumSize];
-      mImagFFTs[i] = new float[mSpectrumSize];
-   }
+   mSpectrums.reinit(mHistoryLen, mSpectrumSize);
+   mGains.reinit(mHistoryLen, mSpectrumSize);
+   mRealFFTs.reinit(mHistoryLen, mSpectrumSize);
+   mImagFFTs.reinit(mHistoryLen, mSpectrumSize);
 
    // Initialize the FFT
-   hFFT = InitializeFFT(mWindowSize);
+   hFFT = GetFFT(mWindowSize);
 
-   mFFTBuffer = new float[mWindowSize];
-   mInWaveBuffer = new float[mWindowSize];
-   mWindow = new float[mWindowSize];
-   mOutOverlapBuffer = new float[mWindowSize];
+   mFFTBuffer.reinit(mWindowSize);
+   mInWaveBuffer.reinit(mWindowSize);
+   mWindow.reinit(mWindowSize);
+   mOutOverlapBuffer.reinit(mWindowSize);
 
    // Create a Hanning window function
-   for(i=0; i<mWindowSize; i++)
+   for(size_t i=0; i<mWindowSize; i++)
       mWindow[i] = 0.5 - 0.5 * cos((2.0*M_PI*i) / mWindowSize);
 
    if (mDoProfile) {
-      for (i = 0; i < mSpectrumSize; i++)
+      for (size_t i = 0; i < mSpectrumSize; i++)
          mNoiseThreshold[i] = float(0);
    }
 }
 
-void EffectNoiseRemoval::Cleanup()
+void EffectNoiseRemoval::End()
 {
-   int i;
-
-   EndFFT(hFFT);
+   hFFT.reset();
 
    if (mDoProfile) {
-      ApplyFreqSmoothing(mNoiseThreshold);
+      ApplyFreqSmoothing(mNoiseThreshold.get());
    }
 
-   for(i = 0; i < mHistoryLen; i++) {
-      delete[] mSpectrums[i];
-      delete[] mGains[i];
-      delete[] mRealFFTs[i];
-      delete[] mImagFFTs[i];
-   }
-   delete[] mSpectrums;
-   delete[] mGains;
-   delete[] mRealFFTs;
-   delete[] mImagFFTs;
+   mSpectrums.reset();
+   mGains.reset();
+   mRealFFTs.reset();
+   mImagFFTs.reset();
 
-   delete[] mFFTBuffer;
-   delete[] mInWaveBuffer;
-   delete[] mWindow;
-   delete[] mOutOverlapBuffer;
+   mFFTBuffer.reset();
+   mInWaveBuffer.reset();
+   mWindow.reset();
+   mOutOverlapBuffer.reset();
+
+   mOutputTrack.reset();
 }
 
 void EffectNoiseRemoval::StartNewTrack()
 {
-   int i, j;
-
-   for(i = 0; i < mHistoryLen; i++) {
-      for(j = 0; j < mSpectrumSize; j++) {
+   for(size_t i = 0; i < mHistoryLen; i++) {
+      for(size_t j = 0; j < mSpectrumSize; j++) {
          mSpectrums[i][j] = 0;
          mGains[i][j] = mNoiseAttenFactor;
          mRealFFTs[i][j] = 0.0;
@@ -339,27 +333,25 @@ void EffectNoiseRemoval::StartNewTrack()
       }
    }
 
-   for(j = 0; j < mWindowSize; j++)
+   for(size_t j = 0; j < mWindowSize; j++)
       mOutOverlapBuffer[j] = 0.0;
 
    mInputPos = 0;
    mInSampleCount = 0;
-   mOutSampleCount = -(mWindowSize / 2) * (mHistoryLen - 1);
+   mOutSampleCount = -(int)((mWindowSize / 2) * (mHistoryLen - 1));
 }
 
-void EffectNoiseRemoval::ProcessSamples(sampleCount len, float *buffer)
+void EffectNoiseRemoval::ProcessSamples(size_t len, float *buffer)
 {
-   int i;
-
    while(len && mOutSampleCount < mInSampleCount) {
-      int avail = wxMin(len, mWindowSize - mInputPos);
-      for(i = 0; i < avail; i++)
+      size_t avail = wxMin(len, mWindowSize - mInputPos);
+      for(size_t i = 0; i < avail; i++)
          mInWaveBuffer[mInputPos + i] = buffer[i];
       buffer += avail;
       len -= avail;
       mInputPos += avail;
 
-      if (mInputPos == mWindowSize) {
+      if (mInputPos == int(mWindowSize)) {
          FillFirstHistoryWindow();
          if (mDoProfile)
             GetProfile();
@@ -368,7 +360,7 @@ void EffectNoiseRemoval::ProcessSamples(sampleCount len, float *buffer)
          RotateHistoryWindows();
 
          // Rotate halfway for overlap-add
-         for(i = 0; i < mWindowSize / 2; i++) {
+         for(size_t i = 0; i < mWindowSize / 2; i++) {
             mInWaveBuffer[i] = mInWaveBuffer[i + mWindowSize / 2];
          }
          mInputPos = mWindowSize / 2;
@@ -378,12 +370,10 @@ void EffectNoiseRemoval::ProcessSamples(sampleCount len, float *buffer)
 
 void EffectNoiseRemoval::FillFirstHistoryWindow()
 {
-   int i;
-
-   for(i=0; i < mWindowSize; i++)
+   for(size_t i = 0; i < mWindowSize; i++)
       mFFTBuffer[i] = mInWaveBuffer[i];
-   RealFFTf(mFFTBuffer, hFFT);
-   for(i = 1; i < (mSpectrumSize-1); i++) {
+   RealFFTf(mFFTBuffer.get(), hFFT.get());
+   for(size_t i = 1; i + 1 < mSpectrumSize; i++) {
       mRealFFTs[0][i] = mFFTBuffer[hFFT->BitReversed[i]  ];
       mImagFFTs[0][i] = mFFTBuffer[hFFT->BitReversed[i]+1];
       mSpectrums[0][i] = mRealFFTs[0][i]*mRealFFTs[0][i] + mImagFFTs[0][i]*mImagFFTs[0][i];
@@ -396,30 +386,24 @@ void EffectNoiseRemoval::FillFirstHistoryWindow()
    mGains[0][mSpectrumSize-1] = mNoiseAttenFactor;
 }
 
+namespace {
+   inline void Rotate(ArraysOf<float> &arrays, size_t historyLen)
+   {
+      Floats temp = std::move( arrays[ historyLen - 1 ] );
+
+      for ( size_t nn = historyLen - 1; nn--; )
+         arrays[ nn + 1 ] = std::move( arrays[ nn ] );
+      arrays[0] = std::move( temp );
+   }
+}
+
 void EffectNoiseRemoval::RotateHistoryWindows()
 {
-   int last = mHistoryLen - 1;
-   int i;
-
    // Remember the last window so we can reuse it
-   float *lastSpectrum = mSpectrums[last];
-   float *lastGain = mGains[last];
-   float *lastRealFFT = mRealFFTs[last];
-   float *lastImagFFT = mImagFFTs[last];
-
-   // Rotate each window forward
-   for(i = last; i >= 1; i--) {
-      mSpectrums[i] = mSpectrums[i-1];
-      mGains[i] = mGains[i-1];
-      mRealFFTs[i] = mRealFFTs[i-1];
-      mImagFFTs[i] = mImagFFTs[i-1];
-   }
-
-   // Reuse the last buffers as the new first window
-   mSpectrums[0] = lastSpectrum;
-   mGains[0] = lastGain;
-   mRealFFTs[0] = lastRealFFT;
-   mImagFFTs[0] = lastImagFFT;
+   Rotate(mSpectrums, mHistoryLen);
+   Rotate(mGains, mHistoryLen);
+   Rotate(mRealFFTs, mHistoryLen);
+   Rotate(mImagFFTs, mHistoryLen);
 }
 
 void EffectNoiseRemoval::FinishTrack()
@@ -428,18 +412,15 @@ void EffectNoiseRemoval::FinishTrack()
    // windows until we've output exactly as many samples as
    // were input.
    // Well, not exactly, but not more than mWindowSize/2 extra samples at the end.
-   // We'll delete them later in ProcessOne.
+   // We'll DELETE them later in ProcessOne.
 
-   float *empty = new float[mWindowSize / 2];
-   int i;
-   for(i = 0; i < mWindowSize / 2; i++)
+   Floats empty{ mWindowSize / 2 };
+   for(size_t i = 0; i < mWindowSize / 2; i++)
       empty[i] = 0.0;
 
    while (mOutSampleCount < mInSampleCount) {
-      ProcessSamples(mWindowSize / 2, empty);
+      ProcessSamples(mWindowSize / 2, empty.get());
    }
-
-   delete [] empty;
 }
 
 void EffectNoiseRemoval::GetProfile()
@@ -450,9 +431,9 @@ void EffectNoiseRemoval::GetProfile()
 
    int start = mHistoryLen - mMinSignalBlocks;
    int finish = mHistoryLen;
-   int i, j;
+   int i;
 
-   for (j = 0; j < mSpectrumSize; j++) {
+   for (size_t j = 0; j < mSpectrumSize; j++) {
       float min = mSpectrums[start][j];
       for (i = start+1; i < finish; i++) {
          if (mSpectrums[i][j] < min)
@@ -467,15 +448,14 @@ void EffectNoiseRemoval::GetProfile()
 
 void EffectNoiseRemoval::RemoveNoise()
 {
-   int center = mHistoryLen / 2;
-   int start = center - mMinSignalBlocks/2;
-   int finish = start + mMinSignalBlocks;
-   int i, j;
+   size_t center = mHistoryLen / 2;
+   size_t start = center - mMinSignalBlocks/2;
+   size_t finish = start + mMinSignalBlocks;
 
    // Raise the gain for elements in the center of the sliding history
-   for (j = 0; j < mSpectrumSize; j++) {
+   for (size_t j = 0; j < mSpectrumSize; j++) {
       float min = mSpectrums[start][j];
-      for (i = start+1; i < finish; i++) {
+      for (size_t i = start+1; i < finish; i++) {
          if (mSpectrums[i][j] < min)
             min = mSpectrums[i][j];
       }
@@ -490,14 +470,14 @@ void EffectNoiseRemoval::RemoveNoise()
    // Decay the gain in both directions;
    // note that mOneBlockAttackDecay is less than 1.0
    // of linear attenuation per block
-   for (j = 0; j < mSpectrumSize; j++) {
-      for (i = center + 1; i < mHistoryLen; i++) {
+   for (size_t j = 0; j < mSpectrumSize; j++) {
+      for (size_t i = center + 1; i < mHistoryLen; i++) {
          if (mGains[i][j] < mGains[i - 1][j] * mOneBlockAttackDecay)
             mGains[i][j] = mGains[i - 1][j] * mOneBlockAttackDecay;
          if (mGains[i][j] < mNoiseAttenFactor)
             mGains[i][j] = mNoiseAttenFactor;
       }
-      for (i = center - 1; i >= 0; i--) {
+      for (size_t i = center; i--;) {
          if (mGains[i][j] < mGains[i + 1][j] * mOneBlockAttackDecay)
             mGains[i][j] = mGains[i + 1][j] * mOneBlockAttackDecay;
          if (mGains[i][j] < mNoiseAttenFactor)
@@ -509,10 +489,10 @@ void EffectNoiseRemoval::RemoveNoise()
    // Apply frequency smoothing to output gain
    int out = mHistoryLen - 1;  // end of the queue
 
-   ApplyFreqSmoothing(mGains[out]);
+   ApplyFreqSmoothing(mGains[out].get());
 
    // Apply gain to FFT
-   for (j = 0; j < (mSpectrumSize-1); j++) {
+   for (size_t j = 0; j < (mSpectrumSize-1); j++) {
       mFFTBuffer[j*2  ] = mRealFFTs[out][j] * mGains[out][j];
       mFFTBuffer[j*2+1] = mImagFFTs[out][j] * mGains[out][j];
    }
@@ -520,10 +500,10 @@ void EffectNoiseRemoval::RemoveNoise()
    mFFTBuffer[1] = mRealFFTs[out][mSpectrumSize-1] * mGains[out][mSpectrumSize-1];
 
    // Invert the FFT into the output buffer
-   InverseRealFFTf(mFFTBuffer, hFFT);
+   InverseRealFFTf(mFFTBuffer.get(), hFFT.get());
 
    // Overlap-add
-   for(j = 0; j < (mSpectrumSize-1); j++) {
+   for(size_t j = 0; j < (mSpectrumSize-1); j++) {
       mOutOverlapBuffer[j*2  ] += mFFTBuffer[hFFT->BitReversed[j]  ] * mWindow[j*2  ];
       mOutOverlapBuffer[j*2+1] += mFFTBuffer[hFFT->BitReversed[j]+1] * mWindow[j*2+1];
    }
@@ -531,11 +511,11 @@ void EffectNoiseRemoval::RemoveNoise()
    // Output the first half of the overlap buffer, they're done -
    // and then shift the next half over.
    if (mOutSampleCount >= 0) {   // ...but not if it's the first half-window
-      mOutputTrack->Append((samplePtr)mOutOverlapBuffer, floatSample,
+      mOutputTrack->Append((samplePtr)mOutOverlapBuffer.get(), floatSample,
                            mWindowSize / 2);
    }
    mOutSampleCount += mWindowSize / 2;
-   for(j = 0; j < mWindowSize / 2; j++) {
+   for(size_t j = 0; j < mWindowSize / 2; j++) {
       mOutOverlapBuffer[j] = mOutOverlapBuffer[j + (mWindowSize / 2)];
       mOutOverlapBuffer[j + (mWindowSize / 2)] = 0.0;
    }
@@ -553,37 +533,35 @@ bool EffectNoiseRemoval::ProcessOne(int count, WaveTrack * track,
       mOutputTrack = mFactory->NewWaveTrack(track->GetSampleFormat(),
                                             track->GetRate());
 
-   sampleCount bufferSize = track->GetMaxBlockSize();
-   float *buffer = new float[bufferSize];
+   auto bufferSize = track->GetMaxBlockSize();
+   Floats buffer{ bufferSize };
 
    bool bLoopSuccess = true;
-   sampleCount blockSize;
-   sampleCount samplePos = start;
+   auto samplePos = start;
    while (samplePos < start + len) {
       //Get a blockSize of samples (smaller than the size of the buffer)
-      blockSize = track->GetBestBlockSize(samplePos);
-
       //Adjust the block size if it is the final block in the track
-      if (samplePos + blockSize > start + len)
-         blockSize = start + len - samplePos;
+      const auto blockSize = limitSampleBufferSize(
+         track->GetBestBlockSize(samplePos),
+         start + len - samplePos
+      );
 
       //Get the samples from the track and put them in the buffer
-      track->Get((samplePtr)buffer, floatSample, samplePos, blockSize);
+      track->Get((samplePtr)buffer.get(), floatSample, samplePos, blockSize);
 
       mInSampleCount += blockSize;
-      ProcessSamples(blockSize, buffer);
+      ProcessSamples(blockSize, buffer.get());
 
       samplePos += blockSize;
 
       // Update the Progress meter
-      if (TrackProgress(count, (samplePos - start) / (double)len)) {
+      if (TrackProgress(count, (samplePos - start).as_double() / len.as_double())) {
          bLoopSuccess = false;
          break;
       }
    }
 
    FinishTrack();
-   delete [] buffer;
 
    if (!mDoProfile) {
       // Flush the output WaveTrack (since it's buffered)
@@ -596,13 +574,8 @@ bool EffectNoiseRemoval::ProcessOne(int count, WaveTrack * track,
          double tLen = mOutputTrack->LongSamplesToTime(len);
          // Filtering effects always end up with more data than they started with.  Delete this 'tail'.
          mOutputTrack->HandleClear(tLen, mOutputTrack->GetEndTime(), false, false);
-         bool bResult = track->ClearAndPaste(t0, t0 + tLen, mOutputTrack, true, false);
-         wxASSERT(bResult); // TO DO: Actually handle this.
+         track->ClearAndPaste(t0, t0 + tLen, mOutputTrack.get(), true, false);
       }
-
-      // Delete the outputTrack now that its data is inserted in place
-      delete mOutputTrack;
-      mOutputTrack = NULL;
    }
 
    return bLoopSuccess;
@@ -644,7 +617,7 @@ enum {
 #define TIME_MAX 100    // Corresponds to 1.000 seconds
 
 
-BEGIN_EVENT_TABLE(NoiseRemovalDialog,wxDialog)
+BEGIN_EVENT_TABLE(NoiseRemovalDialog,wxDialogWrapper)
    EVT_BUTTON(wxID_OK, NoiseRemovalDialog::OnRemoveNoise)
    EVT_BUTTON(wxID_CANCEL, NoiseRemovalDialog::OnCancel)
    EVT_BUTTON(ID_EFFECT_PREVIEW, NoiseRemovalDialog::OnPreview)
@@ -662,8 +635,8 @@ BEGIN_EVENT_TABLE(NoiseRemovalDialog,wxDialog)
 END_EVENT_TABLE()
 
 NoiseRemovalDialog::NoiseRemovalDialog(EffectNoiseRemoval * effect,
-                                       wxWindow *parent) :
-   EffectDialog( parent, _("Noise Removal"), PROCESS_EFFECT)
+                                       wxWindow *parent)
+   : EffectDialog( parent, _("Noise Removal"), EffectTypeProcess)
 {
    m_pEffect = effect;
 
@@ -709,14 +682,16 @@ void NoiseRemovalDialog::OnPreview(wxCommandEvent & WXUNUSED(event))
    m_pEffect->mFreqSmoothingHz =  mFreq;
    m_pEffect->mAttackDecayTime =  mTime;
 
-   m_pEffect->Preview();
+   auto cleanup = finally( [&] {
+      m_pEffect->mSensitivity = oldSensitivity;
+      m_pEffect->mNoiseGain = oldGain;
+      m_pEffect->mFreqSmoothingHz =  oldFreq;
+      m_pEffect->mAttackDecayTime =  oldTime;
+      m_pEffect->mbLeaveNoise = oldLeaveNoise;
+      m_pEffect->mDoProfile = oldDoProfile;
+   } );
 
-   m_pEffect->mSensitivity = oldSensitivity;
-   m_pEffect->mNoiseGain = oldGain;
-   m_pEffect->mFreqSmoothingHz =  oldFreq;
-   m_pEffect->mAttackDecayTime =  oldTime;
-   m_pEffect->mbLeaveNoise = oldLeaveNoise;
-   m_pEffect->mDoProfile = oldDoProfile;
+   m_pEffect->Preview();
 }
 
 void NoiseRemovalDialog::OnRemoveNoise( wxCommandEvent & WXUNUSED(event))
@@ -866,3 +841,5 @@ void NoiseRemovalDialog::OnTimeSlider(wxCommandEvent & WXUNUSED(event))
    mTime = mTimeS->GetValue() / (TIME_MAX*1.0);
    mTimeT->SetValue(wxString::Format(wxT("%.2f"), mTime));
 }
+
+#endif
