@@ -13,40 +13,35 @@
 #ifndef __AUDACITY_EFFECT_REVERSE__
 #define __AUDACITY_EFFECT_REVERSE__
 
-#include <wx/intl.h>
+#include <wx/string.h>
 
 #include "Effect.h"
 
-#define __UNINITIALIZED__ (-1)
+#define REVERSE_PLUGIN_SYMBOL XO("Reverse")
 
-class WaveTrack;
-
-class EffectReverse:public Effect {
-
- public:
+class EffectReverse final : public Effect
+{
+public:
    EffectReverse();
+   virtual ~EffectReverse();
 
-   virtual wxString GetEffectName() {
-      return wxString(_("Reverse"));
-   }
+   // IdentInterface implementation
 
-   virtual std::set<wxString> GetEffectCategories() {
-      std::set<wxString> result;
-      result.insert(wxT("http://audacityteam.org/namespace#TimelineChanger"));
-      return result;
-   }
+   wxString GetSymbol() override;
+   wxString GetDescription() override;
 
-   virtual wxString GetEffectIdentifier() {
-      return wxString(wxT("Reverse"));
-   }
+   // EffectIdentInterface implementation
 
-   virtual wxString GetEffectAction() {
-      return wxString(_("Reversing"));
-   }
+   EffectType GetType() override;
+   bool IsInteractive() override;
 
-   virtual bool Process();
+   // Effect implementation
+
+   bool Process() override;
+
+private:
+   // EffectReverse implementation
 
- private:
    bool ProcessOneClip(int count, WaveTrack* track,
                    sampleCount start, sampleCount len, sampleCount originalStart, sampleCount originalEnd);
    bool ProcessOneWave(int count, WaveTrack* track, sampleCount start, sampleCount len);
