@@ -17,19 +17,29 @@
 #define __AUDACITY_EFFECT_SBSMS__
 
 #include "Effect.h"
+#include "../../../lib-src/header-substitutes/sbsms.h"
 
-#include "sbsms.h"
 using namespace _sbsms_;
 
-class EffectSBSMS : public Effect {
- public:
-   virtual bool Process();
+class LabelTrack;
+
+class EffectSBSMS /* not final */ : public Effect
+{
+public:
+   bool Process() override;
    void setParameters(double rateStart, double rateEnd, double pitchStart, double pitchEnd,
                       SlideType rateSlideType, SlideType pitchSlideType,
                       bool bLinkRatePitch, bool bRateReferenceInput, bool bPitchReferenceInput);
+   void setParameters(double tempoRatio, double pitchRatio);  // Constant ratio (tempoRatio, pitchRatio)
+   static double getInvertedStretchedTime(double rateStart, double rateEnd, SlideType slideType, double outputTime);
+   static double getRate(double rateStart, double rateEnd, SlideType slideType, double t);
 
- private:
-   bool ProcessLabelTrack(Track *track);
+protected:
+   wxString mProxyEffectName { XO("SBSMS Time / Pitch Stretch") };
+   wxString GetName() override { return mProxyEffectName; };
+
+private:
+   bool ProcessLabelTrack(LabelTrack *track);
    double rateStart, rateEnd, pitchStart, pitchEnd;
    bool bLinkRatePitch, bRateReferenceInput, bPitchReferenceInput;
    SlideType rateSlideType;
@@ -38,6 +48,9 @@ class EffectSBSMS : public Effect {
    double mCurT0;
    double mCurT1;
    float mTotalStretch;
+
+   friend class EffectChangeTempo;
+   friend class EffectChangePitch;
 };
 
 #endif
