@@ -35,8 +35,13 @@ and ImportLOF.cpp.
 
 
 
+#include "../Audacity.h"
+#include "Import.h"
+
+#include <algorithm>
+#include "ImportPlugin.h"
+
 #include <wx/textctrl.h>
-#include <wx/msgdlg.h>
 #include <wx/string.h>
 #include <wx/intl.h>
 #include <wx/log.h>
@@ -44,11 +49,8 @@ and ImportLOF.cpp.
 #include <wx/arrimpl.cpp>
 #include <wx/listimpl.cpp>
 #include "../ShuttleGui.h"
-#include "../Audacity.h"
 #include "../Project.h"
 
-#include "Import.h"
-#include "ImportPlugin.h"
 #include "ImportPCM.h"
 #include "ImportMP3.h"
 #include "ImportOGG.h"
@@ -58,19 +60,33 @@ and ImportLOF.cpp.
 #include "ImportFLAC.h"
 #include "ImportFFmpeg.h"
 #include "ImportGStreamer.h"
-#include "../Track.h"
 #include "../Prefs.h"
 
-WX_DEFINE_LIST(ImportPluginList);
-WX_DEFINE_LIST(UnusableImportPluginList);
-WX_DEFINE_LIST(FormatList);
-WX_DEFINE_OBJARRAY(ExtImportItems);
+// ============================================================================
+//
+// Return reference to singleton
+//
+// (Thread-safe...no active threading during construction or after destruction)
+// ============================================================================
+Importer Importer::mInstance;
+Importer & Importer::Get()
+{
+   return mInstance;
+}
 
 Importer::Importer()
 {
-   mImportPluginList = new ImportPluginList;
-   mUnusableImportPluginList = new UnusableImportPluginList;
-   mExtImportItems = NULL;
+}
+
+Importer::~Importer()
+{
+}
+
+bool Importer::Initialize()
+{
+   ImportPluginList{}.swap(mImportPluginList);
+   UnusableImportPluginList{}.swap(mUnusableImportPluginList);
+   ExtImportItems{}.swap(mExtImportItems);
 
    // build the list of import plugin and/or unusableImporters.
    // order is significant.  If none match, they will all be tried
@@ -92,28 +108,30 @@ Importer::Importer()
    #endif
 
    ReadImportItems();
+
+   return true;
 }
 
-Importer::~Importer()
+bool Importer::Terminate()
 {
    WriteImportItems();
-   mImportPluginList->DeleteContents(true);
-   delete mImportPluginList;
-   mUnusableImportPluginList->DeleteContents(true);//JKC
-   delete mUnusableImportPluginList;
-   if (this->mExtImportItems != NULL)
-      delete this->mExtImportItems;
+   ImportPluginList{}.swap( mImportPluginList );
+   UnusableImportPluginList{}.swap( mUnusableImportPluginList );
+
+   return true;
 }
 
 void Importer::GetSupportedImportFormats(FormatList *formatList)
 {
-   ImportPluginList::compatibility_iterator importPluginNode = mImportPluginList->GetFirst();
-   while(importPluginNode)
+   for(const auto &importPlugin : mImportPluginList)
    {
-      ImportPlugin *importPlugin = importPluginNode->GetData();
-      formatList->Append(new Format(importPlugin->GetPluginFormatDescription(),
-                                    importPlugin->GetSupportedExtensions()));
-      importPluginNode = importPluginNode->GetNext();
+#ifdef __AUDACITY_OLD_STD__
+      formatList->push_back(Format{importPlugin->GetPluginFormatDescription(),
+                               importPlugin->GetSupportedExtensions()});
+#else
+      formatList->emplace_back(importPlugin->GetPluginFormatDescription(),
+                               importPlugin->GetSupportedExtensions());
+#endif
    }
 }
 
@@ -131,13 +149,8 @@ void Importer::ReadImportItems()
    wxStringTokenizer toker;
    wxString item_name;
    wxString item_value;
-   ExtImportItem *new_item;
-   ImportPluginList::compatibility_iterator importPluginNode;
 
-   if (this->mExtImportItems != NULL)
-      delete this->mExtImportItems;
-
-   this->mExtImportItems = new ExtImportItems();
+   ExtImportItems{}.swap(mExtImportItems);
    /* Rule string format is:
     * extension1:extension2:extension3\mime_type1:mime_type2:mime_type3|filter1:filter2:filter3\unusedfilter1:unusedfilter2
     * backslashes are escaped and unescaped internally
@@ -155,7 +168,7 @@ void Importer::ReadImportItems()
       if (toker.CountTokens() != 2)
         break;
 
-      new_item = new ExtImportItem();
+      auto new_item = make_movable<ExtImportItem>();
 
       /* First token is the filtering condition, second - the filter list */
       condition = toker.GetNextToken();
@@ -196,29 +209,27 @@ void Importer::ReadImportItems()
       /* Find corresponding filter object for each filter ID */
       for (size_t i = 0; i < new_item->filters.Count(); i++)
       {
-         for (importPluginNode = mImportPluginList->GetFirst();
-            importPluginNode; importPluginNode = importPluginNode->GetNext())
+         bool found = false;
+         for (const auto &importPlugin : mImportPluginList)
          {
-            ImportPlugin *importPlugin = importPluginNode->GetData();
             if (importPlugin->GetPluginStringID().Cmp(new_item->filters[i]) == 0)
             {
-               new_item->filter_objects.Add (importPlugin);
+               new_item->filter_objects.Add (importPlugin.get());
+               found = true;
                break;
             }
          }
          /* IDs that do not have corresponding filters, will be shown as-is */
-         if (!importPluginNode)
+         if (!found)
            new_item->filter_objects.Add (NULL);
       }
       /* Find all filter objects that are not present in the filter list */
-      for (importPluginNode = mImportPluginList->GetFirst();
-         importPluginNode; importPluginNode = importPluginNode->GetNext())
+      for (const auto &importPlugin : mImportPluginList)
       {
          bool found = false;
-         ImportPlugin *importPlugin = importPluginNode->GetData();
          for (size_t i = 0; i < new_item->filter_objects.Count(); i++)
          {
-            if (importPlugin == new_item->filter_objects[i])
+            if (importPlugin.get() == new_item->filter_objects[i])
             {
                found = true;
                break;
@@ -231,12 +242,12 @@ void Importer::ReadImportItems()
             if (new_item->divider < 0)
                index = new_item->filters.Count();
             new_item->filters.Insert(importPlugin->GetPluginStringID(),index);
-            new_item->filter_objects.Insert (importPlugin, index);
+            new_item->filter_objects.Insert (importPlugin.get(), index);
             if (new_item->divider >= 0)
                new_item->divider++;
          }
       }
-      this->mExtImportItems->Add (new_item);
+      this->mExtImportItems.push_back( std::move(new_item) );
    }
 }
 
@@ -244,9 +255,9 @@ void Importer::WriteImportItems()
 {
    size_t i;
    wxString val, name;
-   for (i = 0; i < this->mExtImportItems->Count(); i++)
+   for (i = 0; i < this->mExtImportItems.size(); i++)
    {
-      ExtImportItem *item = &(mExtImportItems->Item(i));
+      ExtImportItem *item = mExtImportItems[i].get();
       val.Clear();
 
       for (size_t j = 0; j < item->extensions.Count(); j++)
@@ -279,20 +290,20 @@ void Importer::WriteImportItems()
                val.Append (wxT(":"));
          }
       }
-      name.Printf (wxT("/ExtImportItems/Item%d"), i);
+      name.Printf (wxT("/ExtImportItems/Item%d"), (int)i);
       gPrefs->Write (name, val);
       gPrefs->Flush();
    }
-   /* If we used to have more items than we have now, delete the excess items.
+   /* If we used to have more items than we have now, DELETE the excess items.
    We just keep deleting items and incrementing until we find there aren't any
-   more to delete.*/
-   i = this->mExtImportItems->Count();
+   more to DELETE.*/
+   i = this->mExtImportItems.size();
    do {
-     name.Printf (wxT("/ExtImportItems/Item%d"), i);
-     // No item to delete?  Then it's time to finish.
+     name.Printf (wxT("/ExtImportItems/Item%d"), (int)i);
+     // No item to DELETE?  Then it's time to finish.
      if (!gPrefs->Read(name, &val))
         break;
-     // Failure to delete probably means a read-only config file.
+     // Failure to DELETE probably means a read-only config file.
      // no point continuing.
      // TODO: Possibly report (once).
      if( !gPrefs->DeleteEntry (name, false))
@@ -301,47 +312,58 @@ void Importer::WriteImportItems()
    } while( true );
 }
 
-ExtImportItem *Importer::CreateDefaultImportItem()
+movable_ptr<ExtImportItem> Importer::CreateDefaultImportItem()
 {
-   ExtImportItem *new_item;
-   ImportPluginList::compatibility_iterator importPluginNode;
-
-   new_item = new ExtImportItem();
+   auto new_item = make_movable<ExtImportItem>();
    new_item->extensions.Add(wxT("*"));
    new_item->mime_types.Add(wxT("*"));
 
-   for (importPluginNode = mImportPluginList->GetFirst();
-      importPluginNode; importPluginNode = importPluginNode->GetNext())
+   for (const auto &importPlugin : mImportPluginList)
    {
-      ImportPlugin *importPlugin = importPluginNode->GetData();
       new_item->filters.Add (importPlugin->GetPluginStringID());
-      new_item->filter_objects.Add (importPlugin);
+      new_item->filter_objects.Add (importPlugin.get());
    }
    new_item->divider = -1;
    return new_item;
 }
 
+bool Importer::IsMidi(const wxString &fName)
+{
+   const auto extension = fName.AfterLast(wxT('.'));
+   return
+      extension.IsSameAs(wxT("gro"), false) ||
+      extension.IsSameAs(wxT("midi"), false) ||
+      extension.IsSameAs(wxT("mid"), false);
+}
+
 // returns number of tracks imported
-int Importer::Import(wxString fName,
+bool Importer::Import(const wxString &fName,
                      TrackFactory *trackFactory,
-                     Track *** tracks,
+                     TrackHolders &tracks,
                      Tags *tags,
                      wxString &errorMessage)
 {
    AudacityProject *pProj = GetActiveProject();
-   pProj->mbBusyImporting = true;
-
-   ImportFileHandle *inFile = NULL;
-   int numTracks = 0;
+   auto cleanup = valueRestorer( pProj->mbBusyImporting, true );
 
    wxString extension = fName.AfterLast(wxT('.'));
 
+   // Always refuse to import MIDI, even though the FFmpeg plugin pretends to know how (but makes very bad renderings)
+#ifdef USE_MIDI
+   // MIDI files must be imported, not opened
+   if (IsMidi(fName)) {
+      errorMessage.Printf(_("\"%s\" \nis a MIDI file, not an audio file. \nAudacity cannot open this type of file for playing, but you can\nedit it by clicking File > Import > MIDI."), fName);
+      return false;
+   }
+#endif
+
+   using ImportPluginPtrs = std::vector< ImportPlugin* >;
+
    // This list is used to call plugins in correct order
-   ImportPluginList importPlugins;
-   ImportPluginList::compatibility_iterator importPluginNode;
+   ImportPluginPtrs importPlugins;
 
    // This list is used to remember plugins that should have been compatible with the file.
-   ImportPluginList compatiblePlugins;
+   ImportPluginPtrs compatiblePlugins;
 
    // If user explicitly selected a filter,
    // then we should try importing via corresponding plugin first
@@ -354,36 +376,33 @@ int Importer::Import(wxString fName,
    bool usersSelectionOverrides;
    gPrefs->Read(wxT("/ExtendedImport/OverrideExtendedImportByOpenFileDialogChoice"), &usersSelectionOverrides, false);
 
-   wxLogDebug(wxT("LastOpenType is %s"),type.c_str());
+   wxLogDebug(wxT("LastOpenType is %s"),type);
    wxLogDebug(wxT("OverrideExtendedImportByOpenFileDialogChoice is %i"),usersSelectionOverrides);
 
    if (usersSelectionOverrides)
    {
-      importPluginNode = mImportPluginList->GetFirst();
-      while (importPluginNode)
+      for (const auto &plugin : mImportPluginList)
       {
-         ImportPlugin *plugin = importPluginNode->GetData();
          if (plugin->GetPluginFormatDescription().CompareTo(type) == 0)
          {
             // This plugin corresponds to user-selected filter, try it first.
-            wxLogDebug(wxT("Inserting %s"),plugin->GetPluginStringID().c_str());
-            importPlugins.Insert(plugin);
+            wxLogDebug(wxT("Inserting %s"),plugin->GetPluginStringID());
+            importPlugins.insert(importPlugins.begin(), plugin.get());
          }
-         importPluginNode = importPluginNode->GetNext();
       }
    }
 
-   wxLogMessage(wxT("File name is %s"),(const char *) fName.c_str());
-   wxLogMessage(wxT("Mime type is %s"),(const char *) mime_type.Lower().c_str());
+   wxLogMessage(wxT("File name is %s"), fName);
+   wxLogMessage(wxT("Mime type is %s"), mime_type.Lower());
 
-   for (size_t i = 0; i < mExtImportItems->Count(); i++)
+   for (const auto &uItem : mExtImportItems)
    {
-      ExtImportItem *item = &(*mExtImportItems)[i];
+      ExtImportItem *item = uItem.get();
       bool matches_ext = false, matches_mime = false;
       wxLogDebug(wxT("Testing extensions"));
       for (size_t j = 0; j < item->extensions.Count(); j++)
       {
-         wxLogDebug(wxT("%s"), (const char *) item->extensions[j].Lower().c_str());
+         wxLogDebug(wxT("%s"), item->extensions[j].Lower());
          if (wxMatchWild (item->extensions[j].Lower(),fName.Lower(), false))
          {
             wxLogDebug(wxT("Match!"));
@@ -424,14 +443,13 @@ int Importer::Import(wxString fName,
             // is still ffmpeg in prefs from previous --with-ffmpeg builds
             if (!(item->filter_objects[j]))
                continue;
-            wxLogDebug(wxT("Inserting %s"),item->filter_objects[j]->GetPluginStringID().c_str());
-            importPlugins.Append(item->filter_objects[j]);
+            wxLogDebug(wxT("Inserting %s"),item->filter_objects[j]->GetPluginStringID());
+            importPlugins.push_back(item->filter_objects[j]);
          }
       }
    }
 
    // Add all plugins that support the extension
-   importPluginNode = mImportPluginList->GetFirst();
 
    // Here we rely on the fact that the first plugin in mImportPluginList is libsndfile.
    // We want to save this for later insertion ahead of libmad, if libmad supports the extension.
@@ -439,14 +457,14 @@ int Importer::Import(wxString fName,
    // is not changed by user selection overrides or any other mechanism, but we include an assert
    // in case subsequent code revisions to the constructor should break this assumption that
    // libsndfile is first.
-   ImportPlugin *libsndfilePlugin = importPluginNode->GetData();
+   ImportPlugin *libsndfilePlugin = mImportPluginList.begin()->get();
    wxASSERT(libsndfilePlugin->GetPluginStringID().IsSameAs(wxT("libsndfile")));
 
-   while (importPluginNode)
+   for (const auto &plugin : mImportPluginList)
    {
-      ImportPlugin *plugin = importPluginNode->GetData();
       // Make sure its not already in the list
-      if (importPlugins.Find(plugin) == NULL)
+      if (importPlugins.end() ==
+          std::find(importPlugins.begin(), importPlugins.end(), plugin.get()))
       {
          if (plugin->SupportsExtension(extension))
          {
@@ -461,94 +479,80 @@ int Importer::Import(wxString fName,
             if (plugin->GetPluginStringID().IsSameAs(wxT("libmad")))
             {
                // Make sure libsndfile is not already in the list
-               if (importPlugins.Find(libsndfilePlugin) == NULL)
+               if (importPlugins.end() ==
+                   std::find(importPlugins.begin(), importPlugins.end(), libsndfilePlugin))
                {
-                  wxLogDebug(wxT("Appending %s"),libsndfilePlugin->GetPluginStringID().c_str());
-                  importPlugins.Append(libsndfilePlugin);
+                  wxLogDebug(wxT("Appending %s"),libsndfilePlugin->GetPluginStringID());
+                  importPlugins.push_back(libsndfilePlugin);
                }
             }
-            wxLogDebug(wxT("Appending %s"),plugin->GetPluginStringID().c_str());
-            importPlugins.Append(plugin);
+            wxLogDebug(wxT("Appending %s"),plugin->GetPluginStringID());
+            importPlugins.push_back(plugin.get());
          }
       }
-
-      importPluginNode = importPluginNode->GetNext();
    }
 
    // Add remaining plugins, except for libmad, which should not be used as a fallback for anything.
    // Otherwise, if FFmpeg (libav) has not been installed, libmad will still be there near the
    // end of the preference list importPlugins, where it will claim success importing FFmpeg file
    // formats unsuitable for it, and produce distorted results.
-   importPluginNode = mImportPluginList->GetFirst();
-   while (importPluginNode)
+   for (const auto &plugin : mImportPluginList)
    {
-      ImportPlugin *plugin = importPluginNode->GetData();
       if (!(plugin->GetPluginStringID().IsSameAs(wxT("libmad"))))
       {
          // Make sure its not already in the list
-         if (importPlugins.Find(plugin) == NULL)
+         if (importPlugins.end() ==
+             std::find(importPlugins.begin(), importPlugins.end(), plugin.get()))
          {
-            wxLogDebug(wxT("Appending %s"),plugin->GetPluginStringID().c_str());
-            importPlugins.Append(plugin);
+            wxLogDebug(wxT("Appending %s"),plugin->GetPluginStringID());
+            importPlugins.push_back(plugin.get());
          }
       }
-
-      importPluginNode = importPluginNode->GetNext();
    }
 
-   importPluginNode = importPlugins.GetFirst();
-   while(importPluginNode)
+   // Try the import plugins, in the permuted sequences just determined
+   for (const auto plugin : importPlugins)
    {
-      ImportPlugin *plugin = importPluginNode->GetData();
       // Try to open the file with this plugin (probe it)
-      wxLogMessage(wxT("Opening with %s"),plugin->GetPluginStringID().c_str());
-      inFile = plugin->Open(fName);
+      wxLogMessage(wxT("Opening with %s"),plugin->GetPluginStringID());
+      auto inFile = plugin->Open(fName);
       if ( (inFile != NULL) && (inFile->GetStreamCount() > 0) )
       {
-         wxLogMessage(wxT("Open(%s) succeeded"),(const char *) fName.c_str());
+         wxLogMessage(wxT("Open(%s) succeeded"), fName);
          // File has more than one stream - display stream selector
          if (inFile->GetStreamCount() > 1)
          {
-            ImportStreamDialog ImportDlg(inFile, NULL, -1, _("Select stream(s) to import"));
+            ImportStreamDialog ImportDlg(inFile.get(), NULL, -1, _("Select stream(s) to import"));
 
             if (ImportDlg.ShowModal() == wxID_CANCEL)
             {
-               delete inFile;
-               pProj->mbBusyImporting = false;
-               return 0;
+               return false;
             }
          }
          // One stream - import it by default
          else
             inFile->SetStreamUsage(0,TRUE);
 
-         int res;
+         auto res = inFile->Import(trackFactory, tracks, tags);
 
-         res = inFile->Import(trackFactory, tracks, &numTracks, tags);
-
-         delete inFile;
-
-         if (res == eProgressSuccess || res == eProgressStopped)
+         if (res == ProgressResult::Success || res == ProgressResult::Stopped)
          {
             // LOF ("list-of-files") has different semantics
             if (extension.IsSameAs(wxT("lof"), false))
             {
-               pProj->mbBusyImporting = false;
-               return 1;
+               return true;
             }
 
-            if (numTracks > 0)
+            if (tracks.size() > 0)
             {
                // success!
-               pProj->mbBusyImporting = false;
-               return numTracks;
+               return true;
             }
          }
 
-         if (res == eProgressCancelled || res == eProgressFailed)
+         if (res == ProgressResult::Cancelled || res == ProgressResult::Failed)
          {
-            pProj->mbBusyImporting = false;
-            return 0;
+            return false;
          }
 
          // We could exit here since we had a match on the file extension,
@@ -556,178 +560,161 @@ int Importer::Import(wxString fName,
          // that may recognize the extension, so we allow the loop to
          // continue.
       }
-      importPluginNode = importPluginNode->GetNext();
    }
    wxLogError(wxT("Importer::Import: Opening failed."));
 
    // None of our plugins can handle this file.  It might be that
    // Audacity supports this format, but support was not compiled in.
    // If so, notify the user of this fact
-   UnusableImportPluginList::compatibility_iterator unusableImporterNode
-      = mUnusableImportPluginList->GetFirst();
-   while(unusableImporterNode)
+   for (const auto &unusableImportPlugin : mUnusableImportPluginList)
    {
-      UnusableImportPlugin *unusableImportPlugin = unusableImporterNode->GetData();
       if( unusableImportPlugin->SupportsExtension(extension) )
       {
          errorMessage.Printf(_("This version of Audacity was not compiled with %s support."),
                              unusableImportPlugin->
-                             GetPluginFormatDescription().c_str());
-         pProj->mbBusyImporting = false;
-         return 0;
+                             GetPluginFormatDescription());
+         return false;
       }
-      unusableImporterNode = unusableImporterNode->GetNext();
    }
 
    /* warnings for unsupported data types */
 
-#ifdef USE_MIDI
-   // MIDI files must be imported, not opened
-   if ((extension.IsSameAs(wxT("midi"), false))||(extension.IsSameAs(wxT("mid"), false))) {
-      errorMessage.Printf(_("\"%s\" \nis a MIDI file, not an audio file. \nAudacity cannot open this type of file for playing, but you can\nedit it by clicking File > Import > MIDI."), fName.c_str());
-      pProj->mbBusyImporting = false;
-      return 0;
-   }
-#endif
-
-   if (compatiblePlugins.GetCount() <= 0)
+   if (compatiblePlugins.empty())
    {
       // if someone has sent us a .cda file, send them away
       if (extension.IsSameAs(wxT("cda"), false)) {
          /* i18n-hint: %s will be the filename */
-         errorMessage.Printf(_("\"%s\" is an audio CD track. \nAudacity cannot open audio CDs directly. \nExtract (rip) the CD tracks to an audio format that \nAudacity can import, such as WAV or AIFF."), fName.c_str());
-         pProj->mbBusyImporting = false;
-         return 0;
+         errorMessage.Printf(_("\"%s\" is an audio CD track. \nAudacity cannot open audio CDs directly. \nExtract (rip) the CD tracks to an audio format that \nAudacity can import, such as WAV or AIFF."), fName);
+         return false;
       }
 
       // playlist type files
       if ((extension.IsSameAs(wxT("m3u"), false))||(extension.IsSameAs(wxT("ram"), false))||(extension.IsSameAs(wxT("pls"), false))) {
-         errorMessage.Printf(_("\"%s\" is a playlist file. \nAudacity cannot open this file because it only contains links to other files. \nYou may be able to open it in a text editor and download the actual audio files."), fName.c_str());
-         pProj->mbBusyImporting = false;
-         return 0;
+         errorMessage.Printf(_("\"%s\" is a playlist file. \nAudacity cannot open this file because it only contains links to other files. \nYou may be able to open it in a text editor and download the actual audio files."), fName);
+         return false;
       }
       //WMA files of various forms
       if ((extension.IsSameAs(wxT("wma"), false))||(extension.IsSameAs(wxT("asf"), false))) {
-         errorMessage.Printf(_("\"%s\" is a Windows Media Audio file. \nAudacity cannot open this type of file due to patent restrictions. \nYou need to convert it to a supported audio format, such as WAV or AIFF."), fName.c_str());
-         pProj->mbBusyImporting = false;
-         return 0;
+         errorMessage.Printf(_("\"%s\" is a Windows Media Audio file. \nAudacity cannot open this type of file due to patent restrictions. \nYou need to convert it to a supported audio format, such as WAV or AIFF."), fName);
+         return false;
       }
       //AAC files of various forms (probably not encrypted)
       if ((extension.IsSameAs(wxT("aac"), false))||(extension.IsSameAs(wxT("m4a"), false))||(extension.IsSameAs(wxT("m4r"), false))||(extension.IsSameAs(wxT("mp4"), false))) {
-         errorMessage.Printf(_("\"%s\" is an Advanced Audio Coding file. \nAudacity cannot open this type of file. \nYou need to convert it to a supported audio format, such as WAV or AIFF."), fName.c_str());
-         pProj->mbBusyImporting = false;
-         return 0;
+         errorMessage.Printf(_("\"%s\" is an Advanced Audio Coding file. \nAudacity cannot open this type of file. \nYou need to convert it to a supported audio format, such as WAV or AIFF."), fName);
+         return false;
       }
       // encrypted itunes files
       if ((extension.IsSameAs(wxT("m4p"), false))) {
-         errorMessage.Printf(_("\"%s\" is an encrypted audio file. \nThese typically are from an online music store. \nAudacity cannot open this type of file due to the encryption. \nTry recording the file into Audacity, or burn it to audio CD then \nextract the CD track to a supported audio format such as WAV or AIFF."), fName.c_str());
-         pProj->mbBusyImporting = false;
-         return 0;
+         errorMessage.Printf(_("\"%s\" is an encrypted audio file. \nThese typically are from an online music store. \nAudacity cannot open this type of file due to the encryption. \nTry recording the file into Audacity, or burn it to audio CD then \nextract the CD track to a supported audio format such as WAV or AIFF."), fName);
+         return false;
       }
       // Real Inc. files of various sorts
       if ((extension.IsSameAs(wxT("ra"), false))||(extension.IsSameAs(wxT("rm"), false))||(extension.IsSameAs(wxT("rpm"), false))) {
-         errorMessage.Printf(_("\"%s\" is a RealPlayer media file. \nAudacity cannot open this proprietary format. \nYou need to convert it to a supported audio format, such as WAV or AIFF."), fName.c_str());
-         pProj->mbBusyImporting = false;
-         return 0;
+         errorMessage.Printf(_("\"%s\" is a RealPlayer media file. \nAudacity cannot open this proprietary format. \nYou need to convert it to a supported audio format, such as WAV or AIFF."), fName);
+         return false;
       }
 
       // Other notes-based formats
       if ((extension.IsSameAs(wxT("kar"), false))||(extension.IsSameAs(wxT("mod"), false))||(extension.IsSameAs(wxT("rmi"), false))) {
-         errorMessage.Printf(_("\"%s\" is a notes-based file, not an audio file. \nAudacity cannot open this type of file. \nTry converting it to an audio file such as WAV or AIFF and \nthen import it, or record it into Audacity."), fName.c_str());
-         pProj->mbBusyImporting = false;
-         return 0;
+         errorMessage.Printf(_("\"%s\" is a notes-based file, not an audio file. \nAudacity cannot open this type of file. \nTry converting it to an audio file such as WAV or AIFF and \nthen import it, or record it into Audacity."), fName);
+         return false;
       }
 
       // MusePack files
       if ((extension.IsSameAs(wxT("mp+"), false))||(extension.IsSameAs(wxT("mpc"), false))||(extension.IsSameAs(wxT("mpp"), false))) {
-         errorMessage.Printf(_("\"%s\" is a Musepack audio file. \nAudacity cannot open this type of file. \nIf you think it might be an mp3 file, rename it to end with \".mp3\" \nand try importing it again. Otherwise you need to convert it to a supported audio \nformat, such as WAV or AIFF."), fName.c_str());
-         pProj->mbBusyImporting = false;
-         return 0;
+         errorMessage.Printf(_("\"%s\" is a Musepack audio file. \nAudacity cannot open this type of file. \nIf you think it might be an mp3 file, rename it to end with \".mp3\" \nand try importing it again. Otherwise you need to convert it to a supported audio \nformat, such as WAV or AIFF."), fName);
+         return false;
       }
 
       // WavPack files
       if ((extension.IsSameAs(wxT("wv"), false))||(extension.IsSameAs(wxT("wvc"), false))) {
-         errorMessage.Printf(_("\"%s\" is a Wavpack audio file. \nAudacity cannot open this type of file. \nYou need to convert it to a supported audio format, such as WAV or AIFF."), fName.c_str());
-         pProj->mbBusyImporting = false;
-         return 0;
+         errorMessage.Printf(_("\"%s\" is a Wavpack audio file. \nAudacity cannot open this type of file. \nYou need to convert it to a supported audio format, such as WAV or AIFF."), fName);
+         return false;
       }
 
       // AC3 files
       if ((extension.IsSameAs(wxT("ac3"), false))) {
-         errorMessage.Printf(_("\"%s\" is a Dolby Digital audio file. \nAudacity cannot currently open this type of file. \nYou need to convert it to a supported audio format, such as WAV or AIFF."), fName.c_str());
-         pProj->mbBusyImporting = false;
-         return 0;
+         errorMessage.Printf(_("\"%s\" is a Dolby Digital audio file. \nAudacity cannot currently open this type of file. \nYou need to convert it to a supported audio format, such as WAV or AIFF."), fName);
+         return false;
       }
 
       // Speex files
       if ((extension.IsSameAs(wxT("spx"), false))) {
-         errorMessage.Printf(_("\"%s\" is an Ogg Speex audio file. \nAudacity cannot currently open this type of file. \nYou need to convert it to a supported audio format, such as WAV or AIFF."), fName.c_str());
-         pProj->mbBusyImporting = false;
-         return 0;
+         errorMessage.Printf(_("\"%s\" is an Ogg Speex audio file. \nAudacity cannot currently open this type of file. \nYou need to convert it to a supported audio format, such as WAV or AIFF."), fName);
+         return false;
       }
 
       // Video files of various forms
       if ((extension.IsSameAs(wxT("mpg"), false))||(extension.IsSameAs(wxT("mpeg"), false))||(extension.IsSameAs(wxT("avi"), false))||(extension.IsSameAs(wxT("wmv"), false))||(extension.IsSameAs(wxT("rv"), false))) {
-         errorMessage.Printf(_("\"%s\" is a video file. \nAudacity cannot currently open this type of file. \nYou need to extract the audio to a supported format, such as WAV or AIFF."), fName.c_str());
-         pProj->mbBusyImporting = false;
-         return 0;
+         errorMessage.Printf(_("\"%s\" is a video file. \nAudacity cannot currently open this type of file. \nYou need to extract the audio to a supported format, such as WAV or AIFF."), fName);
+         return false;
+      }
+
+      // Audacity project
+      if (extension.IsSameAs(wxT("aup"), false)) {
+         errorMessage.Printf(_("\"%s\" is an Audacity Project file. \nUse the 'File > Open' command to open Audacity Projects."), fName);
+         return false;
       }
 
       // we were not able to recognize the file type
-      errorMessage.Printf(_("Audacity did not recognize the type of the file '%s'.\nIf it is uncompressed, try importing it using \"Import Raw\"."),fName.c_str());
+      errorMessage.Printf(_("Audacity did not recognize the type of the file '%s'.\nTry installing FFmpeg. For uncompressed files, also try File > Import > Raw Data."),fName);
    }
    else
    {
       // We DO have a plugin for this file, but import failed.
-      wxString pluglist = wxEmptyString;
+      wxString pluglist;
 
-      importPluginNode = compatiblePlugins.GetFirst();
-      while(importPluginNode)
+      for (const auto &plugin : compatiblePlugins)
       {
-         ImportPlugin *plugin = importPluginNode->GetData();
-         if (pluglist == wxEmptyString)
+         if (pluglist.empty())
            pluglist = plugin->GetPluginFormatDescription();
          else
-           pluglist = pluglist + wxT(", ") + plugin->GetPluginFormatDescription();
-         importPluginNode = importPluginNode->GetNext();
+           pluglist = wxString::Format( _("%s, %s"),
+               pluglist, plugin->GetPluginFormatDescription() );
       }
 
-      errorMessage.Printf(_("Audacity recognized the type of the file '%s'.\nImporters supposedly supporting such files are:\n%s,\nbut none of them understood this file format."),fName.c_str(), pluglist.c_str());
+      errorMessage.Printf(_("Audacity recognized the type of the file '%s'.\nImporters supposedly supporting such files are:\n%s,\nbut none of them understood this file format."),fName, pluglist);
    }
 
-   pProj->mbBusyImporting = false;
-   return 0;
+   return false;
 }
 
 //-------------------------------------------------------------------------
 // ImportStreamDialog
 //-------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE( ImportStreamDialog,wxDialog )
+BEGIN_EVENT_TABLE( ImportStreamDialog, wxDialogWrapper )
    EVT_BUTTON( wxID_OK, ImportStreamDialog::OnOk )
    EVT_BUTTON( wxID_CANCEL, ImportStreamDialog::OnCancel )
 END_EVENT_TABLE()
 
 ImportStreamDialog::ImportStreamDialog( ImportFileHandle *_mFile, wxWindow *parent, wxWindowID id, const wxString &title,
                                        const wxPoint &position, const wxSize& size, long style ):
-wxDialog( parent, id, title, position, size, style | wxRESIZE_BORDER )
+wxDialogWrapper( parent, id, title, position, size, style | wxRESIZE_BORDER )
 {
+   SetName(GetTitle());
+
    mFile = _mFile;
    scount = mFile->GetStreamCount();
    for (wxInt32 i = 0; i < scount; i++)
-      mFile->SetStreamUsage(i,FALSE);
+      mFile->SetStreamUsage(i, FALSE);
 
-   wxBoxSizer *vertSizer = new wxBoxSizer( wxVERTICAL );
-   wxArrayString *choices = mFile->GetStreamInfo();
-   StreamList = new wxListBox(this, -1, wxDefaultPosition, wxDefaultSize, *choices , wxLB_EXTENDED | wxLB_ALWAYS_SB);
+   wxBoxSizer *vertSizer;
+   {
+      auto uVertSizer = std::make_unique<wxBoxSizer>(wxVERTICAL);
+      vertSizer = uVertSizer.get();
 
-   vertSizer->Add( StreamList, 1, wxEXPAND | wxALIGN_LEFT | wxALL, 5 );
+      auto choices = mFile->GetStreamInfo();
+      StreamList = safenew wxListBox(this, -1, wxDefaultPosition, wxDefaultSize, choices, wxLB_EXTENDED | wxLB_ALWAYS_SB);
 
-   vertSizer->Add( CreateStdButtonSizer(this, eCancelButton|eOkButton), 0, wxEXPAND );
+      vertSizer->Add(StreamList, 1, wxEXPAND | wxALIGN_LEFT | wxALL, 5);
 
-   SetAutoLayout( true );
+      vertSizer->Add(CreateStdButtonSizer(this, eCancelButton | eOkButton).release(), 0, wxEXPAND);
 
-   SetSizer( vertSizer );
+      SetAutoLayout(true);
+
+      SetSizer(uVertSizer.release());
+   }
 
    vertSizer->Fit( this );
 
