@@ -14,16 +14,17 @@
 *//*******************************************************************/
 
 
+#include "../Audacity.h"
+#include "ExtImportPrefs.h"
+
 #include <wx/defs.h>
 #include <wx/listctrl.h>
-#include <wx/msgdlg.h>
 #include <wx/dnd.h>
 
-#include "ExtImportPrefs.h"
 #include "../Audacity.h"
-#include "../AudacityApp.h"
 #include "../Prefs.h"
 #include "../ShuttleGui.h"
+#include "../widgets/ErrorDialog.h"
 
 #define EXTIMPORT_MIME_SUPPORT 0
 
@@ -56,16 +57,12 @@ BEGIN_EVENT_TABLE(ExtImportPrefs, PrefsPanel)
 END_EVENT_TABLE()
 
 ExtImportPrefs::ExtImportPrefs(wxWindow * parent)
+/* i18n-hint:  Title of dialog governing "Extended", or "advanced,"
+ * audio file import options */
 :   PrefsPanel(parent, _("Extended Import")), RuleTable(NULL),
     PluginList(NULL), mCreateTable (false), mDragFocus (NULL),
     mFakeKeyEvent (false), mStopRecursiveSelection (false), last_selected (-1)
 {
-   dragtext1 = new wxTextDataObject(wxT(""));
-   dragtext2 = new wxTextDataObject(wxT(""));
-   dragtarget1 = new ExtImportPrefsDropTarget(dragtext1);
-   dragtarget2 = new ExtImportPrefsDropTarget(dragtext2);
-   dragtarget1->SetPrefs (this);
-   dragtarget2->SetPrefs (this);
    Populate();
 }
 
@@ -88,6 +85,7 @@ void ExtImportPrefs::Populate()
 void ExtImportPrefs::PopulateOrExchange(ShuttleGui & S)
 {
    S.SetBorder(2);
+   S.StartScroller();
 
    S.TieCheckBox(_("A&ttempt to use filter in OpenFile dialog first"),
          wxT("/ExtendedImport/OverrideExtendedImportByOpenFileDialogChoice"),
@@ -100,7 +98,7 @@ void ExtImportPrefs::PopulateOrExchange(ShuttleGui & S)
          bool fillRuleTable = false;
          if (RuleTable == NULL)
          {
-            RuleTable = new Grid(S.GetParent(),EIPRuleTable);
+            RuleTable = safenew Grid(S.GetParent(),EIPRuleTable);
 
             RuleTable->SetColLabelSize(RuleTable->GetDefaultRowSize());
 #if EXTIMPORT_MIME_SUPPORT
@@ -117,9 +115,18 @@ void ExtImportPrefs::PopulateOrExchange(ShuttleGui & S)
 #endif
             RuleTable->SetRowLabelSize (0);
             RuleTable->SetSelectionMode (wxGrid::wxGridSelectRows);
-            RuleTable->AutoSizeColumns ();
+            // call SetMinSize to enable scrolling on large content
+            RuleTable->Fit();
+            RuleTable->SetMinSize(RuleTable->GetSize());
+
+            ExtImportPrefsDropTarget *dragtarget1 {};
+            RuleTable->SetDropTarget (
+               dragtarget1 = safenew ExtImportPrefsDropTarget(
+                  dragtext1 = safenew wxTextDataObject(wxT(""))
+               )
+            );
+            dragtarget1->SetPrefs (this);
 
-            RuleTable->SetDropTarget (dragtarget1);
             RuleTable->EnableDragCell (true);
             fillRuleTable = true;
          }
@@ -132,12 +139,24 @@ void ExtImportPrefs::PopulateOrExchange(ShuttleGui & S)
             PluginList->SetSingleStyle (wxLC_REPORT, true);
             PluginList->SetSingleStyle (wxLC_SINGLE_SEL, true);
             PluginList->InsertColumn (0, _("Importer order"));
-            PluginList->SetDropTarget (dragtarget2);
 
-            ExtImportItems *items = wxGetApp().mImporter->GetImportItems();
-            for (unsigned int i = 0; i < items->Count(); i++)
-               AddItemToTable (i, &(*items)[i]);
-            if (items->Count() > 0)
+            ExtImportPrefsDropTarget *dragtarget2 {};
+            PluginList->SetDropTarget (
+               dragtarget2 = safenew ExtImportPrefsDropTarget(
+                  dragtext2 = safenew wxTextDataObject(wxT(""))
+               )
+            );
+            dragtarget2->SetPrefs (this);
+
+            PluginList->SetColumnWidth (0, wxLIST_AUTOSIZE_USEHEADER);
+
+            auto &items = Importer::Get().GetImportItems();
+            {
+               int i = -1;
+               for (const auto &item : items)
+                  AddItemToTable (++i, item.get());
+            }
+            if (!items.empty())
             {
                RuleTable->SelectRow(0);
                RuleTable->SetGridCursor(0,0);
@@ -164,12 +183,14 @@ void ExtImportPrefs::PopulateOrExchange(ShuttleGui & S)
       S.EndHorizontalLay();
    }
    S.EndStatic();
+   S.EndScroller();
+
    Layout();
    Fit();
    SetMinSize(GetSize());
 }
 
-bool ExtImportPrefs::Apply()
+bool ExtImportPrefs::Commit()
 {
    ShuttleGui S(this, eIsSavingToPrefs);
    PopulateOrExchange(S);
@@ -200,8 +221,10 @@ void ExtImportPrefs::SwapPluginRows (int row1, int row2)
    long d, d2;
    ImportPlugin *ip1, *ip2;
 
-   ExtImportItems *items = wxGetApp().mImporter->GetImportItems();
-   ExtImportItem *item = &(*items)[last_selected];
+   auto &items = Importer::Get().GetImportItems();
+   ExtImportItem *item = NULL;
+   if( last_selected >= 0 )
+      item = items[last_selected].get();
 
    t = PluginList->GetItemText (row1);
    d = PluginList->GetItemData (row1);
@@ -212,6 +235,8 @@ void ExtImportPrefs::SwapPluginRows (int row1, int row2)
    {
       PluginList->SetItemData (row1, PluginList->GetItemData (row2));
       PluginList->SetItemData (row2, d);
+      if( !item )
+         return;
       if (d == -1)
       {
          item->divider = row2;
@@ -223,6 +248,8 @@ void ExtImportPrefs::SwapPluginRows (int row1, int row2)
    }
    else
    {
+      if( !item )
+         return;
       ip1 = item->filter_objects[d];
       ip2 = item->filter_objects[d2];
       item->filter_objects[d] = ip2;
@@ -249,8 +276,8 @@ bool ExtImportPrefs::DoOnPluginKeyDown (int code)
    if (last_selected == -1)
          return false;
 
-   ExtImportItems *items = wxGetApp().mImporter->GetImportItems();
-   ExtImportItem *item = &(*items)[last_selected];
+   auto &items = Importer::Get().GetImportItems();
+   ExtImportItem *item = items[last_selected].get();
 
    if (code == WXK_UP && itemIndex == 0)
       return false;
@@ -286,7 +313,6 @@ bool ExtImportPrefs::DoOnPluginKeyDown (int code)
 void ExtImportPrefs::SwapRows (int row1, int row2)
 {
    int t;
-   ExtImportItem *t1, *t2;
    wxString ts;
    if (row1 == row2)
       return;
@@ -296,11 +322,12 @@ void ExtImportPrefs::SwapRows (int row1, int row2)
       row1 = row2;
       row2 = t;
    }
-   ExtImportItems *items = wxGetApp().mImporter->GetImportItems();
-   t1 = items->Detach(row1);
-   t2 = items->Detach(row1);
-   items->Insert (t1, row1);
-   items->Insert (t2, row1);
+   auto &items = Importer::Get().GetImportItems();
+
+   auto &t1 = items[row1];
+   auto &t2 = items[row2];
+   std::swap(t1, t2);
+
    for (int i = 0; i < RuleTable->GetNumberCols(); i++)
    {
       ts = RuleTable->GetCellValue (row2, i);
@@ -315,7 +342,9 @@ void ExtImportPrefs::OnPluginBeginDrag(wxListEvent& WXUNUSED(event))
    dragtext2->SetText(wxT(""));
    dragSource.SetData(*dragtext2);
    mDragFocus = PluginList;
-   wxDragResult result = dragSource.DoDragDrop(TRUE);
+   if( mDragFocus == NULL )
+      return;
+   wxDragResult result = dragSource.DoDragDrop(wxDrag_DefaultMove);
    mDragFocus = NULL;
    switch (result)
    {
@@ -399,14 +428,14 @@ void ExtImportPrefs::OnRuleTableSelectRange (wxGridRangeSelectEvent& event)
 
 void ExtImportPrefs::DoOnRuleTableSelect (int toprow)
 {
-   ExtImportItems *items = wxGetApp().mImporter->GetImportItems();
+   auto &items = Importer::Get().GetImportItems();
 
-   if (toprow < 0 || toprow > (int)items->GetCount())
+   if (toprow < 0 || toprow > (int)items.size())
    {
       return;
    }
 
-   ExtImportItem *item = &(*items)[toprow];
+   ExtImportItem *item = items[toprow].get();
    PluginList->DeleteAllItems();
 
    int fcount;
@@ -451,8 +480,8 @@ void ExtImportPrefs::OnRuleTableEdit (wxGridEvent& event)
 {
    int row = event.GetRow();
    int col = event.GetCol();
-   ExtImportItems *items = wxGetApp().mImporter->GetImportItems();
-   ExtImportItem *item = &(*items)[row];
+   auto &items = Importer::Get().GetImportItems();
+   ExtImportItem *item = items[row].get();
    RuleTable->SaveEditControlValue();
 
    wxString val = RuleTable->GetCellValue (row, col);
@@ -460,7 +489,7 @@ void ExtImportPrefs::OnRuleTableEdit (wxGridEvent& event)
    bool askedAboutSpaces = false;
    wxArrayString vals;
    wxString delims(wxT(":"));
-   wxGetApp().mImporter->StringToList (val, delims, vals);
+   Importer::Get().StringToList (val, delims, vals);
    switch (col)
    {
    case 0:
@@ -480,7 +509,7 @@ void ExtImportPrefs::OnRuleTableEdit (wxGridEvent& event)
       {
          if (!askedAboutSpaces)
          {
-            fixSpaces = wxMessageBox(_(
+            fixSpaces = AudacityMessageBox(_(
 "There are space characters (spaces, newlines, tabs or linefeeds) in one of \
 the items. They are likely to break the pattern matching. Unless you know \
 what you are doing, it is recommended to trim spaces. Do you want \
@@ -522,7 +551,7 @@ Audacity to trim spaces for you?"
    RuleTable->AutoSizeColumns ();
 }
 
-void ExtImportPrefs::AddItemToTable (int index, ExtImportItem *item)
+void ExtImportPrefs::AddItemToTable (int index, const ExtImportItem *item)
 {
    wxString extensions, mime_types;
    if (item->extensions.Count() > 0)
@@ -554,11 +583,12 @@ void ExtImportPrefs::AddItemToTable (int index, ExtImportItem *item)
 
 void ExtImportPrefs::OnAddRule(wxCommandEvent& WXUNUSED(event))
 {
-   ExtImportItems *items = wxGetApp().mImporter->GetImportItems();
-   ExtImportItem *item = wxGetApp().mImporter->CreateDefaultImportItem();
-   items->Add (item);
-
+   auto &items = Importer::Get().GetImportItems();
+   auto uitem = Importer::Get().CreateDefaultImportItem();
+   auto item = uitem.get();
+   items.push_back(std::move(uitem));
    AddItemToTable (RuleTable->GetNumberRows (), item);
+
    RuleTable->SelectRow(RuleTable->GetNumberRows () - 1);
    RuleTable->SetGridCursor (RuleTable->GetNumberRows () - 1, 0);
    RuleTable->SetFocus();
@@ -568,15 +598,16 @@ void ExtImportPrefs::OnDelRule(wxCommandEvent& WXUNUSED(event))
 {
    if (last_selected < 0)
       return;
-   ExtImportItems *items = wxGetApp().mImporter->GetImportItems();
+   auto &items = Importer::Get().GetImportItems();
 
-   int msgres = wxMessageBox (_("Do you really want to delete selected rule?"),
+   int msgres = AudacityMessageBox (_("Do you really want to delete selected rule?"),
       _("Rule deletion confirmation"), wxYES_NO, RuleTable);
-   if (msgres == wxNO || msgres != wxYES)
+   // Yes or no, there is no third!
+   if (msgres != wxYES)
       return;
 
    RuleTable->DeleteRows (last_selected);
-   items->RemoveAt (last_selected);
+   items.erase (items.begin() + last_selected);
    RuleTable->AutoSizeColumns ();
    if (last_selected >= RuleTable->GetNumberRows ())
       last_selected = RuleTable->GetNumberRows () - 1;
@@ -628,7 +659,7 @@ void ExtImportPrefs::OnRuleTableCellClick (wxGridEvent& event)
    dragtext1->SetText(wxT(""));
    dragSource.SetData(*dragtext1);
    mDragFocus = RuleTable;
-   wxDragResult result = dragSource.DoDragDrop(TRUE);
+   wxDragResult result = dragSource.DoDragDrop(wxDrag_DefaultMove);
    mDragFocus = NULL;
    switch (result)
    {
@@ -643,9 +674,14 @@ void ExtImportPrefs::OnRuleTableCellClick (wxGridEvent& event)
    event.Skip();
 }
 
-ExtImportPrefsDropTarget::ExtImportPrefsDropTarget (wxDataObject *dataObject)
+wxString ExtImportPrefs::HelpPageName()
+{
+   return "Extended_Import_Preferences";
+}
+
+ExtImportPrefsDropTarget::ExtImportPrefsDropTarget(wxDataObject *dataObject)
+   : wxDropTarget(dataObject)
 {
-   SetDataObject (dataObject);
    mPrefs = NULL;
 }
 
@@ -695,8 +731,8 @@ bool ExtImportPrefsDropTarget::OnDrop(wxCoord x, wxCoord y)
    Grid *RuleTable = mPrefs->GetRuleTable();
    if (mPrefs->GetDragFocus() == RuleTable)
    {
-      if (RuleTable->YToRow (y -
-            RuleTable->GetColLabelSize ()) == wxNOT_FOUND)
+      if (RuleTable->YToRow(
+            RuleTable->CalcUnscrolledPosition(wxPoint(x, y)).y) == wxNOT_FOUND)
          return false;
    }
    else if (mPrefs->GetDragFocus() == PluginList)
@@ -730,8 +766,7 @@ wxDragResult ExtImportPrefsDropTarget::OnDragOver(wxCoord x, wxCoord y,
    if (mPrefs->GetDragFocus() == RuleTable)
    {
       int row;
-      row = RuleTable->YToRow (y -
-         RuleTable->GetColLabelSize ());
+      row = RuleTable->YToRow(RuleTable->CalcUnscrolledPosition(wxPoint(x, y)).y);
       if (row == wxNOT_FOUND)
          return wxDragNone;
 
@@ -769,14 +804,15 @@ wxDragResult ExtImportPrefsDropTarget::OnDragOver(wxCoord x, wxCoord y,
                wxLIST_STATE_SELECTED);
       }
    }
-   return wxDragCopy;
+   return wxDragMove;
 }
 
 void ExtImportPrefsDropTarget::OnLeave()
 {
 }
 
-void ExtImportPrefsDropTarget::SetDataObject(wxDataObject* data)
+PrefsPanel *ExtImportPrefsFactory::Create(wxWindow *parent)
 {
-   this->m_dataObject = data;
+   wxASSERT(parent); // to justify safenew
+   return safenew ExtImportPrefs(parent);
 }
