@@ -15,6 +15,7 @@
 
 
 #include "../Audacity.h"
+#include "DeviceToolBar.h"
 
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
@@ -29,10 +30,8 @@
 #include <wx/tooltip.h>
 #endif
 
-#include "../AudacityApp.h"
-
-#include "DeviceToolBar.h"
 #include "ToolDock.h"
+#include "../TrackPanel.h"
 
 #include "../AColor.h"
 #include "../AllThemeResources.h"
@@ -40,9 +39,11 @@
 #include "../ImageManipulation.h"
 #include "../Prefs.h"
 #include "../Project.h"
+#include "../ShuttleGui.h"
 #include "../Theme.h"
 #include "../widgets/Grabber.h"
 #include "../DeviceManager.h"
+#include "../widgets/ErrorDialog.h"
 
 IMPLEMENT_CLASS(DeviceToolBar, ToolBar);
 
@@ -63,8 +64,6 @@ DeviceToolBar::DeviceToolBar()
 
 DeviceToolBar::~DeviceToolBar()
 {
-   delete mPlayBitmap;
-   delete mRecordBitmap;
 }
 
 void DeviceToolBar::Create(wxWindow *parent)
@@ -72,19 +71,13 @@ void DeviceToolBar::Create(wxWindow *parent)
    ToolBar::Create(parent);
 
    // Simulate a size event to set initial meter placement/size
-   wxSizeEvent dummy;
-   OnSize(dummy);
-}
-
-void DeviceToolBar::RecreateTipWindows()
-{
+   wxSizeEvent event(GetSize(), GetId());
+   event.SetEventObject(this);
+   GetEventHandler()->ProcessEvent(event);
 }
 
 void DeviceToolBar::DeinitChildren()
 {
-   mPlayBitmap    = NULL;
-   mRecordBitmap  = NULL;
-
    mInput         = NULL;
    mOutput        = NULL;
    mInputChannels = NULL;
@@ -93,51 +86,44 @@ void DeviceToolBar::DeinitChildren()
 
 void DeviceToolBar::Populate()
 {
+   SetBackgroundColour( theTheme.Colour( clrMedium  ) );
    DeinitChildren();
    // Hosts
-   mHost = new wxChoice(this,
+   mHost = safenew wxChoice(this,
                         wxID_ANY,
                         wxDefaultPosition,
                         wxDefaultSize);
-   mHost->SetName(_("Audio Host"));
 
    Add(mHost, 0, wxALIGN_CENTER);
 
-   // Output device
-   mPlayBitmap = new wxBitmap(theTheme.Bitmap(bmpSpeaker));
-   Add(new wxStaticBitmap(this,
+   // Input device
+   Add(safenew wxStaticBitmap(this,
                           wxID_ANY,
-                          *mPlayBitmap), 0, wxALIGN_CENTER);
+                          theTheme.Bitmap(bmpMic)), 0, wxALIGN_CENTER);
+   mInput = safenew wxChoice(this,
+                         wxID_ANY,
+                         wxDefaultPosition,
+                         wxDefaultSize);
+   // Input channels
+   Add(mInput, 0, wxALIGN_CENTER);
+   mInputChannels = safenew wxChoice(this,
+                         wxID_ANY,
+                         wxDefaultPosition,
+                         wxDefaultSize);
+   Add(mInputChannels, 0, wxALIGN_CENTER);
 
-   mOutput = new wxChoice(this,
+   // Output device
+   Add(safenew wxStaticBitmap(this,
+                          wxID_ANY,
+                          theTheme.Bitmap(bmpSpeaker)), 0, wxALIGN_CENTER);
+   mOutput = safenew wxChoice(this,
                                wxID_ANY,
                                wxDefaultPosition,
                                wxDefaultSize);
-   /* i18n-hint: (noun) It's the device used for playback.*/
-   mOutput->SetName(_("Playback Device"));
    Add(mOutput, 0, wxALIGN_CENTER);
 
-   // Input device
-   mRecordBitmap = new wxBitmap(theTheme.Bitmap(bmpMic));
 
-   Add(new wxStaticBitmap(this,
-                          wxID_ANY,
-                          *mRecordBitmap), 0, wxALIGN_CENTER);
 
-   mInput = new wxChoice(this,
-                         wxID_ANY,
-                         wxDefaultPosition,
-                         wxDefaultSize);
-   /* i18n-hint: (noun) It's the device used for recording.*/
-   mInput->SetName(_("Recording Device"));
-   Add(mInput, 0, wxALIGN_CENTER);
-
-   mInputChannels = new wxChoice(this,
-                         wxID_ANY,
-                         wxDefaultPosition,
-                         wxDefaultSize);
-   mInputChannels->SetName(_("Recording Channels"));
-   Add(mInputChannels, 0, wxALIGN_CENTER);
 
    mHost->Connect(wxEVT_SET_FOCUS,
                  wxFocusEventHandler(DeviceToolBar::OnFocus),
@@ -172,6 +158,8 @@ void DeviceToolBar::Populate()
                  NULL,
                  this);
 
+   SetNames();
+
    RefillCombos();
 }
 
@@ -186,13 +174,12 @@ void DeviceToolBar::RefillCombos()
 
 void DeviceToolBar::OnFocus(wxFocusEvent &event)
 {
-   wxCommandEvent e(EVT_CAPTURE_KEYBOARD);
-
    if (event.GetEventType() == wxEVT_KILL_FOCUS) {
-      e.SetEventType(EVT_RELEASE_KEYBOARD);
+      AudacityProject::ReleaseKeyboard(this);
+   }
+   else {
+      AudacityProject::CaptureKeyboard(this);
    }
-   e.SetEventObject(this);
-   GetParent()->GetEventHandler()->ProcessEvent(e);
 
    Refresh(false);
 
@@ -205,15 +192,15 @@ void DeviceToolBar::OnCaptureKey(wxCommandEvent &event)
    int keyCode = kevent->GetKeyCode();
 
    // Pass UP/DOWN/LEFT/RIGHT through for input/output choice
-   if (FindFocus() == mOutput && (keyCode == WXK_LEFT || keyCode == WXK_RIGHT
-                                 || keyCode == WXK_UP || keyCode == WXK_DOWN)) {
-      return;
-   }
    if (FindFocus() == mInput && (keyCode == WXK_LEFT || keyCode == WXK_RIGHT
                                  || keyCode == WXK_UP || keyCode == WXK_DOWN)) {
       return;
    }
 
+   if (FindFocus() == mOutput && (keyCode == WXK_LEFT || keyCode == WXK_RIGHT
+                                 || keyCode == WXK_UP || keyCode == WXK_DOWN)) {
+      return;
+   }
    event.Skip();
 
    return;
@@ -244,7 +231,7 @@ void DeviceToolBar::UpdatePrefs()
    if (sourceName == wxT(""))
       desc = devName;
    else
-      desc = devName + wxString(": ", wxConvLocal) + sourceName;
+      desc = devName + wxT(": ") + sourceName;
 
    if (mInput->GetStringSelection() != desc &&
        mInput->FindString(desc) != wxNOT_FOUND) {
@@ -274,7 +261,7 @@ void DeviceToolBar::UpdatePrefs()
    if (sourceName == wxT(""))
       desc = devName;
    else
-      desc = devName + wxString(": ", wxConvLocal) + sourceName;
+      desc = devName + wxT(": ") + sourceName;
 
    if (mOutput->GetStringSelection() != desc &&
        mOutput->FindString(desc) != wxNOT_FOUND) {
@@ -299,7 +286,7 @@ void DeviceToolBar::UpdatePrefs()
       }
    }
 
-   long oldChannels = 1, newChannels;
+   long oldChannels, newChannels;
    oldChannels = mInputChannels->GetSelection() + 1;
    gPrefs->Read(wxT("/AudioIO/RecordChannels"), &newChannels, 0);
    if (newChannels > 0 && oldChannels != newChannels)
@@ -345,9 +332,20 @@ void DeviceToolBar::EnableDisableButtons()
    }
 }
 
+void DeviceToolBar::SetNames()
+{
+   /* i18n-hint: (noun) It's the device used for playback.*/
+   mOutput->SetName(_("Playback Device"));
+   /* i18n-hint: (noun) It's the device used for recording.*/
+   mInput->SetName(_("Recording Device"));
+   mHost->SetName(_("Audio Host"));
+   mInputChannels->SetName(_("Recording Channels"));
+}
+
 void DeviceToolBar::RegenerateTooltips()
 {
 #if wxUSE_TOOLTIPS
+   SetNames();
    mOutput->SetToolTip(mOutput->GetName() + wxT(" - ") + mOutput->GetStringSelection());
    mInput->SetToolTip(mInput->GetName() + wxT(" - ") + mInput->GetStringSelection());
    mHost->SetToolTip(mHost->GetName() + wxT(" - ") + mHost->GetStringSelection());
@@ -396,7 +394,6 @@ static bool RepositionCombo(wxWindow *combo, int toolbarWidth, wxSize desiredSiz
    desiredSize.x -= marginPixels;
 
    combo->SetMinSize(desiredSize);
-   combo->SetMaxSize(desiredSize);
 
    return constrained;
 }
@@ -438,20 +435,26 @@ void DeviceToolBar::RepositionCombos()
       return;
 
    // set up initial sizes and ratios
+   // Note that the y values of the desired sizes are not changed, so that the height
+   // of the toolbar is not changed
    hostRatio     = kHostWidthRatio;
    inputRatio    = kInputWidthRatio;
    outputRatio   = kOutputWidthRatio;
    channelsRatio = kChannelsWidthRatio;
 
-   desiredHost     = mHost->GetBestSize();
-   desiredInput    = mInput->GetBestSize();
-   desiredOutput   = mOutput->GetBestSize();
-   desiredChannels = mInputChannels->GetBestSize();
+   desiredHost.x     = mHost->GetBestSize().x;
+   desiredHost.y     = mHost->GetSize().y;
+   desiredInput.x    = mInput->GetBestSize().x;
+   desiredInput.y    = mInput->GetSize().y;
+   desiredOutput.x   = mOutput->GetBestSize().x;
+   desiredOutput.y   = mOutput->GetSize().y;
+   desiredChannels.x = mInputChannels->GetBestSize().x;
+   desiredChannels.y = mInputChannels->GetSize().y;
 
-   // wxGtk has larger comboboxes than the other platforms.  For DeviceToolBar this will cause
-   // the height to be double because of the discrete grid layout.  So we shrink it to prevent this.
+   // wxGtk (Gnome) has larger comboboxes than the other platforms.  For DeviceToolBar this prevents
+   // the toolbar docking on a single height row. So we shrink it to prevent this.
 #ifdef __WXGTK__
-   desiredHost.SetHeight(desiredHost.GetHeight() -4);
+   desiredHost.SetHeight(mHost->GetBestSize().y -4);
    desiredInput.SetHeight(desiredHost.GetHeight());
    desiredOutput.SetHeight(desiredHost.GetHeight());
    desiredChannels.SetHeight(desiredHost.GetHeight());
@@ -463,8 +466,8 @@ void DeviceToolBar::RepositionCombos()
    while (constrained && ratioUnused > 0.01f && i < 5) {
       i++;
       constrained = RepositionCombo(mHost,   w,   desiredHost,   hostRatio, ratioUnused, 0, true);
-      constrained |= RepositionCombo(mInput,  w,  desiredInput,  inputRatio, ratioUnused, mRecordBitmap->GetWidth(), true);
-      constrained |= RepositionCombo(mOutput, w, desiredOutput, outputRatio, ratioUnused, mPlayBitmap->GetWidth(), true);
+      constrained |= RepositionCombo(mInput,  w,  desiredInput,  inputRatio, ratioUnused, theTheme.Bitmap(bmpMic).GetWidth(), true);
+      constrained |= RepositionCombo(mOutput, w, desiredOutput, outputRatio, ratioUnused, theTheme.Bitmap(bmpSpeaker).GetWidth(), true);
       constrained |= RepositionCombo(mInputChannels, w, desiredChannels, channelsRatio, ratioUnused, 0, true);
    }
 
@@ -491,6 +494,9 @@ void DeviceToolBar::FillHosts()
 
    if (hosts.GetCount() == 0)
       mHost->Enable(false);
+
+   mHost->InvalidateBestSize();
+   mHost->SetMaxSize(mHost->GetBestSize());
 }
 
 void DeviceToolBar::FillHostDevices()
@@ -532,12 +538,15 @@ void DeviceToolBar::FillHostDevices()
          foundHostIndex = inMaps[0].hostIndex;
    }
 
+   // Make sure in/out are clear in case no host was found
+   mInput->Clear();
+   mOutput->Clear();
+
    // If we still have no host it means no devices, in which case do nothing.
    if (foundHostIndex == -1)
       return;
 
    // Repopulate the Input/Output device list available to the user
-   mInput->Clear();
    for (i = 0; i < inMaps.size(); i++) {
       if (foundHostIndex == inMaps[i].hostIndex) {
          mInput->Append(MakeDeviceSourceString(&inMaps[i]));
@@ -550,7 +559,9 @@ void DeviceToolBar::FillHostDevices()
    }
    mInput->Enable(mInput->GetCount() ? true : false);
 
-   mOutput->Clear();
+   mInput->InvalidateBestSize();
+   mInput->SetMaxSize(mInput->GetBestSize());
+
    for (i = 0; i < outMaps.size(); i++) {
       if (foundHostIndex == outMaps[i].hostIndex) {
          mOutput->Append(MakeDeviceSourceString(&outMaps[i]));
@@ -564,6 +575,9 @@ void DeviceToolBar::FillHostDevices()
    }
    mOutput->Enable(mOutput->GetCount() ? true : false);
 
+   mOutput->InvalidateBestSize();
+   mOutput->SetMaxSize(mOutput->GetBestSize());
+
    // The setting of the Device is left up to OnChoice
 }
 
@@ -618,7 +632,7 @@ void DeviceToolBar::FillInputChannels()
                name = _("2 (Stereo) Recording Channels");
             }
             else {
-               name = wxString::Format(wxT("%d"), j + 1);
+               name = wxString::Format(wxT("%d"), (int) j + 1);
             }
             mInputChannels->Append(name);
          }
@@ -635,7 +649,11 @@ void DeviceToolBar::FillInputChannels()
    }
    if (index == -1)
       mInputChannels->Enable(false);
+
+   mInputChannels->InvalidateBestSize();
+   mInputChannels->SetMaxSize(mInputChannels->GetBestSize());
 }
+
 void DeviceToolBar::SetDevices(const DeviceSourceMap *in, const DeviceSourceMap *out)
 {
    if (in) {
@@ -735,7 +753,7 @@ void DeviceToolBar::OnChoice(wxCommandEvent &event)
    }
 
    // Update all projects' DeviceToolBar.
-   for (size_t i = 0; i < gAudacityProjects.GetCount(); i++) {
+   for (size_t i = 0; i < gAudacityProjects.size(); i++) {
       gAudacityProjects[i]->GetDeviceToolBar()->UpdatePrefs();
    }
 }
@@ -760,14 +778,15 @@ void DeviceToolBar::ShowChannelsDialog()
 void DeviceToolBar::ShowComboDialog(wxChoice *combo, const wxString &title)
 {
    if (!combo || combo->GetCount() == 0) {
-      wxMessageBox(_("Device information is not available."));
+      AudacityMessageBox(_("Device information is not available."));
       return;
    }
 
 #if USE_PORTMIXER
    wxArrayString inputSources = combo->GetStrings();
 
-   wxDialog dlg(NULL, wxID_ANY, title);
+   wxDialogWrapper dlg(nullptr, wxID_ANY, title);
+   dlg.SetName(dlg.GetTitle());
    ShuttleGui S(&dlg, eIsCreating);
    wxChoice *c;
 
@@ -780,11 +799,11 @@ void DeviceToolBar::ShowComboDialog(wxChoice *combo, const wxString &title)
                          &inputSources);
       }
       S.EndHorizontalLay();
-      S.AddStandardButtons();
    }
    S.EndVerticalLay();
+   S.AddStandardButtons();
 
-   dlg.SetSize(dlg.GetSizer()->GetMinSize());
+   dlg.GetSizer()->SetSizeHints(&dlg);
    dlg.Center();
 
    if (dlg.ShowModal() == wxID_OK)
