@@ -11,6 +11,7 @@
 #ifndef __AUDACITY_MIXER_TOOLBAR__
 #define __AUDACITY_MIXER_TOOLBAR__
 
+#include "../MemoryX.h"
 #include "ToolBar.h"
 
 class wxImage;
@@ -21,23 +22,22 @@ class wxStaticBitmap;
 
 class ASlider;
 
-class MixerToolBar:public ToolBar {
+class MixerToolBar final : public ToolBar {
 
  public:
 
    MixerToolBar();
    virtual ~MixerToolBar();
 
-   void Create(wxWindow * parent);
+   void Create(wxWindow * parent) override;
 
-   void RecreateTipWindows();
-   void UpdatePrefs();
+   void UpdatePrefs() override;
    void UpdateControls();
    void SetMixer(wxCommandEvent &event);
 
-   virtual void Populate();
-   virtual void Repaint(wxDC * WXUNUSED(dc)) {};
-   virtual void EnableDisableButtons() {};
+   void Populate() override;
+   void Repaint(wxDC * WXUNUSED(dc)) override {};
+   void EnableDisableButtons() override {};
 
    void OnFocus(wxFocusEvent &event);
    void OnCaptureKey(wxCommandEvent &event);
@@ -50,6 +50,8 @@ class MixerToolBar:public ToolBar {
    void AdjustOutputGain(int adj);
    void AdjustInputGain(int adj);
 
+   void RegenerateTooltips() override {};
+
  protected:
    float mInputSliderVolume;
    float mOutputSliderVolume;
@@ -59,16 +61,13 @@ class MixerToolBar:public ToolBar {
    void InitializeMixerToolBar();
    void SetToolTips();
 
-   wxBitmap *mPlayBitmap;
-   wxBitmap *mRecordBitmap;
-
    ASlider *mInputSlider;
    ASlider *mOutputSlider;
 
  public:
 
-   DECLARE_CLASS(MixerToolBar);
-   DECLARE_EVENT_TABLE();
+   DECLARE_CLASS(MixerToolBar)
+   DECLARE_EVENT_TABLE()
 };
 
 #endif
