@@ -15,6 +15,7 @@
 *//*******************************************************************/
 
 #include "../Audacity.h"
+#include "TranscriptionToolBar.h"
 
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
@@ -27,21 +28,23 @@
 #include <wx/intl.h>
 #endif // WX_PRECOMP
 
-#include "TranscriptionToolBar.h"
+#include "../Envelope.h"
 
 #include "ControlToolBar.h"
-#include "../AudacityApp.h"
 #include "../AllThemeResources.h"
 #include "../AudioIO.h"
 #include "../Experimental.h"
 #include "../ImageManipulation.h"
 #include "../Project.h"
 #include "../TimeTrack.h"
-#include "../VoiceKey.h"
 #include "../WaveTrack.h"
 #include "../widgets/AButton.h"
 #include "../widgets/ASlider.h"
 
+#ifdef EXPERIMENTAL_VOICE_DETECTION
+#include "../VoiceKey.h"
+#endif
+
 IMPLEMENT_CLASS(TranscriptionToolBar, ToolBar);
 
 ///////////////////////////////////////////
@@ -51,9 +54,13 @@ IMPLEMENT_CLASS(TranscriptionToolBar, ToolBar);
 
 BEGIN_EVENT_TABLE(TranscriptionToolBar, ToolBar)
    EVT_CHAR(TranscriptionToolBar::OnKeyEvent)
+   EVT_COMMAND(wxID_ANY, EVT_CAPTURE_KEY, TranscriptionToolBar::OnCaptureKey)
+
    EVT_COMMAND_RANGE(TTB_PlaySpeed, TTB_PlaySpeed,
                      wxEVT_COMMAND_BUTTON_CLICKED, TranscriptionToolBar::OnPlaySpeed)
    EVT_SLIDER(TTB_PlaySpeedSlider, TranscriptionToolBar::OnSpeedSlider)
+
+#ifdef EXPERIMENTAL_VOICE_DETECTION
    EVT_COMMAND_RANGE(TTB_StartOn, TTB_StartOn,
                      wxEVT_COMMAND_BUTTON_CLICKED, TranscriptionToolBar::OnStartOn)
    EVT_COMMAND_RANGE(TTB_StartOff, TTB_StartOff,
@@ -75,7 +82,7 @@ BEGIN_EVENT_TABLE(TranscriptionToolBar, ToolBar)
    EVT_SLIDER(TTB_SensitivitySlider, TranscriptionToolBar::OnSensitivitySlider)
 
    EVT_CHOICE(TTB_KeyType, TranscriptionToolBar::SetKeyType)
-   EVT_COMMAND(wxID_ANY, EVT_CAPTURE_KEY, TranscriptionToolBar::OnCaptureKey)
+#endif
 END_EVENT_TABLE()
    ;   //semicolon enforces  proper automatic indenting in emacs.
 
@@ -83,22 +90,14 @@ END_EVENT_TABLE()
 TranscriptionToolBar::TranscriptionToolBar()
 : ToolBar(TranscriptionBarID, _("Transcription"), wxT("Transcription"))
 {
-   mPlaySpeed = 1.0;
-   mTimeTrack = NULL;
+   mPlaySpeed = 1.0 * 100.0;
 #ifdef EXPERIMENTAL_VOICE_DETECTION
-   mVk = new VoiceKey();
+   mVk = std::make_unique<VoiceKey>();
 #endif
 }
 
 TranscriptionToolBar::~TranscriptionToolBar()
 {
-#ifdef EXPERIMENTAL_VOICE_DETECTION
-   delete mVk;
-#endif
-   if (mTimeTrack) {
-      delete mTimeTrack;
-      mTimeTrack = NULL;
-   }
 }
 
 void TranscriptionToolBar::Create(wxWindow * parent)
@@ -144,8 +143,8 @@ AButton *TranscriptionToolBar::AddButton(
 {
    AButton *&r = mButtons[id];
 
-   r = ToolBar::MakeButton(
-      bmpRecoloredUpSmall, bmpRecoloredDownSmall, bmpRecoloredHiliteSmall,
+   r = ToolBar::MakeButton(this,
+      bmpRecoloredUpSmall, bmpRecoloredDownSmall, bmpRecoloredUpHiliteSmall,bmpRecoloredHiliteSmall,
       eFore, eFore, eDisabled,
       wxWindowID(id),
       wxDefaultPosition,
@@ -161,24 +160,38 @@ AButton *TranscriptionToolBar::AddButton(
    return r;
 }
 
+void TranscriptionToolBar::MakeAlternateImages(
+   teBmps eFore, teBmps eDisabled,
+   int id, unsigned altIdx)
+{
+   ToolBar::MakeAlternateImages(*mButtons[id], altIdx,
+      bmpRecoloredUpSmall, bmpRecoloredDownSmall, bmpRecoloredUpHiliteSmall,bmpRecoloredHiliteSmall,
+      eFore, eFore, eDisabled,
+      theTheme.ImageSize( bmpRecoloredUpSmall ));
+}
+
 void TranscriptionToolBar::Populate()
 {
+   SetBackgroundColour( theTheme.Colour( clrMedium  ) );
 // Very similar to code in ControlToolBar...
 // Very similar to code in EditToolBar
    MakeButtonBackgroundsSmall();
 
    AddButton(bmpPlay,     bmpPlayDisabled,   TTB_PlaySpeed,
       _("Play at selected speed"));
+   MakeAlternateImages(bmpLoop, bmpLoopDisabled, TTB_PlaySpeed, 1);
+   MakeAlternateImages(bmpCutPreview, bmpCutPreviewDisabled, TTB_PlaySpeed, 2);
+   mButtons[TTB_PlaySpeed]->FollowModifierKeys();
 
    //Add a slider that controls the speed of playback.
    const int SliderWidth=100;
-   mPlaySpeedSlider = new ASlider(this,
+   mPlaySpeedSlider = safenew ASlider(this,
                                   TTB_PlaySpeedSlider,
                                   _("Playback Speed"),
                                   wxDefaultPosition,
                                   wxSize(SliderWidth,25),
                                   SPEED_SLIDER);
-   mPlaySpeedSlider->Set(1.0);
+   mPlaySpeedSlider->Set(mPlaySpeed / 100.0);
    mPlaySpeedSlider->SetLabel(_("Playback Speed"));
    //  6 steps using page up/down, and 60 using arrow keys
    mPlaySpeedSlider->SetScroll(0.16667f, 1.6667f);
@@ -215,7 +228,7 @@ void TranscriptionToolBar::Populate()
    AddButton(bmpTnCalibrate, bmpTnCalibrateDisabled, TTB_Calibrate,
       TRANSLATABLE("Calibrate voicekey"));
 
-   mSensitivitySlider = new ASlider(this,
+   mSensitivitySlider = safenew ASlider(this,
                                     TTB_SensitivitySlider,
                                     TRANSLATABLE("Adjust Sensitivity"),
                                     wxDefaultPosition,
@@ -234,7 +247,7 @@ void TranscriptionToolBar::Populate()
       TRANSLATABLE("Direction Changes (High Threshold)")
    };
 
-   mKeyTypeChoice = new wxChoice(this, TTB_KeyType,
+   mKeyTypeChoice = safenew wxChoice(this, TTB_KeyType,
                                  wxDefaultPosition,
                                  wxDefaultSize,
                                  5,
@@ -250,6 +263,25 @@ void TranscriptionToolBar::Populate()
    UpdatePrefs();
 }
 
+void TranscriptionToolBar::EnableDisableButtons()
+{
+#ifdef EXPERIMENTAL_VOICE_DETECTION
+   AudacityProject *p = GetActiveProject();
+   if (!p) return;
+   // Is anything selected?
+   bool selection = false;
+   TrackListIterator iter(p->GetTracks());
+   for (Track *t = iter.First(); t; t = iter.Next())
+      if (t->GetSelected()) {
+         selection = true;
+         break;
+      }
+   selection &= (p->GetSel0() < p->GetSel1());
+
+   mButtons[TTB_Calibrate]->SetEnabled(selection);
+#endif
+}
+
 void TranscriptionToolBar::UpdatePrefs()
 {
    RegenerateTooltips();
@@ -263,9 +295,33 @@ void TranscriptionToolBar::UpdatePrefs()
 
 void TranscriptionToolBar::RegenerateTooltips()
 {
-#if wxUSE_TOOLTIPS
-   mButtons[TTB_PlaySpeed]->SetToolTip(_("Play-at-speed"));
-   mPlaySpeedSlider->SetToolTip(_("Playback Speed"));
+   // We could also mention the shift- and ctrl-modified versions in the
+   // tool tip... but it would get long
+
+   static const struct Entry {
+      int tool;
+      wxString commandName;
+      wxString untranslatedLabel;
+      wxString commandName2;
+      wxString untranslatedLabel2;
+   } table[] = {
+      { TTB_PlaySpeed,   wxT("PlayAtSpeed"),    XO("Play-at-Speed"),
+      wxT("PlayAtSpeedLooped"),    XO("Looped-Play-at-Speed")
+      },
+   };
+
+   LocalizedCommandNameVector commands;
+   for (const auto &entry : table) {
+      commands.clear();
+      commands.push_back( LocalizedCommandName(
+         wxGetTranslation(entry.untranslatedLabel), entry.commandName
+      ) );
+      commands.push_back( LocalizedCommandName(
+         wxGetTranslation(entry.untranslatedLabel2), entry.commandName2
+      ) );
+      ToolBar::SetButtonToolTip(*mButtons[entry.tool], commands);
+   }
+
 
 #ifdef EXPERIMENTAL_VOICE_DETECTION
    mButtons[TTB_StartOn]->SetToolTip(TRANSLATABLE("Left-to-On"));
@@ -281,18 +337,16 @@ void TranscriptionToolBar::RegenerateTooltips()
    mSensitivitySlider->SetToolTip(TRANSLATABLE("Sensitivity"));
    mKeyTypeChoice->SetToolTip(TRANSLATABLE("Key type"));
 #endif
-#endif
 }
 
 void TranscriptionToolBar::OnFocus(wxFocusEvent &event)
 {
-   wxCommandEvent e(EVT_CAPTURE_KEYBOARD);
-
    if (event.GetEventType() == wxEVT_KILL_FOCUS) {
-      e.SetEventType(EVT_RELEASE_KEYBOARD);
+      AudacityProject::ReleaseKeyboard(this);
+   }
+   else {
+      AudacityProject::CaptureKeyboard(this);
    }
-   e.SetEventObject(this);
-   GetParent()->GetEventHandler()->ProcessEvent(e);
 
    Refresh(false);
 
@@ -347,7 +401,8 @@ void TranscriptionToolBar::SetButton(bool down, AButton* button)
    }
 }
 
-void TranscriptionToolBar::GetSamples(WaveTrack *t, sampleCount *s0, sampleCount *slen)
+void TranscriptionToolBar::GetSamples(
+   const WaveTrack *t, sampleCount *s0, sampleCount *slen)
 {
    // GetSamples attempts to translate the start and end selection markers into sample indices
    // These selection numbers are doubles.
@@ -363,8 +418,8 @@ void TranscriptionToolBar::GetSamples(WaveTrack *t, sampleCount *s0, sampleCount
    double start = p->GetSel0();
    double end = p->GetSel1();
 
-   sampleCount ss0 = sampleCount( (start - t->GetOffset()) * t->GetRate() );
-   sampleCount ss1 = sampleCount( (end - t->GetOffset()) * t->GetRate() );
+   auto ss0 = sampleCount( (start - t->GetOffset()) * t->GetRate() );
+   auto ss1 = sampleCount( (end - t->GetOffset()) * t->GetRate() );
 
    if (start < t->GetOffset()) {
       ss0 = 0;
@@ -385,7 +440,8 @@ void TranscriptionToolBar::GetSamples(WaveTrack *t, sampleCount *s0, sampleCount
    *slen = ss1 - ss0;
 }
 
-void TranscriptionToolBar::OnPlaySpeed(wxCommandEvent & WXUNUSED(event))
+// Come here from button clicks, or commands
+void TranscriptionToolBar::PlayAtSpeed(bool looped, bool cutPreview)
 {
    // Can't do anything without an active project
    AudacityProject * p = GetActiveProject();
@@ -395,7 +451,7 @@ void TranscriptionToolBar::OnPlaySpeed(wxCommandEvent & WXUNUSED(event))
 
    // Create a TimeTrack if we haven't done so already
    if (!mTimeTrack) {
-      mTimeTrack = new TimeTrack(p->GetDirManager());
+      mTimeTrack = p->GetTrackFactory()->NewTimeTrack();
       if (!mTimeTrack) {
          return;
       }
@@ -421,20 +477,37 @@ void TranscriptionToolBar::OnPlaySpeed(wxCommandEvent & WXUNUSED(event))
    // Start playing
    if (playRegionStart >= 0) {
 //      playRegionEnd = playRegionStart + (playRegionEnd-playRegionStart)* 100.0/mPlaySpeed;
-#ifdef EXPERIMENTAL_MIDI_OUT
-      gAudioIO->SetMidiPlaySpeed(mPlaySpeed);
-#endif
-      p->GetControlToolBar()->PlayPlayRegion(playRegionStart,
-                                             playRegionEnd,
-                                             false,
-                                             false,
-                                             mTimeTrack);
+      AudioIOStartStreamOptions options(p->GetDefaultPlayOptions());
+      options.playLooped = looped;
+      options.timeTrack = mTimeTrack.get();
+      ControlToolBar::PlayAppearance appearance =
+         cutPreview ? ControlToolBar::PlayAppearance::CutPreview
+         : looped ? ControlToolBar::PlayAppearance::Looped
+         : ControlToolBar::PlayAppearance::Straight;
+      p->GetControlToolBar()->PlayPlayRegion
+         (SelectedRegion(playRegionStart, playRegionEnd),
+          options,
+          PlayMode::normalPlay,
+          appearance);
    }
 }
 
+// Come here from button clicks only
+void TranscriptionToolBar::OnPlaySpeed(wxCommandEvent & WXUNUSED(event))
+{
+   auto button = mButtons[TTB_PlaySpeed];
+
+   // Let control have precedence over shift
+   const bool cutPreview = mButtons[TTB_PlaySpeed]->WasControlDown();
+   const bool looped = !cutPreview &&
+      button->WasShiftDown();
+   PlayAtSpeed(looped, cutPreview);
+}
+
 void TranscriptionToolBar::OnSpeedSlider(wxCommandEvent& WXUNUSED(event))
 {
    mPlaySpeed = (mPlaySpeedSlider->Get()) * 100;
+   RegenerateTooltips();
 
    // If IO is busy, abort immediately
    // AWD: This is disabled to work around a hang on Linux when PulseAudio is
@@ -444,6 +517,7 @@ void TranscriptionToolBar::OnSpeedSlider(wxCommandEvent& WXUNUSED(event))
    //}
 }
 
+#ifdef EXPERIMENTAL_VOICE_DETECTION
 void TranscriptionToolBar::OnStartOn(wxCommandEvent & WXUNUSED(event))
 {
    //If IO is busy, abort immediately
@@ -460,15 +534,16 @@ void TranscriptionToolBar::OnStartOn(wxCommandEvent & WXUNUSED(event))
 
    Track *t = iter.First();   //Make a track
    if(t ) {
-      sampleCount start,len;
-      GetSamples((WaveTrack*)t, &start,&len);
+      auto wt = static_cast<const WaveTrack*>(t);
+      sampleCount start, len;
+      GetSamples(wt, &start, &len);
 
       //Adjust length to end if selection is null
       //if(len == 0)
-      //len = (WaveTrack*)t->GetSequence()->GetNumSamples()-start;
+      //len = wt->GetSequence()->GetNumSamples()-start;
 
-      sampleCount newstart = mVk->OnForward(*(WaveTrack*)t,start,len);
-      double newpos = newstart / ((WaveTrack*)t)->GetRate();
+      auto newstart = mVk->OnForward(*wt, start, len);
+      double newpos = newstart.as_double() / wt->GetRate();
 
       p->SetSel0(newpos);
       p->RedrawProject();
@@ -493,15 +568,16 @@ void TranscriptionToolBar::OnStartOff(wxCommandEvent & WXUNUSED(event))
    SetButton(false, mButtons[TTB_StartOff]);
    Track *t = iter.First();   //Make a track
    if(t) {
-      sampleCount start,len;
-      GetSamples((WaveTrack*)t, &start,&len);
+      auto wt = static_cast<const WaveTrack*>(t);
+      sampleCount start, len;
+      GetSamples(wt, &start, &len);
 
       //Adjust length to end if selection is null
       //if(len == 0)
-      //len = (WaveTrack*)t->GetSequence()->GetNumSamples()-start;
+      //len = wt->GetSequence()->GetNumSamples()-start;
 
-      sampleCount newstart = mVk->OffForward(*(WaveTrack*)t,start,len);
-      double newpos = newstart / ((WaveTrack*)t)->GetRate();
+      auto newstart = mVk->OffForward(*wt, start, len);
+      double newpos = newstart.as_double() / wt->GetRate();
 
       p->SetSel0(newpos);
       p->RedrawProject();
@@ -526,8 +602,9 @@ void TranscriptionToolBar::OnEndOn(wxCommandEvent & WXUNUSED(event))
 
    Track *t = iter.First();   //Make a track
    if(t) {
-      sampleCount start,len;
-      GetSamples((WaveTrack*)t, &start,&len);
+      auto wt = static_cast<const WaveTrack*>(t);
+      sampleCount start, len;
+      GetSamples(wt, &start, &len);
 
       //Adjust length to end if selection is null
       if(len == 0)
@@ -535,8 +612,8 @@ void TranscriptionToolBar::OnEndOn(wxCommandEvent & WXUNUSED(event))
             len = start;
             start = 0;
          }
-      sampleCount newEnd = mVk->OnBackward(*(WaveTrack*)t,start+ len,len);
-      double newpos = newEnd / ((WaveTrack*)t)->GetRate();
+      auto newEnd = mVk->OnBackward(*wt, start + len, len);
+      double newpos = newEnd.as_double() / wt->GetRate();
 
       p->SetSel1(newpos);
       p->RedrawProject();
@@ -562,16 +639,17 @@ void TranscriptionToolBar::OnEndOff(wxCommandEvent & WXUNUSED(event))
 
    Track *t = iter.First();   //Make a track
    if(t) {
-      sampleCount start,len;
-      GetSamples((WaveTrack*)t, &start,&len);
+      auto wt = static_cast<const WaveTrack*>(t);
+      sampleCount start, len;
+      GetSamples(wt, &start, &len);
 
       //Adjust length to end if selection is null
       if(len == 0) {
          len = start;
          start = 0;
       }
-      sampleCount newEnd = mVk->OffBackward(*(WaveTrack*)t,start+ len,len);
-      double newpos = newEnd / ((WaveTrack*)t)->GetRate();
+      auto newEnd = mVk->OffBackward(*wt, start + len, len);
+      double newpos = newEnd.as_double() / wt->GetRate();
 
       p->SetSel1(newpos);
       p->RedrawProject();
@@ -597,25 +675,27 @@ void TranscriptionToolBar::OnSelectSound(wxCommandEvent & WXUNUSED(event))
 
 
    TrackList *tl = p->GetTracks();
-   TrackListIterator iter(tl);
+   TrackListOfKindIterator iter(Track::Wave, tl);
 
    Track *t = iter.First();   //Make a track
    if(t)
       {
-         sampleCount start,len;
-         GetSamples((WaveTrack*)t, &start,&len);
+         auto wt = static_cast<const WaveTrack*>(t);
+         sampleCount start, len;
+         GetSamples(wt, &start, &len);
 
          //Adjust length to end if selection is null
          //if(len == 0)
-         //len = (WaveTrack*)t->GetSequence()->GetNumSamples()-start;
+         //len = wt->GetSequence()->GetNumSamples()-start;
 
-         double rate =  ((WaveTrack*)t)->GetRate();
-         sampleCount newstart = mVk->OffBackward(*(WaveTrack*)t,start,start);
-         sampleCount newend   = mVk->OffForward(*(WaveTrack*)t,start+len,(int)(tl->GetEndTime()*rate));
+         double rate =  wt->GetRate();
+         auto newstart = mVk->OffBackward(*wt, start, start);
+         auto newend   =
+            mVk->OffForward(*wt, start + len, (int)(tl->GetEndTime() * rate));
 
          //reset the selection bounds.
-         p->SetSel0(newstart / rate);
-         p->SetSel1(newend /  rate);
+         p->SetSel0(newstart.as_double() / rate);
+         p->SetSel1(newend.as_double() /  rate);
          p->RedrawProject();
 
       }
@@ -637,24 +717,26 @@ void TranscriptionToolBar::OnSelectSilence(wxCommandEvent & WXUNUSED(event))
 
 
    TrackList *tl = p->GetTracks();
-   TrackListIterator iter(tl);
+   TrackListOfKindIterator iter(Track::Wave, tl);
 
    Track *t = iter.First();   //Make a track
    if(t)
       {
-         sampleCount start,len;
-         GetSamples((WaveTrack*)t, &start,&len);
+         auto wt = static_cast<const WaveTrack*>(t);
+         sampleCount start, len;
+         GetSamples(wt, &start, &len);
 
          //Adjust length to end if selection is null
          //if(len == 0)
-         //len = (WaveTrack*)t->GetSequence()->GetNumSamples()-start;
-         double rate =  ((WaveTrack*)t)->GetRate();
-         sampleCount newstart = mVk->OnBackward(*(WaveTrack*)t,start,start);
-         sampleCount newend   = mVk->OnForward(*(WaveTrack*)t,start+len,(int)(tl->GetEndTime()*rate));
+         //len = wt->GetSequence()->GetNumSamples()-start;
+         double rate =  wt->GetRate();
+         auto newstart = mVk->OnBackward(*wt, start, start);
+         auto newend   =
+            mVk->OnForward(*wt, start + len, (int)(tl->GetEndTime() * rate));
 
          //reset the selection bounds.
-         p->SetSel0(newstart /  rate);
-         p->SetSel1(newend / rate);
+         p->SetSel0(newstart.as_double() /  rate);
+         p->SetSel1(newend.as_double() / rate);
          p->RedrawProject();
 
       }
@@ -677,15 +759,16 @@ void TranscriptionToolBar::OnCalibrate(wxCommandEvent & WXUNUSED(event))
    AudacityProject *p = GetActiveProject();
 
    TrackList *tl = p->GetTracks();
-   TrackListIterator iter(tl);
+   TrackListOfKindIterator iter(Track::Wave, tl);
    Track *t = iter.First();   //Get a track
 
    if(t)
       {
-         sampleCount start,len;
-         GetSamples((WaveTrack*)t, &start,&len);
+         auto wt = static_cast<const WaveTrack*>(t);
+         sampleCount start, len;
+         GetSamples(wt, &start, &len);
 
-         mVk->CalibrateNoise(*((WaveTrack*)t),start,len);
+         mVk->CalibrateNoise(*wt, start, len);
          mVk->AdjustThreshold(3);
 
          mButtons[TTB_StartOn]->Enable();
@@ -730,13 +813,14 @@ void TranscriptionToolBar::OnAutomateSelection(wxCommandEvent & WXUNUSED(event))
    mVk->AdjustThreshold(GetSensitivity());
    AudacityProject *p = GetActiveProject();
    TrackList *tl = p->GetTracks();
-   TrackListIterator iter(tl);
+   TrackListOfKindIterator iter(Track::Wave, tl);
 
    Track *t = iter.First();   //Make a track
    if(t)
       {
-         sampleCount start,len;
-         GetSamples((WaveTrack*)t, &start,&len);
+         auto wt = static_cast<const WaveTrack*>(t);
+         sampleCount start, len;
+         GetSamples(wt, &start, &len);
 
          //Adjust length to end if selection is null
          if(len == 0)
@@ -744,13 +828,12 @@ void TranscriptionToolBar::OnAutomateSelection(wxCommandEvent & WXUNUSED(event))
                len = start;
                start = 0;
             }
-         int lastlen = 0;
-         sampleCount newStart, newEnd;
+         sampleCount lastlen = 0;
          double newStartPos, newEndPos;
 
 
          //This is the minumum word size in samples (.05 is 50 ms)
-         int minWordSize = (int)(((WaveTrack*)t)->GetRate() * .05);
+         int minWordSize = (int)(wt->GetRate() * .05);
 
          //Continue until we have processed the entire
          //region, or we are making no progress.
@@ -759,13 +842,13 @@ void TranscriptionToolBar::OnAutomateSelection(wxCommandEvent & WXUNUSED(event))
 
                lastlen = len;
 
-               newStart = mVk->OnForward(*(WaveTrack*)t,start,len);
+               auto newStart = mVk->OnForward(*wt, start, len);
 
                //JKC: If no start found then don't add any labels.
                if( newStart==start)
                   break;
 
-               //Adjust len by the new start position
+               //Adjust len by the NEW start position
                len -= (newStart - start);
 
                //Adjust len by the minimum word size
@@ -773,10 +856,11 @@ void TranscriptionToolBar::OnAutomateSelection(wxCommandEvent & WXUNUSED(event))
 
 
 
-               //OK, now we have found a new starting point.  A 'word' should be at least
+               //OK, now we have found a NEW starting point.  A 'word' should be at least
                //50 ms long, so jump ahead minWordSize
 
-               newEnd   = mVk->OffForward(*(WaveTrack*)t,newStart+minWordSize, len);
+               auto newEnd   =
+                  mVk->OffForward(*wt, newStart + minWordSize, len);
 
                //If newEnd didn't move, we should give up, because
                // there isn't another end before the end of the selection.
@@ -784,18 +868,18 @@ void TranscriptionToolBar::OnAutomateSelection(wxCommandEvent & WXUNUSED(event))
                   break;
 
 
-               //Adjust len by the new word end
+               //Adjust len by the NEW word end
                len -= (newEnd - newStart);
 
                //Calculate the start and end of the words, in seconds
-               newStartPos = newStart / ((WaveTrack*)t)->GetRate();
-               newEndPos = newEnd / ((WaveTrack*)t)->GetRate();
+               newStartPos = newStart.as_double() / wt->GetRate();
+               newEndPos = newEnd.as_double() / wt->GetRate();
 
 
                //Increment
                start = newEnd;
 
-               p->DoAddLabel(newStartPos, newEndPos);
+               p->DoAddLabel(SelectedRegion(newStartPos, newEndPos));
                p->RedrawProject();
             }
          SetButton(false, mButtons[TTB_AutomateSelection]);
@@ -806,7 +890,7 @@ void TranscriptionToolBar::OnMakeLabel(wxCommandEvent & WXUNUSED(event))
 {
    AudacityProject *p = GetActiveProject();
    SetButton(false, mButtons[TTB_MakeLabel]);
-   p->DoAddLabel(p->GetSel0(),  p->GetSel1());
+   p->DoAddLabel(SelectedRegion(p->GetSel0(),  p->GetSel1()));
 }
 
 //This returns a double z-score between 0 and 10.
@@ -820,26 +904,6 @@ void TranscriptionToolBar::OnSensitivitySlider(wxCommandEvent & WXUNUSED(event))
    mSensitivity = (mSensitivitySlider->Get());
 }
 
-void TranscriptionToolBar::EnableDisableButtons()
-{
-
-#ifdef EXPERIMENTAL_VOICE_DETECTION
-   AudacityProject *p = GetActiveProject();
-   if (!p) return;
-   // Is anything selected?
-   bool selection = false;
-   TrackListIterator iter(p->GetTracks());
-   for (Track *t = iter.First(); t; t = iter.Next())
-      if (t->GetSelected()) {
-         selection = true;
-         break;
-      }
-   selection &= (p->GetSel0() < p->GetSel1());
-
-   mButtons[TTB_Calibrate]->SetEnabled(selection);
-#endif
-}
-
 void TranscriptionToolBar::SetKeyType(wxCommandEvent & WXUNUSED(event))
 {
    int value = mKeyTypeChoice->GetSelection();
@@ -865,12 +929,7 @@ void TranscriptionToolBar::SetKeyType(wxCommandEvent & WXUNUSED(event))
       }
 
 }
-
-void TranscriptionToolBar::PlayAtSpeed()
-{
-   wxCommandEvent e;
-   OnPlaySpeed(e);
-}
+#endif
 
 void TranscriptionToolBar::ShowPlaySpeedDialog()
 {
@@ -880,6 +939,24 @@ void TranscriptionToolBar::ShowPlaySpeedDialog()
    OnSpeedSlider(e);
 }
 
+void TranscriptionToolBar::SetEnabled(bool enabled)
+{
+   mButtons[TTB_PlaySpeed]->SetEnabled(enabled);
+}
+
+void TranscriptionToolBar::SetPlaying(bool down, bool looped, bool cutPreview)
+{
+   AButton *const button = mButtons[TTB_PlaySpeed];
+   if (down) {
+      button->SetAlternateIdx(cutPreview ? 2 : looped ? 1 : 0);
+      button->PushDown();
+   }
+   else {
+      button->SetAlternateIdx(0);
+      button->PopUp();
+   }
+}
+
 void TranscriptionToolBar::AdjustPlaySpeed(float adj)
 {
    if (adj < 0) {
