@@ -26,10 +26,13 @@
 #include "AButton.h"
 #include "../AColor.h"
 
+#include <wx/app.h>
 #include <wx/dcclient.h>
 #include <wx/dcmemory.h>
 #include <wx/dcbuffer.h>
+#include <wx/eventfilter.h>
 #include <wx/image.h>
+#include <wx/timer.h>
 
 //This is needed for tooltips
 #include "../Project.h"
@@ -46,6 +49,68 @@ BEGIN_EVENT_TABLE(AButton, wxWindow)
    EVT_ERASE_BACKGROUND(AButton::OnErase)
 END_EVENT_TABLE()
 
+// LL:  An alternative to this might be to just use the wxEVT_KILL_FOCUS
+//      or wxEVT_ACTIVATE events.
+class AButton::Listener final
+   : public wxEventFilter
+{
+public:
+   Listener (AButton *button);
+   ~Listener();
+
+   int FilterEvent(wxEvent &event) override;
+
+   void OnEvent();
+
+private:
+   AButton *mButton;
+};
+
+AButton::Listener::Listener (AButton *button)
+: mButton(button)
+{
+   wxEvtHandler::AddFilter(this);
+}
+
+AButton::Listener::~Listener ()
+{
+   wxEvtHandler::RemoveFilter(this);
+}
+
+void AButton::Listener::OnEvent()
+{
+   if (!mButton->IsDown())
+   {
+      int idx = 0;
+      // Ignore the event, consult key states.  One modifier key might
+      // have gone up but another remained down.
+      // Note that CMD (or CTRL) takes precedence over Shift if both are down
+      // and alternates are defined for both
+      // see also AButton::OnMouseEvent()
+      if (wxGetKeyState(WXK_CONTROL) && mButton->HasAlternateImages(2))
+         idx = 2;
+      else if (wxGetKeyState(WXK_SHIFT) && mButton->HasAlternateImages(1))
+         idx = 1;
+
+      // Turn e.g. the "Play" button into a "Loop" button
+      // or "Cut Preview" button
+      mButton->SetAlternateIdx(idx);
+   }
+}
+
+int AButton::Listener::FilterEvent(wxEvent &event)
+{
+   if (event.GetEventType() == wxEVT_KEY_DOWN ||
+       event.GetEventType() == wxEVT_KEY_UP)
+      OnEvent();
+   else if (event.GetEventType() == wxEVT_SET_FOCUS)
+      // A modal dialog might have eaten the modifier key-up with its own
+      // filter before we saw it; this is adequate to fix the button image
+      // when the dialog disappears.
+      OnEvent();
+   return Event_Skip;
+}
+
 AButton::AButton(wxWindow * parent,
                  wxWindowID id,
                  const wxPoint & pos,
@@ -53,13 +118,14 @@ AButton::AButton(wxWindow * parent,
                  wxImage up,
                  wxImage over,
                  wxImage down,
+                 wxImage overDown,
                  wxImage dis,
                  bool toggle):
    wxWindow()
 {
    Init(parent, id, pos, size,
         ImageRoll(up), ImageRoll(over),
-        ImageRoll(down), ImageRoll(dis),
+        ImageRoll(down), ImageRoll(overDown), ImageRoll(dis),
         toggle);
 }
 
@@ -70,17 +136,20 @@ AButton::AButton(wxWindow * parent,
                  ImageRoll up,
                  ImageRoll over,
                  ImageRoll down,
+                 ImageRoll overDown,
                  ImageRoll dis,
                  bool toggle):
    wxWindow()
 {
    Init(parent, id, pos, size,
-        up, over, down, dis,
+        up, over, down, overDown, dis,
         toggle);
 }
 
 AButton::~AButton()
 {
+   if(HasCapture())
+      ReleaseMouse();
 }
 
 void AButton::Init(wxWindow * parent,
@@ -90,6 +159,7 @@ void AButton::Init(wxWindow * parent,
                    ImageRoll up,
                    ImageRoll over,
                    ImageRoll down,
+                   ImageRoll overDown,
                    ImageRoll dis,
                    bool toggle)
 {
@@ -108,22 +178,24 @@ void AButton::Init(wxWindow * parent,
    mToggle = toggle;
    mUseDisabledAsDownHiliteImage = false;
 
-   mImage[0] = up;
-   mImage[1] = over;
-   mImage[2] = down;
-   mImage[3] = dis;
+   mImages.resize(1);
+   mImages[0].mArr[0] = up;
+   mImages[0].mArr[1] = over;
+   mImages[0].mArr[2] = down;
+   mImages[0].mArr[3] = overDown;
+   mImages[0].mArr[4] = dis;
 
-   mAlternate = false;
+   mAlternateIdx = 0;
 
-   mButtonIsFocused = false;
-   mFocusRect = GetRect().Deflate( 3, 3 );
+   mFocusRect = GetClientRect().Deflate( 3, 3 );
+   mForceFocusRect = false;
 
-   SetSizeHints(mImage[0].GetMinSize(),
-                mImage[0].GetMaxSize());
+   SetSizeHints(mImages[0].mArr[0].GetMinSize(),
+                mImages[0].mArr[0].GetMaxSize());
 
 #if wxUSE_ACCESSIBILITY
    SetName( wxT("") );
-   SetAccessible(new AButtonAx(this));
+   SetAccessible(safenew AButtonAx(this));
 #endif
 }
 
@@ -132,41 +204,67 @@ void AButton::UseDisabledAsDownHiliteImage(bool flag)
    mUseDisabledAsDownHiliteImage = flag;
 }
 
-void AButton::SetAlternateImages(wxImage up,
+// This compensates for a but in wxWidgets 3.0.2 for mac:
+// Couldn't set focus from keyboard when AcceptsFocus returns false;
+// this bypasses that limitation
+void AButton::SetFocusFromKbd()
+{
+   auto temp = TemporarilyAllowFocus();
+   SetFocus();
+}
+
+void AButton::SetAlternateImages(unsigned idx,
+                                 wxImage up,
                                  wxImage over,
                                  wxImage down,
+                                 wxImage overDown,
                                  wxImage dis)
 {
-   mAltImage[0] = ImageRoll(up);
-   mAltImage[1] = ImageRoll(over);
-   mAltImage[2] = ImageRoll(down);
-   mAltImage[3] = ImageRoll(dis);
+   if (1 + idx > mImages.size())
+      mImages.resize(1 + idx);
+   mImages[idx].mArr[0] = ImageRoll(up);
+   mImages[idx].mArr[1] = ImageRoll(over);
+   mImages[idx].mArr[2] = ImageRoll(down);
+   mImages[idx].mArr[3] = ImageRoll(overDown);
+   mImages[idx].mArr[4] = ImageRoll(dis);
 }
 
-void AButton::SetAlternateImages(ImageRoll up,
+void AButton::SetAlternateImages(unsigned idx,
+                                 ImageRoll up,
                                  ImageRoll over,
                                  ImageRoll down,
+                                 ImageRoll overDown,
                                  ImageRoll dis)
 {
-   mAltImage[0] = up;
-   mAltImage[1] = over;
-   mAltImage[2] = down;
-   mAltImage[3] = dis;
+   if (1 + idx > mImages.size())
+      mImages.resize(1 + idx);
+   mImages[idx].mArr[0] = up;
+   mImages[idx].mArr[1] = over;
+   mImages[idx].mArr[2] = down;
+   mImages[idx].mArr[3] = overDown;
+   mImages[idx].mArr[4] = dis;
 }
 
-void AButton::SetAlternate(bool useAlternateImages)
+void AButton::SetAlternateIdx(unsigned idx)
 {
    // If alternate-image-state is already correct then
    // nothing to do (saves repainting button).
-   if( mAlternate == useAlternateImages )
+   if( mAlternateIdx == idx )
       return;
-   mAlternate = useAlternateImages;
+   mAlternateIdx = idx;
    Refresh(false);
 }
 
+void AButton::FollowModifierKeys()
+{
+   if(!mListener)
+      mListener = std::make_unique<Listener>(this);
+}
+
 void AButton::SetFocusRect(wxRect & r)
 {
    mFocusRect = r;
+   mForceFocusRect = true;
 }
 
 AButton::AButtonState AButton::GetState()
@@ -181,11 +279,11 @@ AButton::AButtonState AButton::GetState()
          if (mIsClicking) {
             state = mButtonIsDown ? AButtonUp : AButtonDown;
             if (mUseDisabledAsDownHiliteImage) {
-               state = mButtonIsDown ? AButtonOver : AButtonDis;
+               state = mButtonIsDown ? AButtonOverDown : AButtonDis;
             }
          }
          else {
-            state = mButtonIsDown ? AButtonDown : AButtonOver;
+            state = mButtonIsDown ? AButtonOverDown : AButtonOver;
             if (mUseDisabledAsDownHiliteImage) {
                state = mButtonIsDown ? AButtonDis : AButtonOver;
             }
@@ -196,17 +294,17 @@ AButton::AButtonState AButton::GetState()
             state = mButtonIsDown ? AButtonOver : AButtonDown;
          }
          else {
-            state = mButtonIsDown ? AButtonDown : AButtonOver;
+            state = mButtonIsDown ? AButtonOverDown : AButtonOver;
          }
       }
    }
    else {
-      if (mToggle) {
+      //if (mToggle) {
          state = mButtonIsDown ? AButtonDown : AButtonUp;
-      }
-      else {
-         state = mButtonIsDown ? AButtonDown : AButtonUp;
-      }
+      //}
+      //else {
+         //state = mButtonIsDown ? AButtonDown : AButtonUp;
+      //}
    }
 
    return state;
@@ -218,17 +316,12 @@ void AButton::OnPaint(wxPaintEvent & WXUNUSED(event))
 
    AButtonState buttonState = GetState();
 
-   if (mAlternate)
-      mAltImage[buttonState].Draw(dc, GetClientRect());
-   else
-      mImage[buttonState].Draw(dc, GetClientRect());
+   mImages[mAlternateIdx].mArr[buttonState].Draw(dc, GetClientRect());
 
-#if defined(__WXMSW__)
-   if( mButtonIsFocused )
+   if( this == wxWindow::FindFocus() )
    {
       AColor::DrawFocus( dc, mFocusRect );
    }
-#endif
 }
 
 void AButton::OnErase(wxEraseEvent & WXUNUSED(event))
@@ -238,16 +331,27 @@ void AButton::OnErase(wxEraseEvent & WXUNUSED(event))
 
 void AButton::OnSize(wxSizeEvent & WXUNUSED(event))
 {
-   mFocusRect = GetRect().Deflate( 3, 3 );
+   if (!mForceFocusRect)
+   {
+      mFocusRect = GetClientRect().Deflate( 3, 3 );
+   }
    Refresh(false);
 }
 
-bool AButton::HasAlternateImages()
+bool AButton::s_AcceptsFocus{ false };
+
+bool AButton::HasAlternateImages(unsigned idx)
 {
-   return (mAltImage[0].Ok() &&
-           mAltImage[1].Ok() &&
-           mAltImage[2].Ok() &&
-           mAltImage[3].Ok());
+   if (mImages.size() <= idx)
+      return false;
+
+   const ImageArr &images = mImages[idx];
+   const ImageRoll (&arr)[5] = images.mArr;
+   return (arr[0].Ok() &&
+           arr[1].Ok() &&
+           arr[2].Ok() &&
+           arr[3].Ok() &&
+           arr[4].Ok());
 }
 
 void AButton::OnMouseEvent(wxMouseEvent & event)
@@ -255,8 +359,14 @@ void AButton::OnMouseEvent(wxMouseEvent & event)
    wxSize clientSize = GetClientSize();
    AButtonState prevState = GetState();
 
-   if (event.Entering())
+   if (event.Entering()) {
+      // Bug 1201:  On Mac, unsetting and re-setting the tooltip may be needed
+      // to make it pop up when we want it.
+      auto text = GetToolTipText();
+      UnsetToolTip();
+      SetToolTip(text);
       mCursorIsInWindow = true;
+   }
    else if (event.Leaving())
       mCursorIsInWindow = false;
    else
@@ -264,13 +374,13 @@ void AButton::OnMouseEvent(wxMouseEvent & event)
          (event.m_x >= 0 && event.m_y >= 0 &&
           event.m_x < clientSize.x && event.m_y < clientSize.y);
 
-   if (HasAlternateImages() && !mButtonIsDown)
-      mAlternate = event.ShiftDown();
-
    if (mEnabled && event.IsButton()) {
       if (event.ButtonIsDown(wxMOUSE_BTN_ANY)) {
          mIsClicking = true;
-         CaptureMouse();
+         if (event.ButtonDClick())
+            mIsDoubleClicked = true;
+         if( !HasCapture() )
+            CaptureMouse();
       }
       else if (mIsClicking) {
          mIsClicking = false;
@@ -298,22 +408,30 @@ void AButton::OnMouseEvent(wxMouseEvent & event)
    if (newState != prevState) {
       Refresh(false);
 
-      if (mCursorIsInWindow) {
-       #if wxUSE_TOOLTIPS // Not available in wxX11
-         // Display the tooltip in the status bar
-         wxToolTip * pTip = this->GetToolTip();
-         if( pTip ) {
-            wxString tipText = pTip->GetTip();
-            if (!mEnabled)
-               tipText += _(" (disabled)");
-            GetActiveProject()->TP_DisplayStatusMessage(tipText);
-         }
-       #endif
-      }
+      if (mCursorIsInWindow)
+         UpdateStatus();
       else {
          GetActiveProject()->TP_DisplayStatusMessage(wxT(""));
       }
    }
+   else
+      event.Skip();
+}
+
+void AButton::UpdateStatus()
+{
+   if (mCursorIsInWindow) {
+#if wxUSE_TOOLTIPS // Not available in wxX11
+      // Display the tooltip in the status bar
+      wxToolTip * pTip = this->GetToolTip();
+      if( pTip ) {
+         wxString tipText = pTip->GetTip();
+         if (!mEnabled)
+            tipText += _(" (disabled)");
+         GetActiveProject()->TP_DisplayStatusMessage(tipText);
+      }
+#endif
+   }
 }
 
 void AButton::OnCaptureLost(wxMouseCaptureLostEvent & WXUNUSED(event))
@@ -331,19 +449,20 @@ void AButton::OnKeyDown(wxKeyEvent & event)
    switch( event.GetKeyCode() )
    {
    case WXK_RIGHT:
-      Navigate();
+      Navigate(wxNavigationKeyEvent::IsForward);
       break;
    case WXK_LEFT:
       Navigate(wxNavigationKeyEvent::IsBackward);
       break;
    case WXK_TAB:
-      if (event.ShiftDown())
-         Navigate(wxNavigationKeyEvent::IsBackward);
-      else
-         Navigate();
+      Navigate(event.ShiftDown()
+               ? wxNavigationKeyEvent::IsBackward
+               : wxNavigationKeyEvent::IsForward);
       break;
    case WXK_RETURN:
    case WXK_NUMPAD_ENTER:
+      if( !mEnabled )
+         break;
       mWasShiftDown = event.ShiftDown();
       mWasControlDown = event.ControlDown();
       Click();
@@ -355,13 +474,11 @@ void AButton::OnKeyDown(wxKeyEvent & event)
 
 void AButton::OnSetFocus(wxFocusEvent & WXUNUSED(event))
 {
-   mButtonIsFocused = true;
    Refresh( false );
 }
 
 void AButton::OnKillFocus(wxFocusEvent & WXUNUSED(event))
 {
-   mButtonIsFocused = false;
    Refresh( false );
 }
 
@@ -377,16 +494,24 @@ bool AButton::WasControlDown()
 
 void AButton::Enable()
 {
+   wxWindow::Enable(true);
    mEnabled = true;
-   this->Refresh(false);
+   Refresh(false);
 }
 
 void AButton::Disable()
 {
+   // Bug 1565: Tooltips not showing on disabled buttons.
+   // The fix is to NOT tell windows that the button is disabled.
+   // The button's appearance will still change to show it is disabled
+   // since we control that rather than windows.
+#ifndef __WXMSW__
+   wxWindow::Enable(false);
+#endif
    mEnabled = false;
    if (GetCapture()==this)
       ReleaseMouse();
-   this->Refresh(false);
+   Refresh(false);
 }
 
 void AButton::PushDown()
@@ -409,7 +534,9 @@ void AButton::Click()
 {
    wxCommandEvent event(wxEVT_COMMAND_BUTTON_CLICKED, GetId());
    event.SetEventObject(this);
-   GetEventHandler()->ProcessEvent(event);
+   // Be sure to use SafelyProcessEvent so that exceptions do not propagate
+   // out of DoDefaultAction
+   GetEventHandler()->SafelyProcessEvent(event);
 }
 
 void AButton::SetShift(bool shift)
@@ -417,6 +544,16 @@ void AButton::SetShift(bool shift)
    mWasShiftDown = shift;
 }
 
+void AButton::SetControl(bool control)
+{
+   mWasControlDown = control;
+}
+
+auto AButton::TemporarilyAllowFocus() -> TempAllowFocus {
+   s_AcceptsFocus = true;
+   return TempAllowFocus{ &s_AcceptsFocus };
+}
+
 #if wxUSE_ACCESSIBILITY
 
 AButtonAx::AButtonAx( wxWindow *window ):
@@ -599,6 +736,11 @@ wxAccStatus AButtonAx::GetState(int WXUNUSED(childId), long* state)
          *state = wxACC_STATE_SYSTEM_HOTTRACKED | wxACC_STATE_SYSTEM_FOCUSABLE;
       break;
 
+      case AButton::AButtonOverDown:
+         *state = wxACC_STATE_SYSTEM_HOTTRACKED | wxACC_STATE_SYSTEM_PRESSED |
+            wxACC_STATE_SYSTEM_FOCUSABLE;
+      break;
+
       case AButton::AButtonDis:
          *state = wxACC_STATE_SYSTEM_UNAVAILABLE;
       break;
@@ -608,8 +750,6 @@ wxAccStatus AButtonAx::GetState(int WXUNUSED(childId), long* state)
       break;
    }
 
-   // Do not use mButtonIsFocused is not set until after this method
-   // is called.
    *state |= ( ab == wxWindow::FindFocus() ? wxACC_STATE_SYSTEM_FOCUSED : 0 );
 
    return wxACC_OK;
