@@ -72,6 +72,7 @@ ExpandingToolBar.
 #include <wx/window.h>
 #endif
 
+#include <wx/wx.h>
 #include <wx/dcmemory.h>
 #include <wx/log.h>
 #include <wx/dragimag.h>
@@ -106,13 +107,13 @@ public:
 // ExpandingToolBar
 //
 
-BEGIN_EVENT_TABLE(ExpandingToolBar, wxPanel)
+BEGIN_EVENT_TABLE(ExpandingToolBar, wxPanelWrapper)
    EVT_SIZE(ExpandingToolBar::OnSize)
    EVT_TIMER(kTimerID, ExpandingToolBar::OnTimer)
    EVT_BUTTON(kToggleButtonID, ExpandingToolBar::OnToggle)
 END_EVENT_TABLE()
 
-IMPLEMENT_CLASS(ExpandingToolBar, wxPanel)
+IMPLEMENT_CLASS(ExpandingToolBar, wxPanelWrapper)
 
 //static
 int ExpandingToolBar::msNoAutoExpandStack = 0;
@@ -121,7 +122,7 @@ ExpandingToolBar::ExpandingToolBar(wxWindow* parent,
                                    wxWindowID id,
                                    const wxPoint& pos,
                                    const wxSize& size):
-   wxPanel(parent, id, pos, size),
+   wxPanelWrapper(parent, id, pos, size),
    mIsAutoExpanded(false),
    mIsManualExpanded(false),
    mIsExpanded(false),
@@ -130,13 +131,12 @@ ExpandingToolBar::ExpandingToolBar(wxWindow* parent,
    mFrameParent(NULL),
    mDialogParent(NULL),
    mAreaParent(NULL),
-   mSavedArrangement(NULL),
-   mDragImage(NULL),
+   mSavedArrangement{},
    mTopLevelParent(NULL)
 {
-   mMainPanel = new wxPanel(this, -1,
+   mMainPanel = safenew wxPanelWrapper(this, -1,
                             wxDefaultPosition, wxSize(1, 1));
-   mExtraPanel = new wxPanel(this, -1,
+   mExtraPanel = safenew wxPanelWrapper(this, -1,
                              wxDefaultPosition, wxSize(1, 1));
 
    mGrabber = NULL;
@@ -144,14 +144,14 @@ ExpandingToolBar::ExpandingToolBar(wxWindow* parent,
    ToolBarArea *toolBarParent =
       dynamic_cast<ToolBarArea *>(GetParent());
    if (toolBarParent)
-      mGrabber = new ToolBarGrabber(this, -1, this);
+      mGrabber = safenew ToolBarGrabber(this, -1, this);
 
    /// \todo check whether this is a memory leak (and check similar code)
-   wxImage hbar = theTheme.Image(bmpToolBarToggle);
-   wxColour magicColor = wxColour(0, 255, 255);
-   ImageArray fourStates = ImageRoll::SplitV(hbar, magicColor);
-
-   mToggleButton = new AButton(this, kToggleButtonID,
+   //wxImage hbar = theTheme.Image(bmpToolBarToggle);
+   //wxColour magicColor = wxColour(0, 255, 255);
+   //ImageArray fourStates = ImageRoll::SplitV(hbar, magicColor);
+/*
+   mToggleButton = safenew AButton(this, kToggleButtonID,
                                wxDefaultPosition, wxDefaultSize,
                                ImageRoll(ImageRoll::HorizontalRoll,
                                          fourStates[0], magicColor),
@@ -163,7 +163,7 @@ ExpandingToolBar::ExpandingToolBar(wxWindow* parent,
                                          fourStates[3], magicColor),
                                true);
    mToggleButton->UseDisabledAsDownHiliteImage(true);
-
+*/
    SetAutoLayout(true);
    mTimer.SetOwner(this, kTimerID);
 }
@@ -275,49 +275,54 @@ void ExpandingToolBar::TryAutoCollapse()
 #endif
 }
 
-class ExpandingToolBarEvtHandler : public wxEvtHandler
+class ExpandingToolBarEvtHandler final : public wxEvtHandler
 {
  public:
    ExpandingToolBarEvtHandler(ExpandingToolBar *toolbar,
+                              wxWindow *window,
                               wxEvtHandler *inheritedEvtHandler)
    {
       mToolBar = toolbar;
+      mWindow = window;
       mInheritedEvtHandler = inheritedEvtHandler;
-   }
 
-   virtual bool ProcessEvent(wxEvent& evt)
+      window->PushEventHandler(this);
+}
+
+   bool ProcessEvent(wxEvent& evt) override
    {
-      if (mToolBar->IsCursorInWindow())
-         mToolBar->TryAutoExpand();
-      else
+//      if (mToolBar->IsCursorInWindow())
          mToolBar->TryAutoExpand();
+//      else
+//         mToolBar->TryAutoExpand();
 
       return mInheritedEvtHandler->ProcessEvent(evt);
    }
 
+   ~ExpandingToolBarEvtHandler()
+   {
+      mWindow->RemoveEventHandler(this);
+   }
+
 protected:
    ExpandingToolBar *mToolBar;
+   wxWindow *mWindow;
    wxEvtHandler *mInheritedEvtHandler;
 
-   DECLARE_NO_COPY_CLASS(ExpandingToolBarEvtHandler);
+   DECLARE_NO_COPY_CLASS(ExpandingToolBarEvtHandler)
 };
 
 void ExpandingToolBar::RecursivelyPushEventHandlers(wxWindow *win)
 {
    if (!mWindowHash[win]) {
-      ExpandingToolBarEvtHandler *evtHandler =
-         new ExpandingToolBarEvtHandler(this, win->GetEventHandler());
-      win->PushEventHandler(evtHandler);
+      mHandlers.push_back(make_movable<ExpandingToolBarEvtHandler>
+         (this, win, win->GetEventHandler()));
       mWindowHash[win] = 1;
    }
 
    wxWindowList children = win->GetChildren();
-
-   typedef wxWindowList::compatibility_iterator Node;
-   for(Node node = children.GetFirst(); node; node = node->GetNext()) {
-      wxWindow *child = node->GetData();
+   for(auto child : children)
       RecursivelyPushEventHandlers(child);
-   }
 }
 
 bool ExpandingToolBar::Layout()
@@ -537,11 +542,11 @@ void ExpandingToolBar::StartMoving()
    mDropTarget = kDummyRect;
 
    wxColour magicColor = wxColour(0, 255, 255);
-   wxImage tgtImage = theTheme.Image(bmpToolBarTarget);
-   ImageRoll tgtImageRoll = ImageRoll(ImageRoll::VerticalRoll,
-                                      tgtImage,
-                                      magicColor);
-   mTargetPanel = new ImageRollPanel(mAreaParent, -1, tgtImageRoll,
+//   wxImage tgtImage = theTheme.Image(bmpToolBarTarget);
+//   ImageRoll tgtImageRoll = ImageRoll(ImageRoll::VerticalRoll,
+//                                      tgtImage,
+//                                      magicColor);
+   mTargetPanel = safenew ImageRollPanel(mAreaParent, -1, //tgtImageRoll,
                                      wxDefaultPosition,
                                      wxDefaultSize,
                                      wxTRANSPARENT_WINDOW);
@@ -555,7 +560,7 @@ void ExpandingToolBar::StartMoving()
 
    mAreaParent->SetCapturedChild(this);
 
-   mDragImage = new wxDragImage(toolbarBitmap);
+   mDragImage = std::make_unique<wxDragImage>(toolbarBitmap);
    mDragImage->BeginDrag(hotSpot, mAreaParent, mTopLevelParent);
    mDragImage->Show();
    mDragImage->Move(ScreenToClient(wxGetMousePosition()));
@@ -621,7 +626,7 @@ void ExpandingToolBar::FinishMoving()
    if (!mAreaParent || !mSavedArrangement)
       return;
 
-   delete mTargetPanel;
+   // DELETE mTargetPanel; // I think this is not needed, but unreachable anyway -- PRL
 
    mAreaParent->SetCapturedChild(NULL);
 
@@ -631,13 +636,10 @@ void ExpandingToolBar::FinishMoving()
    msNoAutoExpandStack--;
 
    if (mDropTarget == kDummyRect) {
-      mAreaParent->RestoreArrangement(mSavedArrangement);
-      mSavedArrangement = NULL;
+      mAreaParent->RestoreArrangement(std::move(mSavedArrangement));
    }
    else {
-      delete mSavedArrangement;
-      mSavedArrangement = NULL;
-
+      mSavedArrangement.reset();
       mAreaParent->MoveChild(this, mDropTarget);
    }
 
@@ -654,22 +656,23 @@ void ExpandingToolBar::FinishMoving()
 // ToolBarGrabber
 //
 
-BEGIN_EVENT_TABLE(ToolBarGrabber, wxPanel)
+BEGIN_EVENT_TABLE(ToolBarGrabber, wxPanelWrapper)
    EVT_PAINT(ToolBarGrabber::OnPaint)
    EVT_SIZE(ToolBarGrabber::OnSize)
    EVT_MOUSE_EVENTS(ToolBarGrabber::OnMouse)
 END_EVENT_TABLE()
 
-IMPLEMENT_CLASS(ToolBarGrabber, wxPanel)
+IMPLEMENT_CLASS(ToolBarGrabber, wxPanelWrapper)
 
 ToolBarGrabber::ToolBarGrabber(wxWindow *parent,
                                wxWindowID id,
                                ExpandingToolBar *ownerToolbar,
                                const wxPoint& pos,
                                const wxSize& size):
-   wxPanel(parent, id, pos, size),
+   wxPanelWrapper(parent, id, pos, size),
    mOwnerToolBar(ownerToolbar)
 {
+#if 0
    wxImage grabberImages = theTheme.Image(bmpToolBarGrabber);
    wxColour magicColor = wxColour(0, 255, 255);
    ImageArray images = ImageRoll::SplitH(grabberImages, magicColor);
@@ -683,7 +686,7 @@ ToolBarGrabber::ToolBarGrabber(wxWindow *parent,
 
    SetSizeHints(mImageRoll[0].GetMinSize(),
                 mImageRoll[1].GetMaxSize());
-
+#endif
    mState = 0;
 }
 
@@ -718,7 +721,7 @@ void ToolBarGrabber::OnPaint(wxPaintEvent & WXUNUSED(event))
 {
    wxPaintDC dc(this);
 
-   mImageRoll[mState].Draw(dc, GetClientRect());
+  // mImageRoll[mState].Draw(dc, GetClientRect());
 }
 
 void ToolBarGrabber::OnSize(wxSizeEvent & WXUNUSED(event))
@@ -730,16 +733,16 @@ void ToolBarGrabber::OnSize(wxSizeEvent & WXUNUSED(event))
 // ToolBarDialog
 //
 
-BEGIN_EVENT_TABLE(ToolBarDialog, wxDialog)
+BEGIN_EVENT_TABLE(ToolBarDialog, wxDialogWrapper)
 END_EVENT_TABLE()
 
-IMPLEMENT_CLASS(ToolBarDialog, wxDialog)
+IMPLEMENT_CLASS(ToolBarDialog, wxDialogWrapper)
 
 ToolBarDialog::ToolBarDialog(wxWindow* parent,
                            wxWindowID id,
                            const wxString& name,
                            const wxPoint& pos):
-   wxDialog(parent, id, name, pos, wxSize(1, 1),
+   wxDialogWrapper(parent, id, name, pos, wxSize(1, 1),
 // Workaround for bug in __WXMSW__.  No close box on a wxDialog unless wxSYSTEM_MENU is used.
 #ifdef __WXMSW__
       wxSYSTEM_MENU |
@@ -834,18 +837,18 @@ void ToolBarFrame::Fit()
 // ToolBarArea
 //
 
-BEGIN_EVENT_TABLE(ToolBarArea, wxPanel)
+BEGIN_EVENT_TABLE(ToolBarArea, wxPanelWrapper)
    EVT_SIZE(ToolBarArea::OnSize)
    EVT_MOUSE_EVENTS(ToolBarArea::OnMouse)
 END_EVENT_TABLE()
 
-IMPLEMENT_CLASS(ToolBarArea, wxPanel)
+IMPLEMENT_CLASS(ToolBarArea, wxPanelWrapper)
 
 ToolBarArea::ToolBarArea(wxWindow* parent,
                          wxWindowID id,
                          const wxPoint& pos,
                          const wxSize& size):
-   wxPanel(parent, id, pos, size),
+   wxPanelWrapper(parent, id, pos, size),
    mInOnSize(false),
    mCapturedChild(NULL)
 {
@@ -952,7 +955,7 @@ void ToolBarArea::LayoutOne(int childIndex)
       wxPoint p = mChildArray[childIndex]->GetPosition();
       wxSize s = mChildArray[childIndex]->GetSize();
 
-      printf("ToolBar %d moved to row %d at (%d, %d), size (%d x %d)\n",
+      wxPrintf("ToolBar %d moved to row %d at (%d, %d), size (%d x %d)\n",
              childIndex, mRowArray[childIndex],
              p.x, p.y, s.x, s.y);
       #endif
@@ -1004,7 +1007,7 @@ void ToolBarArea::LayoutOne(int childIndex)
    wxPoint p = mChildArray[childIndex]->GetPosition();
    wxSize s = mChildArray[childIndex]->GetSize();
 
-   printf("ToolBar %d moved to row %d at (%d, %d), size (%d x %d)\n",
+   wxPrintf("ToolBar %d moved to row %d at (%d, %d), size (%d x %d)\n",
           childIndex, mRowArray[childIndex],
           p.x, p.y, s.x, s.y);
    #endif
@@ -1112,7 +1115,7 @@ void ToolBarArea::OnSize(wxSizeEvent & WXUNUSED(event))
 
    wxSize currentSize = GetClientSize();
 
-   if (abs(currentSize.x - mLastLayoutSize.x >= 100)) {
+   if (abs(currentSize.x - mLastLayoutSize.x) >= 100) {
       // If they resize by more than 100 pixels (horizontally),
       // we totally redo the layout, preserving the order of the
       // toolbars but not the exact position.
@@ -1181,9 +1184,9 @@ void ToolBarArea::RemoveChild(ExpandingToolBar *child)
    }
 }
 
-ToolBarArrangement *ToolBarArea::SaveArrangement()
+std::unique_ptr<ToolBarArrangement> ToolBarArea::SaveArrangement()
 {
-   ToolBarArrangement *arrangement = new ToolBarArrangement();
+   auto arrangement = std::make_unique<ToolBarArrangement>();
    int i;
 
    arrangement->childArray = mChildArray;
@@ -1195,7 +1198,7 @@ ToolBarArrangement *ToolBarArea::SaveArrangement()
    return arrangement;
 }
 
-void ToolBarArea::RestoreArrangement(ToolBarArrangement *arrangement)
+void ToolBarArea::RestoreArrangement(std::unique_ptr<ToolBarArrangement>&& arrangement)
 {
    int i;
 
@@ -1209,7 +1212,7 @@ void ToolBarArea::RestoreArrangement(ToolBarArrangement *arrangement)
 
    Fit(false, true);
 
-   delete arrangement;
+   arrangement.reset();
 }
 
 wxArrayRect ToolBarArea::GetDropTargets()
