@@ -24,6 +24,8 @@
 #include "KeyView.h"
 
 #include <wx/arrimpl.cpp>
+#include <wx/dc.h>
+#include "../Internat.h"
 
 // Various drawing constants
 #define KV_BITMAP_SIZE 16
@@ -46,6 +48,9 @@ BEGIN_EVENT_TABLE(KeyView, wxVListBox)
    EVT_SCROLLWIN(KeyView::OnScroll)
 END_EVENT_TABLE();
 
+wxString    KeyView::CommandTranslated="Command";
+
+
 // ============================================================================
 // KeyView class
 // ============================================================================
@@ -53,64 +58,26 @@ KeyView::KeyView(wxWindow *parent,
                  wxWindowID id,
                  const wxPoint & pos,
                  const wxSize & size)
-: wxVListBox(parent, id, pos, size, wxBORDER_THEME),
-  mOpen(NULL),
-  mClosed(NULL),
+: wxVListBox(parent, id, pos, size, wxBORDER_THEME | wxHSCROLL | wxVSCROLL),
   mScrollX(0),
   mWidth(0)
 {
 #if wxUSE_ACCESSIBILITY
    // Create and set accessibility object
-   mAx = new KeyViewAx(this);
-   SetAccessible(mAx);
+   SetAccessible(mAx = safenew KeyViewAx(this));
 #endif
 
-   // Create the device context
-   wxMemoryDC dc;
-
-   // Create the open/expanded bitmap
-   mOpen = new wxBitmap(16, 16);
-   dc.SelectObject(*mOpen);
-
-   dc.SetBrush(*wxWHITE_BRUSH);
-   dc.SetPen(*wxWHITE_PEN);
-   dc.DrawRectangle(0, 0, 16, 16);
-
-   dc.SetPen(*wxBLACK_PEN);
-   dc.DrawRectangle(3, 4, 9, 9);
-   dc.DrawLine(5, 8, 10, 8);
-   dc.SelectObject(wxNullBitmap);
-
-   // Create the closed/collapsed bitmap
-   mClosed = new wxBitmap(16, 16);
-   dc.SelectObject(*mClosed);
-
-   dc.SetBrush(*wxWHITE_BRUSH);
-   dc.SetPen(*wxWHITE_PEN);
-   dc.DrawRectangle(0, 0, 16, 16);
-
-   dc.SetPen(*wxBLACK_PEN);
-   dc.DrawRectangle(3, 4, 9, 9);
-   dc.DrawLine(7, 6, 7, 11);
-   dc.DrawLine(5, 8, 10, 8);
-   dc.SelectObject(wxNullBitmap);
+   SetMinSize({-1, 150});
 
    // The default view
    mViewType = ViewByTree;
+
+   // Calculate measurements used for columns and scrolling
+   RecalcExtents();
 }
 
 KeyView::~KeyView()
 {
-   // Cleanup
-   if (mOpen)
-   {
-      delete mOpen;
-   }
-
-   if (mClosed)
-   {
-      delete mClosed;
-   }
 }
 
 //
@@ -305,7 +272,7 @@ KeyView::SetKey(int index, const wxString & key)
       return false;
    }
 
-   // Set the new key
+   // Set the NEW key
    node.key = key;
 
    // Check to see if the key column needs to be expanded
@@ -378,7 +345,7 @@ KeyView::SetView(ViewByType type)
    // Unselect any currently selected line...do even if none selected
    SelectNode(-1);
 
-   // Save new type
+   // Save NEW type
    mViewType = type;
 
    // Refresh the view lines
@@ -390,6 +357,15 @@ KeyView::SetView(ViewByType type)
       SelectNode(index);
    }
 
+#if 0
+   // JKC: Optional code to list commants and shortcuts to debug console.
+   int nLines = mLines.GetCount();
+   int flags = 8;
+   for(int i=0;i<nLines;i++){
+      wxLogDebug("T.Add( %2i, %2i,  0, \"%s%s\" );", mLines[i]->depth-1, flags, mLines[i]->label,mLines[i]->key ); 
+   }
+#endif
+
    return;
 }
 
@@ -527,9 +503,9 @@ KeyView::UpdateHScroll()
 
    // Calculate the full line width
    mWidth = KV_LEFT_MARGIN +
-            mCommandWidth +
-            KV_COLUMN_SPACER +
             mKeyWidth +
+            KV_COLUMN_SPACER +
+            mCommandWidth +
             KV_VSCROLL_WIDTH;
 
    // Retrieve the current horizontal scroll amount
@@ -551,17 +527,17 @@ KeyView::UpdateHScroll()
 }
 
 //
-// Process a new set of bindings
+// Process a NEW set of bindings
 //
 void
 KeyView::RefreshBindings(const wxArrayString & names,
                          const wxArrayString & categories,
                          const wxArrayString & prefixes,
                          const wxArrayString & labels,
-                         const wxArrayString & keys)
+                         const wxArrayString & keys,
+                         bool bSort
+                         )
 {
-   bool firsttime = mNodes.GetCount() == 0;
-
    // Start clean
    mNodes.Clear();
 
@@ -585,8 +561,8 @@ KeyView::RefreshBindings(const wxArrayString & names,
       int x, y;
 
       // Remove any menu code from the category and prefix
-      wxString cat = wxMenuItem::GetLabelFromText(categories[i]);
-      wxString pfx = wxMenuItem::GetLabelFromText(prefixes[i]);
+      wxString cat = wxMenuItem::GetLabelText(categories[i]);
+      wxString pfx = wxMenuItem::GetLabelText(prefixes[i]);
 
       // Append "Menu" this node is for a menu title
       if (cat != wxT("Command"))
@@ -595,10 +571,10 @@ KeyView::RefreshBindings(const wxArrayString & names,
          cat += _("Menu");
       }
 
-      // Process a new category
+      // Process a NEW category
       if (cat != lastcat)
       {
-         // A new category always finishes any current subtree
+         // A NEW category always finishes any current subtree
          if (inpfx)
          {
             // Back to category level
@@ -617,7 +593,7 @@ KeyView::RefreshBindings(const wxArrayString & names,
          // Remember for next iteration
          lastcat = cat;
 
-         // Add a new category node
+         // Add a NEW category node
          if (cat != wxEmptyString)
          {
             KeyNode node;
@@ -631,6 +607,7 @@ KeyView::RefreshBindings(const wxArrayString & names,
             node.iscat = true;
             node.isparent = true;
             node.depth = depth++;
+            node.isopen = true;
 
             // Add it to the tree
             mNodes.Add(node);
@@ -643,7 +620,7 @@ KeyView::RefreshBindings(const wxArrayString & names,
          }
       }
 
-      // Process a new prefix
+      // Process a NEW prefix
       if (pfx != lastpfx)
       {
          // Done with prefix branch
@@ -656,7 +633,7 @@ KeyView::RefreshBindings(const wxArrayString & names,
          // Remember for next iteration
          lastpfx = pfx;
 
-         // Add a new prefix node
+         // Add a NEW prefix node
          if (pfx != wxEmptyString)
          {
             KeyNode node;
@@ -670,6 +647,7 @@ KeyView::RefreshBindings(const wxArrayString & names,
             node.ispfx = true;
             node.isparent = true;
             node.depth = depth++;
+            node.isopen = true;
 
             // Add it to the tree
             mNodes.Add(node);
@@ -696,7 +674,7 @@ KeyView::RefreshBindings(const wxArrayString & names,
       else
       {
          // Strip any menu codes from label
-         node.label = wxMenuItem::GetLabelFromText(labels[i].BeforeFirst(wxT('\t')));
+         node.label = wxMenuItem::GetLabelText(labels[i].BeforeFirst(wxT('\t')));
       }
 
       // Fill in remaining info
@@ -740,10 +718,10 @@ KeyView::RefreshBindings(const wxArrayString & names,
          node.isparent,
          node.iscat,
          node.ispfx,
-         node.name.c_str(),
-         node.category.c_str(),
-         node.prefix.c_str(),
-         node.label.c_str());
+         node.name,
+         node.category,
+         node.prefix,
+         node.label);
    }
 #endif
 
@@ -751,10 +729,10 @@ KeyView::RefreshBindings(const wxArrayString & names,
    UpdateHScroll();
 
    // Refresh the view lines
-   RefreshLines();
+   RefreshLines(bSort);
 
-   // Set the selected node if this was the first time through
-   if (firsttime)
+   // Set the selected node if we've just reprepared the list and nothing was selected.
+   if ((GetSelection()==wxNOT_FOUND) && bSort )
    {
       SelectNode(LineToIndex(0));
    }
@@ -764,7 +742,7 @@ KeyView::RefreshBindings(const wxArrayString & names,
 // Refresh the list of lines within the current view
 //
 void
-KeyView::RefreshLines()
+KeyView::RefreshLines(bool bSort)
 {
    int cnt = (int) mNodes.GetCount();
    int linecnt = 0;
@@ -946,20 +924,33 @@ KeyView::RefreshLines()
       }
    }
 
-   // Sort list based on type
-   switch (mViewType)
+   // Sorting is costly.  If bSort is false, we do not have to sort.
+   // bSort false means we know that the list will be updated again before
+   // the user needs to see it.
+   if( bSort )
    {
-      case ViewByTree:
-         mLines.Sort(CmpKeyNodeByTree);
-      break;
+      //To see how many lines are being sorted (and how often).
+      //wxLogDebug("Sorting %i lines for type %i", mLines.GetCount(), mViewType);
 
-      case ViewByName:
-         mLines.Sort(CmpKeyNodeByName);
-      break;
+      // Speed up the comparison function used in sorting
+      // by only translating this string once.
+      CommandTranslated = _("Command");
 
-      case ViewByKey:
-         mLines.Sort(CmpKeyNodeByKey);
-      break;
+      // Sort list based on type
+      switch (mViewType)
+      {
+         case ViewByTree:
+            mLines.Sort(CmpKeyNodeByTree);
+         break;
+
+         case ViewByName:
+            mLines.Sort(CmpKeyNodeByName);
+         break;
+
+         case ViewByKey:
+            mLines.Sort(CmpKeyNodeByKey);
+         break;
+      }
    }
 
    // Now, reassign the line numbers
@@ -981,20 +972,21 @@ KeyView::RefreshLines()
          node.isparent,
          node.iscat,
          node.ispfx,
-         node.name.c_str(),
-         node.category.c_str(),
-         node.prefix.c_str(),
-         node.label.c_str());
+         node.name,
+         node.category,
+         node.prefix,
+         node.label);
    }
 #endif
 
-   // Tell listbox the new count and refresh the entire view
+   // Tell listbox the NEW count and refresh the entire view
    SetItemCount(mLines.GetCount());
    RefreshAll();
 
 #if wxUSE_ACCESSIBILITY
    // Let accessibility know that the list has changed
-   mAx->ListUpdated();
+   if( bSort )
+      mAx->ListUpdated();
 #endif
 }
 
@@ -1065,23 +1057,26 @@ KeyView::OnDrawBackground(wxDC & dc, const wxRect & rect, size_t line) const
 {
    const KeyNode *node = mLines[line];
    wxRect r = rect;
+   wxRect r2 = rect; // for just the key shortcut.
    wxCoord indent = 0;
 
    // When in tree view mode, each younger branch gets indented by the
    // width of the open/close bitmaps
    if (mViewType == ViewByTree)
    {
-      indent += node->depth * KV_BITMAP_SIZE;
+      indent += mKeyWidth + KV_COLUMN_SPACER + node->depth * KV_BITMAP_SIZE;
    }
 
    // Offset left side by the indentation (if any) and scroll amounts
    r.x = indent - mScrollX;
+   r2.x = -mScrollX;
 
    // If the line width is less than the client width, then we want to
    // extend the background to the right edge of the client view.  Otherwise,
    // go all the way to the end of the line width...this will draw past the
    // right edge, but that's what we want.
    r.width = wxMax(mWidth, r.width);
+   r2.width = mKeyWidth;
 
    // Selected lines get a solid background
    if (IsSelected(line))
@@ -1089,16 +1084,30 @@ KeyView::OnDrawBackground(wxDC & dc, const wxRect & rect, size_t line) const
       if (FindFocus() == this)
       {
          // Focused lines get highlighted background
+         dc.SetPen(*wxTRANSPARENT_PEN);
          dc.SetBrush(wxBrush(wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT)));
-         AColor::DrawFocus(dc, r);
          dc.DrawRectangle(r);
+         
+         // and they also get a dotted focus rect.  This could just be left out.
+         // The focus rect does very little for us, as it is the same size as the 
+         // rectangle itself.  Consequently for themes that have black text it
+         // disappears.  But on HiContrast you do get a dotted green border which
+         // may have some utility.
+         AColor::DrawFocus(dc, r);
+
+         if (mViewType == ViewByTree){
+            dc.DrawRectangle(r2);
+            AColor::DrawFocus(dc, r2);
+         }
       }
       else
       {
-         // Non ocused lines get a light background
-         dc.SetBrush(wxBrush(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE)));
+         // Non focused lines get a light background
          dc.SetPen(*wxTRANSPARENT_PEN);
+         dc.SetBrush(wxBrush(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE)));
          dc.DrawRectangle(r);
+         if (mViewType == ViewByTree)
+            dc.DrawRectangle(r2);
       }
    }
    else
@@ -1106,6 +1115,8 @@ KeyView::OnDrawBackground(wxDC & dc, const wxRect & rect, size_t line) const
       // Non-selected lines get a thin bottom border
       dc.SetPen(wxColour(240, 240, 240));
       dc.DrawLine(r.GetLeft(), r.GetBottom(), r.GetRight(), r.GetBottom());
+      if (mViewType == ViewByTree )
+         dc.DrawLine(r2.GetLeft(), r2.GetBottom(), r2.GetRight(), r2.GetBottom());
    }
 }
 
@@ -1139,23 +1150,36 @@ KeyView::OnDrawItem(wxDC & dc, const wxRect & rect, size_t line) const
       // Adjust left edge to account for scrolling
       wxCoord x = rect.x - mScrollX;
 
-      if (node->iscat)
+      if (node->iscat || node->ispfx)
       {
-         // Draw categories bitmap at left edge
-         dc.DrawBitmap(node->isopen ? *mOpen : *mClosed, x, rect.y);
-      }
-      else if (node->ispfx)
-      {
-         // Draw prefix bitmap to the right of the category bitmap
-         dc.DrawBitmap(node->isopen ? *mOpen : *mClosed, x + KV_BITMAP_SIZE, rect.y);
+         wxCoord bx = x + mKeyWidth  + KV_COLUMN_SPACER;
+         wxCoord by = rect.y;
+
+         if (node->ispfx)
+         {
+            bx += KV_BITMAP_SIZE;
+         }
+
+         dc.SetBrush(*wxTRANSPARENT_BRUSH);
+         dc.SetPen(*wxBLACK_PEN);
+         dc.DrawRectangle(bx + 3, by + 4, 9, 9);
+         if (node->isopen)
+         {
+            AColor::Line(dc, bx + 5, by + 8, bx + 9, by + 8);
+         }
+         else
+         {
+            AColor::Line(dc, bx + 7, by + 6, bx + 7, by + 10);
+            AColor::Line(dc, bx + 5, by + 8, bx + 9, by + 8);
+         }
       }
 
       // Indent text
       x += KV_LEFT_MARGIN;
 
-      // Draw the command and key columns
-      dc.DrawText(label, x + node->depth * KV_BITMAP_SIZE, rect.y);
-      dc.DrawText(node->key, x + mCommandWidth + KV_COLUMN_SPACER, rect.y);
+      // Draw the key and command columns
+      dc.DrawText(node->key, x , rect.y);
+      dc.DrawText(label, x + mKeyWidth  + KV_COLUMN_SPACER + node->depth * KV_BITMAP_SIZE, rect.y);
    }
    else
    {
@@ -1168,14 +1192,8 @@ KeyView::OnDrawItem(wxDC & dc, const wxRect & rect, size_t line) const
          label = node->prefix + wxT(" - ") + label;
       }
 
-      // Swap the columns based on view type
-      if(mViewType == ViewByName)
-      {
-         // Draw command column and then key column
-         dc.DrawText(label, x, rect.y);
-         dc.DrawText(node->key, x + mCommandWidth + KV_COLUMN_SPACER, rect.y);
-      }
-      else if(mViewType == ViewByKey)
+      // don't swap the columns based on view type
+      if((mViewType == ViewByName) || (mViewType == ViewByKey))
       {
          // Draw key columnd and then command column
          dc.DrawText(node->key, x, rect.y);
@@ -1225,11 +1243,11 @@ KeyView::OnSetFocus(wxFocusEvent & event)
    event.Skip();
 
    // Refresh the selected line to pull in any changes while
-   // focus was away...like when setting a new key value.  This
+   // focus was away...like when setting a NEW key value.  This
    // will also refresh the visual (highlighted) state.
    if (GetSelection() != wxNOT_FOUND)
    {
-      RefreshLine(GetSelection());
+	   RefreshRow(GetSelection());
    }
 
 #if wxUSE_ACCESSIBILITY
@@ -1250,7 +1268,7 @@ KeyView::OnKillFocus(wxFocusEvent & event)
    // Refresh the selected line to adjust visual highlighting.
    if (GetSelection() != wxNOT_FOUND)
    {
-      RefreshLine(GetSelection());
+	   RefreshRow(GetSelection());
    }
 }
 
@@ -1278,7 +1296,7 @@ KeyView::OnScroll(wxScrollWinEvent & event)
       return;
    }
 
-   // Get new scroll position and scroll the view
+   // Get NEW scroll position and scroll the view
    mScrollX = event.GetPosition();
    SetScrollPos(wxHORIZONTAL, mScrollX);
 
@@ -1325,7 +1343,7 @@ KeyView::OnKeyDown(wxKeyEvent & event)
             RefreshLines();
 
             // Reset the original top line
-            ScrollToLine(topline);
+            ScrollToRow(topline);
 
             // And make sure current line is still selected
             SelectNode(LineToIndex(line));
@@ -1389,7 +1407,7 @@ KeyView::OnKeyDown(wxKeyEvent & event)
                RefreshLines();
 
                // Reset the original top line
-               ScrollToLine(topline);
+               ScrollToRow(topline);
 
                // And make sure current line is still selected
                SelectNode(LineToIndex(line));
@@ -1510,7 +1528,7 @@ KeyView::OnLeftDown(wxMouseEvent & event)
    wxPoint pos = event.GetPosition();
 
    // And see if it was on a line within the view
-   int line = HitTest(pos);
+   int line = VirtualHitTest(pos.y);
 
    // It was on a line
    if (line != wxNOT_FOUND)
@@ -1531,7 +1549,7 @@ KeyView::OnLeftDown(wxMouseEvent & event)
          RefreshLines();
 
          // Reset the original top line
-         ScrollToLine(topline);
+         ScrollToRow(topline);
 
          // And make sure current line is still selected
          SelectNode(LineToIndex(line));
@@ -1561,48 +1579,30 @@ KeyView::CmpKeyNodeByTree(KeyNode ***n1, KeyNode ***n2)
 {
    KeyNode *t1 = (**n1);
    KeyNode *t2 = (**n2);
-   wxString k1 = t1->label;
-   wxString k2 = t2->label;
+
+   unsigned int k1UInt= 0xffffffff;
+   unsigned int k2UInt= 0xffffffff;
 
    // This is a "command" node if its category is "Command"
    // and it is a child of the "Command" category.  This latter
    // test ensures that the "Command" parent will be handled
    // as a "menu" node and remain at the bottom of the list.
-   if (t1->category == _("Command") && !t1->isparent)
-   {
-      // A "command" node, so prepend the highest hex value
-      k1.Printf(wxT("ffffffff%s"), t1->label.c_str());
-   }
-   else
-   {
-      // A "menu" node, so prepend the line number
-      k1.Printf(wxT("%08x%s"), t1->line, t1->label.c_str());
-   }
+   if (t1->category != CommandTranslated || t1->isparent)
+      k1UInt = (unsigned int) t1->line;
 
    // See above for explanation
-   if (t2->category == _("Command") && !t2->isparent)
-   {
-      // A "command" node, so prepend the highest hex value
-      k2.Printf(wxT("ffffffff%s"), t2->label.c_str());
-   }
-   else
-   {
-      // A "menu" node, so prepend the line number
-      k2.Printf(wxT("%08x%s"), t2->line, t2->label.c_str());
-   }
+   if (t2->category != CommandTranslated || t2->isparent)
+      k2UInt = (unsigned int) t2->line;
 
-   // See wxWidgets documentation for explanation of comparison results.
-   // These will produce an ascending order.
-   if (k1 < k2)
-   {
+   if( k1UInt < k2UInt ) 
       return -1;
-   }
+   if( k1UInt > k2UInt )
+      return +1;
 
-   if (k1 > k2)
-   {
+   if( t1->label < t2->label )
+      return -1;
+   if( t1->label > t2->label )
       return 1;
-   }
-
    return 0;
 }
 
@@ -1757,7 +1757,7 @@ KeyView::GetLineHeight(int line)
       return 0;
    }
 
-   return OnGetLineHeight(line);
+   return OnGetRowHeight(line);
 }
 
 //
@@ -1837,7 +1837,7 @@ KeyViewAx::ListUpdated()
 }
 
 //
-// Inform accessibility a new line has been selected and/or a previously
+// Inform accessibility a NEW line has been selected and/or a previously
 // selected line is being unselected
 //
 void
@@ -1892,7 +1892,7 @@ KeyViewAx::IdToLine(int childId, int & line)
    line = childId - 1;
 
    // Make sure id is valid
-   if (line < 0 || line >= (int) mView->GetLineCount())
+   if (line < 0 || line >= (int) mView->GetItemCount())
    {
       // Indicate the control itself in this case
       return false;
@@ -1908,7 +1908,7 @@ bool
 KeyViewAx::LineToId(int line, int & childId)
 {
    // Make sure line is valid
-   if (line < 0 || line >= (int) mView->GetLineCount())
+   if (line < 0 || line >= (int) mView->GetItemCount())
    {
       // Indicate the control itself in this case
       childId = wxACC_SELF;
@@ -1966,7 +1966,7 @@ KeyViewAx::GetChild(int childId, wxAccessible** child)
 wxAccStatus
 KeyViewAx::GetChildCount(int *childCount)
 {
-   *childCount = (int) mView->GetLineCount();
+   *childCount = (int) mView->GetItemCount();
 
    return wxACC_OK;
 }
