@@ -28,6 +28,9 @@
 
 #include "../Audacity.h"
 
+#include "../MemoryX.h"
+#include <algorithm>
+
 #include <wx/defs.h>
 #include <wx/app.h>
 #include <wx/button.h>
@@ -35,27 +38,20 @@
 #include <wx/datetime.h>
 #include <wx/dialog.h>
 #include <wx/event.h>
+#include <wx/evtloop.h>
 #include <wx/frame.h>
 #include <wx/intl.h>
-#if defined(__WXMAC__)
-#include <wx/menu.h>
-#endif
+#include <wx/msgdlg.h>
 #include <wx/settings.h>
 #include <wx/sizer.h>
+#include <wx/sound.h>
 #include <wx/stopwatch.h>
 #include <wx/window.h>
 
 #include "ProgressDialog.h"
+#include "ErrorDialog.h"
 #include "../Prefs.h"
-#include "../ShuttleGui.h"
-
-#include <wx/sound.h>
-
-// Remove this when wxSound patch gets applied to wxWidgets
-// http://trac.wxwidgets.org/ticket/10362
-#if defined(__WXMAC__)
-#include <Carbon/Carbon.h>
-#endif
+#include "../Internat.h"
 
 // This really should be a Preferences setting
 static const unsigned char beep[] =
@@ -989,7 +985,7 @@ static const unsigned char beep[] =
 /// Methods for ProgressDialog
 ////////////////////////////////////////////////////////////
 
-BEGIN_EVENT_TABLE(ProgressDialog, wxDialog)
+BEGIN_EVENT_TABLE(ProgressDialog, wxDialogWrapper)
    EVT_BUTTON(wxID_CANCEL, ProgressDialog::OnCancel)
    EVT_BUTTON(wxID_OK, ProgressDialog::OnStop)
    EVT_CLOSE(ProgressDialog::OnCloseWindow)
@@ -998,199 +994,27 @@ END_EVENT_TABLE()
 //
 // Constructor
 //
-ProgressDialog::ProgressDialog(const wxString & title, const wxString & message, ProgressDialogFlags flags)
-: wxDialog(wxTheApp->GetTopWindow(),
-           wxID_ANY,
-           title,
-           wxDefaultPosition,
-           wxDefaultSize,
-           wxDEFAULT_DIALOG_STYLE |
-           wxFRAME_FLOAT_ON_PARENT),
-   mLastValue(0),
-   mDisable(NULL)
+ProgressDialog::ProgressDialog()
+:  wxDialogWrapper()
 {
-   wxBoxSizer *v;
-   wxWindow *w;
-   wxSize ds;
+}
 
-   // There's a problem where the focus is not returned to the window that had
-   // it before creating this object.  The reason is not entirely understood
-   // but if the dialog window never gets shown then the focus does not get
-   // returned to the original window.  It seems to have something to do with
-   // wxWindowDisabler as the problem doesn't occur when it isn't created.
-   //
-   // This never used to be a problem for us because we didn't actually create
-   // the wxProgressDialog until after the elapsed time reached .5 seconds.
-   // This also meant that the app modal state was not established until .5
-   // seconds had passed.  This left a small window where the user would be able
-   // to interact with the main window and possibly do things like get two
-   // effects running at the same time.
+ProgressDialog::ProgressDialog(const wxString & title,
+                               const wxString & message /* = wxEmptyString*/,
+                               int flags /* = pdlgDefaultFlags */,
+                               const wxString & sRemainingLabelText /* = wxEmptyString */)
+:  wxDialogWrapper()
+{
+   Create(title, message, flags, sRemainingLabelText);
+}
 
-   mHadFocus = wxWindow::FindFocus();
-
-#if defined(__WXGTK__)
-   // Under GTK, when applying any effect that prompts the user, it's more than
-   // like that FindFocus() will return NULL.  So, make sure something has focus.
-   if (GetParent()) {
-      GetParent()->SetFocus();
-   }
-#endif
-
-   SetExtraStyle(GetExtraStyle() | wxWS_EX_TRANSIENT);
-
-   v = new wxBoxSizer(wxVERTICAL);
-
-   mMessage = new wxStaticText(this,
-                               wxID_ANY,
-                               message,
-                               wxDefaultPosition,
-                               wxDefaultSize,
-                               wxALIGN_LEFT);
-   mMessage->SetName(message); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
-   v->Add(mMessage, 0, wxEXPAND | wxALL, 10);
-   ds.y += mMessage->GetSize().y + 20;
-
-   //
-   //
-   //
-   mGauge = new wxGauge(this,
-                        wxID_ANY,
-                        1000,
-                        wxDefaultPosition,
-                        wxDefaultSize,
-                        wxGA_HORIZONTAL);
-   v->Add(mGauge, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, 10);
-   ds.y += mGauge->GetSize().y + 10;
-
-   //
-   //
-   //
-   wxFlexGridSizer *g = new wxFlexGridSizer(2, 2, 10, 10);
-
-   w = new wxStaticText(this,
-                        wxID_ANY,
-                        _("Elapsed Time:"),
-                        wxDefaultPosition,
-                        wxDefaultSize,
-                        wxALIGN_RIGHT);
-   w->SetName(w->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
-   g->Add(w, 0, wxALIGN_RIGHT);
-
-   mElapsed = new wxStaticText(this,
-                               wxID_ANY,
-                               wxT("00:00:00"),
-                               wxDefaultPosition,
-                               wxDefaultSize,
-                               wxALIGN_LEFT);
-   mElapsed->SetName(mElapsed->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
-   g->Add(mElapsed, 0, wxALIGN_LEFT);
-   ds.y += mElapsed->GetSize().y + 10;
-
-   //
-   //
-   //
-   w = new wxStaticText(this,
-                        wxID_ANY,
-                        _("Remaining Time:"),
-                        wxDefaultPosition,
-                        wxDefaultSize,
-                        wxALIGN_RIGHT);
-   w->SetName(w->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
-   g->Add(w, 0, wxALIGN_RIGHT);
-
-   mRemaining = new wxStaticText(this,
-                                 wxID_ANY,
-                                 wxT("00:00:00"),
-                                 wxDefaultPosition,
-                                 wxDefaultSize,
-                                 wxALIGN_LEFT);
-   mRemaining->SetName(mRemaining->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
-   g->Add(mRemaining, 0, wxALIGN_LEFT);
-
-   v->Add(g, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT | wxBOTTOM, 10);
-   ds.y += mRemaining->GetSize().y + 10;
-
-   wxBoxSizer *h = new wxBoxSizer(wxHORIZONTAL);
-
-   if (!(flags & pdlgHideStopButton))
-   {
-      w = new wxButton(this, wxID_OK, _("Stop"));
-      h->Add(w, 0, wxALIGN_RIGHT | wxRIGHT | wxBOTTOM, 10);
-      ds.x += w->GetSize().x + 10;
-   }
-
-   if (!(flags & pdlgHideCancelButton))
-   {
-      w = new wxButton(this, wxID_CANCEL, _("Cancel"));
-      h->Add(w, 0, wxALIGN_RIGHT | wxRIGHT | wxBOTTOM, 10);
-      ds.x += w->GetSize().x + 10;
-   }
-
-   v->Add(h, 0, wxALIGN_RIGHT | wxRIGHT | wxBOTTOM, 10);
-
-   ds.y += w->GetSize().y + 10;
-
-   SetSizerAndFit(v);
-
-   wxClientDC dc(this);
-   dc.SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
-   long widthText = 0;
-   dc.GetTextExtent(message, &widthText, NULL, NULL, NULL, NULL);
-   ds.x = (wxCoord) wxMax(wxMax(3 * widthText / 2, 4 * ds.y / 3), 300);
-   SetClientSize(ds);
-
-   Centre(wxCENTER_FRAME | wxBOTH);
-
-   mStartTime = wxGetLocalTimeMillis().GetValue();
-   mLastUpdate = mStartTime;
-   mCancel = false;
-   mStop = false;
-
-   Show(false);
-
-   // Even though we won't necessarily show the dialog due to the the 500ms
-   // delay, we MUST disable other windows/menus anyway since we run the risk
-   // of allowing other tasks to run before this one is complete.
-   mDisable = new wxWindowDisabler(this);
-
-#if defined(__WXMAC__)
-   // LL:  On the Mac, the parent windows get disabled, but they still respond
-   //      to the close button being clicked and the application quit menu item
-   //      is still enabled.  We do not want the parent window to be destroyed
-   //      while we're active, so we have to kludge around a bit to keep this
-   //      from happening.
-   WindowRef windowRef = (WindowRef) MacGetWindowRef();
-   if (windowRef) {
-      SetWindowModality(windowRef, kWindowModalityAppModal, NULL);
-      BeginAppModalStateForWindow(windowRef);
-   }
-
-   w = wxTheApp->GetTopWindow();
-   if (w) {
-      w = wxGetTopLevelParent(w);
-      if (w) {
-         wxFrame *f = wxStaticCast(w, wxFrame);
-         if (f) {
-            wxMenuBar *bar = f->GetMenuBar();
-            if (bar) {
-               bar->Enable(wxID_PREFERENCES, false);
-               bar->Enable(wxID_EXIT, false);
-            }
-         }
-      }
-   }
-#endif
-
-#if defined(__WXMSW__)
-   // See Bug #334
-   // LL:  On Windows, the application message loop is still active even though
-   //      all of the windows have been disabled.  So, keyboard shortcuts still
-   //      work in windows not related to the progress diawhich allows interaction
-   //      when it should be blocked.
-   //      This disabled the application message loop so keyboard shortcuts will
-   //      no longer be processed.
-   wxTheApp->SetEvtHandlerEnabled(false);
-#endif
+ProgressDialog::ProgressDialog(const wxString & title,
+                               const MessageTable &columns,
+                               int flags /* = pdlgDefaultFlags */,
+                               const wxString & sRemainingLabelText /* = wxEmptyString */)
+:  wxDialogWrapper()
+{
+   Create(title, columns, flags, sRemainingLabelText);
 }
 
 //
@@ -1198,6 +1022,10 @@ ProgressDialog::ProgressDialog(const wxString & title, const wxString & message,
 //
 ProgressDialog::~ProgressDialog()
 {
+   // Delete the window disabler before hiding the dialog to allow
+   // focus to return to the original window.
+   mDisable.reset();
+
    if (IsShown())
    {
       Show(false);
@@ -1205,38 +1033,6 @@ ProgressDialog::~ProgressDialog()
       Beep();
    }
 
-#if defined(__WXMSW__)
-   // Undo above fix for bug 334.
-   wxTheApp->SetEvtHandlerEnabled(true);
-#endif
-
-#if defined(__WXMAC__)
-   wxWindow *w = wxTheApp->GetTopWindow();
-   if (w) {
-      w = wxGetTopLevelParent(w);
-      if (w) {
-         wxFrame *f = wxStaticCast(w, wxFrame);
-         if (f) {
-            wxMenuBar *bar = f->GetMenuBar();
-            if (bar) {
-               bar->Enable(wxID_PREFERENCES, true);
-               bar->Enable(wxID_EXIT, true);
-            }
-         }
-      }
-   }
-
-   WindowRef windowRef = (WindowRef) MacGetWindowRef();
-   if (windowRef) {
-      EndAppModalStateForWindow(windowRef);
-   }
-#endif
-
-   if (mDisable)
-   {
-      delete mDisable;
-   }
-
 #if defined(__WXGTK__)
    // Under GTK, when applying any effect that prompts the user, it's more than
    // like that FindFocus() will return NULL.  So, make sure something has focus.
@@ -1245,76 +1041,300 @@ ProgressDialog::~ProgressDialog()
    }
 #endif
 
-   if (mHadFocus) {
+   // Restore saved focus, but only if the window still exists.
+   //
+   // It is possible that it was a deferred deletion and it was deleted since
+   // we captured the focused window.  So, we need to verify that the window
+   // still exists by searching all of the wxWidgets windows.  It's the only
+   // sure way.
+   if (mHadFocus && SearchForWindow(wxTopLevelWindows, mHadFocus)) {
       mHadFocus->SetFocus();
    }
 }
 
-//
-// Show/Hide the dialog
-//
-// At least on the Mac, deleting the WindowDisabler before continuing to the
-// base class is VERY important since menu items can remain in the disabled
-// state.  This has to do with the Mac not honoring the Enable() if the
-// application is still in a modal state.
-//
-// An example is generating a tone in an empty project.  The Export menus
-// will not get enabled.
-//
-bool
-ProgressDialog::Show(bool show)
+void ProgressDialog::Init()
 {
-   if (!show)
+   // There's a problem where the focus is not returned to the window that had
+   // it before creating this object.  The reason is because the focus events
+   // that are sent to the parent window after the wxWindowDisabler are created
+   // are tossed and focus will not get restored to the parent once the disabler
+   // is deleted.  (See bug #1173 for more info)
+   //
+   // So, we capture and restore the focus ourselves.
+   mHadFocus = wxWindow::FindFocus();
+
+#if defined(__WXGTK__)
+   // Under GTK, when applying any effect that prompts the user, it's more than
+   // likely that FindFocus() will return NULL.  So, make sure something has focus.
+   if (GetParent())
    {
-      if (mDisable)
-      {
-         delete mDisable;
-         mDisable = NULL;
-      }
+      GetParent()->SetFocus();
    }
-   else
-   {
-      if (!mDisable)
-      {
-         mDisable = new wxWindowDisabler(this);
+#endif
+}
 
-         #if defined(__WXMAC__)
-            // LL:  On the Mac, the parent windows get disabled, but they still respond
-            //      to the close button being clicked and the application quit menu item
-            //      is still enabled.  We do not want the parent window to be destroyed
-            //      while we're active, so we have to kludge around a bit to keep this
-            //      from happening.
-            WindowRef windowRef = (WindowRef) MacGetWindowRef();
-            SetWindowModality( windowRef, kWindowModalityAppModal, NULL ) ;
-            BeginAppModalStateForWindow(windowRef);
+void ProgressDialog::Reinit()
+{
+   mLastValue = 0;
 
-            wxMenuBar *bar = wxStaticCast(wxGetTopLevelParent(wxTheApp->GetTopWindow()), wxFrame)->GetMenuBar();
-            bar->Enable(wxID_PREFERENCES, false);
-            bar->Enable(wxID_EXIT, false);
-         #endif
-      }
+   mStartTime = wxGetLocalTimeMillis().GetValue();
+   mLastUpdate = mStartTime;
+   mYieldTimer = mStartTime;
+   mCancel = false;
+   mStop = false;
+
+   // Because wxGTK is very sensitive about maintaining focus when
+   // this window is not shown, we always show it.  But, since we
+   // want a 500ms delay before it's actually visible for those
+   // quick tasks, we show it as transparent.  If the initial
+   // delay is exceeded, then we reset the dialog to full opacity.
+   SetTransparent(0);
+   mIsTransparent = true;
+
+   auto button = FindWindowById(wxID_CANCEL, this);
+   if (button)
+      button->Enable();
+   button = FindWindowById(wxID_OK, this);
+   if (button)
+      button->Enable();
+
+   wxDialogWrapper::Show(true);
+}
+
+// Add a NEW text column each time this is called.
+void ProgressDialog::AddMessageAsColumn(wxBoxSizer * pSizer,
+                                        const MessageColumn & column,
+                                        bool bFirstColumn) {
+
+   // Assuming that we don't want empty columns, bail out if there is no text.
+   if (column.empty())
+      return;
+
+   // Join strings
+   auto sText = column[0];
+   std::for_each( column.begin() + 1, column.end(),
+      [&](const wxString &text) { sText += wxT("\n") + text; });
+
+   // Create a statictext object and add to the sizer
+   wxStaticText* oText = safenew wxStaticText(this,
+                                              wxID_ANY,
+                                              sText,
+                                              wxDefaultPosition,
+                                              wxDefaultSize,
+                                              wxALIGN_LEFT);
+   oText->SetName(sText); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+
+   // If this is the first column then set the mMessage pointer so non-TimerRecord usages
+   // will still work correctly in SetMessage()
+   if (bFirstColumn) {
+      mMessage = oText;
    }
 
-    return wxDialog::Show(show);
+   pSizer->Add(oText, 1, wxEXPAND | wxALL, 5);
+}
+
+bool ProgressDialog::Create(const wxString & title,
+                            const wxString & message /* = wxEmptyString */,
+                            int flags /* = pdlgDefaultFlags */,
+                            const wxString & sRemainingLabelText /* = wxEmptyString */)
+{
+   MessageTable columns(1);
+   columns.back().push_back(message);
+   auto result = Create(title, columns, flags, sRemainingLabelText);
+
+   if (result) {
+      // Record some values used in case of change of message
+      // TODO: make the following work in case of message tables
+      wxClientDC dc(this);
+      dc.GetMultiLineTextExtent(message, &mLastW, &mLastH);
+   }
+
+   return result;
+}
+
+bool ProgressDialog::Create(const wxString & title,
+                            const MessageTable & columns,
+                            int flags /* = pdlgDefaultFlags */,
+                            const wxString & sRemainingLabelText /* = wxEmptyString */)
+{
+   Init();
+
+   wxWindow *parent = GetParentForModalDialog(NULL, 0);
+
+   // Set this boolean to indicate if we are using the "Elapsed" labels
+   m_bShowElapsedTime = !(flags & pdlgHideElapsedTime);
+   // Set this boolean to indicate if we confirm the Cancel/Stop actions
+   m_bConfirmAction = (flags & pdlgConfirmStopCancel)!=0;
+
+   bool success = wxDialogWrapper::Create(parent,
+                                   wxID_ANY,
+                                   title,
+                                   wxDefaultPosition,
+                                   wxDefaultSize,
+                                   wxDEFAULT_DIALOG_STYLE |
+                                   wxFRAME_FLOAT_ON_PARENT);
+   if (!success)
+   {
+      return false;
+   }
+
+   SetName(GetTitle()); // This was added for NVDA screen reader and may now be redundant.
+   SetExtraStyle(GetExtraStyle() | wxWS_EX_TRANSIENT);   // Ancient code. Is it still required?
+
+   {
+      wxWindow *window;
+
+      // There may be more than one column, so create a BoxSizer container
+      auto uColSizer = std::make_unique<wxBoxSizer>(wxHORIZONTAL);
+      auto colSizer = uColSizer.get();
+
+      // TODO:  this setting-up of a grid of text in a sizer might be worth
+      // extracting as a utility for building other dialogs.
+      {
+         bool bFirstCol = true;
+         for (const auto &column : columns) {
+            AddMessageAsColumn(colSizer, column, bFirstCol);
+            bFirstCol = false;
+         }
+      }
+
+      // and put message column(s) into a main vertical sizer.
+      auto vertSizer = std::make_unique<wxBoxSizer>(wxVERTICAL);
+      vertSizer->Add(uColSizer.release(), 1, wxEXPAND | wxALL, 10);
+
+      mGauge = safenew wxGauge(this,
+                               wxID_ANY,
+                               1000,
+                               wxDefaultPosition,
+                               wxDefaultSize,
+                               wxGA_HORIZONTAL);
+      vertSizer->Add(mGauge, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, 10);
+
+      {
+         auto uGridSizer = std::make_unique<wxFlexGridSizer>(2, 2, 10, 10);
+         // Only one row if we are not going to show the elapsed time
+         if (m_bShowElapsedTime == false) {
+            uGridSizer = std::make_unique<wxFlexGridSizer>(1, 2, 10, 10);
+         }
+         auto gridSizer = uGridSizer.get();
+
+         if (m_bShowElapsedTime) {
+            window = safenew wxStaticText(this,
+                                          wxID_ANY,
+                                          _("Elapsed Time:"),
+                                          wxDefaultPosition,
+                                          wxDefaultSize,
+                                          wxALIGN_RIGHT);
+
+            window->SetName(window->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+            gridSizer->Add(window, 0, wxALIGN_RIGHT);
+
+            mElapsed = safenew wxStaticText(this,
+                                            wxID_ANY,
+                                            wxT("00:00:00"),
+                                            wxDefaultPosition,
+                                            wxDefaultSize,
+                                            wxALIGN_LEFT);
+
+            mElapsed->SetName(mElapsed->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+            gridSizer->Add(mElapsed, 0, wxALIGN_LEFT);
+         }
+
+         // Customised "Remaining" label text
+         wxString sRemainingText = sRemainingLabelText;
+         if (sRemainingText == wxEmptyString) {
+            sRemainingText = _("Remaining Time:");
+         }
+
+         window = safenew wxStaticText(this,
+                                       wxID_ANY,
+                                       sRemainingText,
+                                       wxDefaultPosition,
+                                       wxDefaultSize,
+                                       wxALIGN_RIGHT);
+         window->SetName(window->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+         gridSizer->Add(window, 0, wxALIGN_RIGHT);
+
+         mRemaining = safenew wxStaticText(this,
+                                           wxID_ANY,
+                                           wxT("00:00:00"),
+                                           wxDefaultPosition,
+                                           wxDefaultSize,
+                                           wxALIGN_LEFT);
+         mRemaining->SetName(mRemaining->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+         gridSizer->Add(mRemaining, 0, wxALIGN_LEFT);
+
+         vertSizer->Add(uGridSizer.release(), 0, wxALIGN_CENTER | wxLEFT | wxRIGHT | wxBOTTOM, 10);
+      }
+
+      {
+         auto uButtonBar = std::make_unique<wxBoxSizer>(wxHORIZONTAL);
+         auto buttonBarSizer = uButtonBar.get();
+
+         if (!(flags & pdlgHideStopButton)) {
+            window = safenew wxButton(this, wxID_OK, _("Stop"));
+            buttonBarSizer->Add(window, 0, wxRIGHT, 10);
+         }
+         if (!(flags & pdlgHideCancelButton)) {
+            window = safenew wxButton(this, wxID_CANCEL, _("Cancel"));
+            buttonBarSizer->Add(window, 0, wxRIGHT, 10);
+         }
+         vertSizer->Add(uButtonBar.release(), 0, wxALIGN_RIGHT | wxRIGHT | wxBOTTOM, 10);
+      }
+
+      SetSizerAndFit(vertSizer.release());
+   }
+   Layout();
+
+   Centre(wxCENTER_FRAME | wxBOTH);
+
+   Reinit();
+
+   // Even though we won't necessarily show the dialog due to the 500ms
+   // delay, we MUST disable other windows/menus anyway since we run the risk
+   // of allowing other tasks to run before this one is complete.
+   //
+   // Reviewed this code per Proposed Features #1, at
+   // http://wiki.audacityteam.org/wiki/Proposal_Timer_Record_Improvements.
+   // Note that this causes a problem for Timer Record wait dialog
+   // (see TimerRecordDialog::RunWaitDialog()), because it makes it
+   // impossible to do any editing, even in other open projects,
+   // while waiting for Timer Record to start -- and then also
+   // while it's recording, it has a ProgressDialog, so really,
+   // no editing in any project until Timer Record finishes.
+   mDisable = std::make_unique<wxWindowDisabler>(this);
+
+   return true;
 }
 
 //
 // Update the time and, optionally, the message
 //
-int
-ProgressDialog::Update(int value, const wxString & message)
+ProgressResult ProgressDialog::Update(int value, const wxString & message)
 {
    if (mCancel)
    {
       // for compatibility with old Update, that returned false on cancel
-      return eProgressCancelled;
+      return ProgressResult::Cancelled;
    }
    else if (mStop)
    {
-      return eProgressStopped;
+      return ProgressResult::Stopped;
    }
 
-   SetMessage(message);
+   wxLongLong_t now = wxGetLocalTimeMillis().GetValue();
+   wxLongLong_t elapsed = now - mStartTime;
+
+   if (elapsed < 500)
+   {
+      return ProgressResult::Success;
+   }
+
+   if (mIsTransparent)
+   {
+      SetTransparent(255);
+      mIsTransparent = false;
+   }
 
    if (value <= 0)
    {
@@ -1326,46 +1346,64 @@ ProgressDialog::Update(int value, const wxString & message)
       value = 1000;
    }
 
-   wxLongLong_t now = wxGetLocalTimeMillis().GetValue();
-   wxLongLong_t elapsed = now - mStartTime;
    wxLongLong_t estimate = elapsed * 1000ll / value;
    wxLongLong_t remains = (estimate + mStartTime) - now;
 
-   if (!IsShown() && elapsed > 500)
-   {
-      Show(true);
-   }
+   SetMessage(message);
 
    if (value != mLastValue)
    {
       mGauge->SetValue(value);
+      mGauge->Update();
       mLastValue = value;
    }
 
-   // Only update if a full second has passed.
-   if (now - mLastUpdate > 1000)
+   // Only update if a full second has passed or track progress is complete
+   if ((now - mLastUpdate > 1000) || (value == 1000))
    {
-      wxTimeSpan tsElapsed(0, 0, 0, elapsed);
-      wxTimeSpan tsRemains(0, 0, 0, remains);
+      if (m_bShowElapsedTime) {
+         wxTimeSpan tsElapsed(0, 0, 0, elapsed);
+         mElapsed->SetLabel(tsElapsed.Format(wxT("%H:%M:%S")));
+         mElapsed->SetName(mElapsed->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+         mElapsed->Update();
+      }
 
-      mElapsed->SetLabel(tsElapsed.Format(wxT("%H:%M:%S")));
-      mElapsed->SetName(mElapsed->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+      wxTimeSpan tsRemains(0, 0, 0, remains);
       mRemaining->SetLabel(tsRemains.Format(wxT("%H:%M:%S")));
       mRemaining->SetName(mRemaining->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+      mRemaining->Update();
 
       mLastUpdate = now;
    }
 
-   wxYieldIfNeeded();
+   wxDialogWrapper::Update();
 
-   return eProgressSuccess;
+   // Copied from wx 3.0.2 generic progress dialog
+   //
+   // we have to yield because not only we want to update the display but
+   // also to process the clicks on the cancel and skip buttons
+   // NOTE: using YieldFor() this call shouldn't give re-entrancy problems
+   //       for event handlers not interested to UI/user-input events.
+   //
+   // LL:  Added timer category to prevent extreme delays when processing effects
+   //      (and probably other things).  I do not yet know why this happens and
+   //      I'm not too keen on having timer events processed here, but you do
+   //      what you have to do.
+
+   // Nyquist effects call Update on every callback, but YieldFor is
+   // quite slow on Linux / Mac, so don't call too frequently. (bug 1575)
+   if ((now - mYieldTimer > 50) || (value == 1000)) {
+      wxEventLoopBase::GetActive()->YieldFor(wxEVT_CATEGORY_UI | wxEVT_CATEGORY_USER_INPUT | wxEVT_CATEGORY_TIMER);
+      mYieldTimer = now;
+   }
+
+   return ProgressResult::Success;
 }
 
 //
 // Update the time and, optionally, the message
 //
-int
-ProgressDialog::Update(double current, const wxString & message)
+ProgressResult ProgressDialog::Update(double current, const wxString & message)
 {
    return Update((int)(current * 1000), message);
 }
@@ -1373,8 +1411,7 @@ ProgressDialog::Update(double current, const wxString & message)
 //
 // Update the time and, optionally, the message
 //
-int
-ProgressDialog::Update(wxULongLong_t current, wxULongLong_t total, const wxString & message)
+ProgressResult ProgressDialog::Update(wxULongLong_t current, wxULongLong_t total, const wxString & message)
 {
    if (total != 0)
    {
@@ -1389,8 +1426,7 @@ ProgressDialog::Update(wxULongLong_t current, wxULongLong_t total, const wxStrin
 //
 // Update the time and, optionally, the message
 //
-int
-ProgressDialog::Update(wxLongLong current, wxLongLong total, const wxString & message)
+ProgressResult ProgressDialog::Update(wxLongLong current, wxLongLong total, const wxString & message)
 {
    if (total.GetValue() != 0)
    {
@@ -1405,8 +1441,7 @@ ProgressDialog::Update(wxLongLong current, wxLongLong total, const wxString & me
 //
 // Update the time and, optionally, the message
 //
-int
-ProgressDialog::Update(wxLongLong_t current, wxLongLong_t total, const wxString & message)
+ProgressResult ProgressDialog::Update(wxLongLong_t current, wxLongLong_t total, const wxString & message)
 {
    if (total != 0)
    {
@@ -1421,8 +1456,7 @@ ProgressDialog::Update(wxLongLong_t current, wxLongLong_t total, const wxString
 //
 // Update the time and, optionally, the message
 //
-int
-ProgressDialog::Update(int current, int total, const wxString & message)
+ProgressResult ProgressDialog::Update(int current, int total, const wxString & message)
 {
    if (total != 0)
    {
@@ -1437,8 +1471,7 @@ ProgressDialog::Update(int current, int total, const wxString & message)
 //
 // Update the time and, optionally, the message
 //
-int
-ProgressDialog::Update(double current, double total, const wxString & message)
+ProgressResult ProgressDialog::Update(double current, double total, const wxString & message)
 {
    if (total != 0)
    {
@@ -1453,45 +1486,93 @@ ProgressDialog::Update(double current, double total, const wxString & message)
 //
 // Update the message text
 //
-void
-ProgressDialog::SetMessage(const wxString & message)
+void ProgressDialog::SetMessage(const wxString & message)
 {
    if (!message.IsEmpty())
    {
-      wxSize sizeBefore = this->GetClientSize();
       mMessage->SetLabel(message);
-      wxSize sizeAfter = this->GetBestSize();
-      wxSize sizeNeeded;
-      sizeNeeded.x = wxMax(sizeBefore.x, sizeAfter.x);
-      sizeNeeded.y = wxMax(sizeBefore.y, sizeAfter.y);
-      this->SetClientSize(sizeNeeded);
-      wxYieldIfNeeded();
+
+      int w, h;
+      wxClientDC dc(mMessage);
+      dc.GetMultiLineTextExtent(message, &w, &h);
+
+      bool sizeUpdated = false;
+      wxSize ds = GetClientSize();
+
+      // TODO: make the following work in case of message tables
+      if (w > mLastW)
+      {
+         ds.x += (w - mLastW);
+         sizeUpdated = true;
+         mLastW = w;
+      }
+
+      if (h > mLastH)
+      {
+         ds.y += (h - mLastH);
+         sizeUpdated = true;
+         mLastH = h;
+      }
+
+      if (sizeUpdated)
+      {
+#if defined(__WXMAC__)
+         wxSize sz = mMessage->GetSize();
+         mMessage->SetMinSize(wxSize(wxMax(sz.x, mLastW), wxMax(sz.y, mLastH)));
+#endif
+         // No need to adjust for the margin here since we only add
+         // to the existing dimensions.
+         ds.x = wxMax(wxMax(ds.x, mLastW), wxMax(ds.y, mLastH));
+         SetClientSize(ds);
+         wxDialogWrapper::Update();
+      }
    }
 }
 
-void
-ProgressDialog::OnCancel(wxCommandEvent & WXUNUSED(event))
+//
+// Recursivaly search the window list for the given window.
+//
+bool ProgressDialog::SearchForWindow(const wxWindowList & list, const wxWindow *searchfor) const
 {
+   for (auto win : list)
+   {
+      if (win == searchfor || SearchForWindow(win->GetChildren(), searchfor))
+      {
+         return true;
+      }
+   }
+
+   return false;
+}
+
+void ProgressDialog::OnCancel(wxCommandEvent & WXUNUSED(event))
+{
+   if (!ConfirmAction(_("Are you sure you wish to cancel?"), _("Confirm Cancel"), wxID_CANCEL)) {
+      return;
+   }
    FindWindowById(wxID_CANCEL, this)->Disable();
    mCancel = true;
 }
 
-void
-ProgressDialog::OnStop(wxCommandEvent & WXUNUSED(event))
+void ProgressDialog::OnStop(wxCommandEvent & WXUNUSED(event))
 {
+   if (!ConfirmAction(_("Are you sure you wish to stop?"), _("Confirm Stop"), wxID_OK)) {
+      return;
+   }
    FindWindowById(wxID_OK, this)->Disable();
    mCancel = false;
    mStop = true;
 }
 
-void
-ProgressDialog::OnCloseWindow(wxCloseEvent & WXUNUSED(event))
+void ProgressDialog::OnCloseWindow(wxCloseEvent & WXUNUSED(event))
 {
+   if (!ConfirmAction(_("Are you sure you wish to close?"), _("Confirm Close"))) {
+      return;
+   }
    mCancel = true;
 }
 
-void
-ProgressDialog::Beep()
+void ProgressDialog::Beep() const
 {
    int after;
    bool should;
@@ -1506,10 +1587,12 @@ ProgressDialog::Beep()
       wxBusyCursor busy;
       wxSound s;
 
-      if (name.IsEmpty()) {
+      if (name.IsEmpty())
+      {
          s.Create(sizeof(beep), beep);
       }
-      else {
+      else
+      {
          s.Create(name);
       }
 
@@ -1520,59 +1603,130 @@ ProgressDialog::Beep()
    }
 }
 
+// Confirm action taken by user.
+// Returns TRUE if the user confirms Yes
+bool ProgressDialog::ConfirmAction(const wxString & sPrompt,
+                                   const wxString & sTitle,
+                                   int iButtonID /* = -1 */) {
+
+   // Check if confirmations are enabled?
+   // If not then return TRUE
+   if (m_bConfirmAction == false) {
+      return true;
+   }
+
+   AudacityMessageDialog dlgMessage(this,
+      sPrompt,
+      sTitle,
+      wxYES_NO | wxICON_QUESTION | wxNO_DEFAULT | wxSTAY_ON_TOP);
+   int iAction = dlgMessage.ShowModal();
+
+   bool bReturn = (iAction == wxID_YES);
+   if ((bReturn == false) && (iButtonID > -1)) {
+      // Set the focus back to the relevant button
+      FindWindowById(iButtonID, this)->SetFocus();
+   }
+
+   return bReturn;
+}
+
 TimerProgressDialog::TimerProgressDialog(const wxLongLong_t duration,
-                                          const wxString & title,
-                                          const wxString & message /*= wxEmptyString*/,
-                                          ProgressDialogFlags flags /*= pdlgEmptyFlags*/)
-: ProgressDialog(title, message, flags)
+                                         const wxString & title,
+                                         const MessageTable & columns,
+                                         int flags /* = pdlgDefaultFlags */,
+                                         const wxString & sRemainingLabelText /* = wxEmptyString */)
+: ProgressDialog(title, columns, flags, sRemainingLabelText)
 {
    mDuration = duration;
 }
 
-int TimerProgressDialog::Update(const wxString & message /*= wxEmptyString*/)
+ProgressResult TimerProgressDialog::UpdateProgress()
 {
    if (mCancel)
    {
       // for compatibility with old Update, that returned false on cancel
-      return eProgressCancelled;
+      return ProgressResult::Cancelled;
    }
    else if (mStop)
    {
-      return eProgressStopped;
+      return ProgressResult::Stopped;
    }
 
-   SetMessage(message);
-
    wxLongLong_t now = wxGetLocalTimeMillis().GetValue();
    wxLongLong_t elapsed = now - mStartTime;
-   wxLongLong_t remains = mStartTime + mDuration - now;
 
-   if (!IsShown() && elapsed > 500)
+   if (elapsed < 500)
    {
-      Show(true);
+      return ProgressResult::Success;
    }
 
+   if (mIsTransparent)
+   {
+      SetTransparent(255);
+      mIsTransparent = false;
+   }
+
+   wxLongLong_t remains = mStartTime + mDuration - now;
+
    int nGaugeValue = (1000 * elapsed) / mDuration; // range = [0,1000]
+   // Running in TimerRecordDialog::RunWaitDialog(), for some unknown reason, 
+   // nGaugeValue here is often a little over 1000. 
+   // From testing, it's never shown bigger than 1009, but 
+   // give it a little extra, to 1010. 
+   //   wxASSERT((nGaugeValue >= 0) && (nGaugeValue <= 1000)); // This ought to work. 
+   // wxASSERT((nGaugeValue >= 0) && (nGaugeValue <= 1010));
+   //
+   // stf. Update was being called after wxMilliSleep(<ms>), which could be up to <ms>
+   // beyond the completion time. My gusess is that the microsleep in RunWaitDialog was originally 10 ms
+   // (same as other uses of Update) but was updated to kTimerInterval = 50 ms, thus triggering
+   // the Assert (Bug 1367). By calling Update() before sleeping then I think nGaugeValue <= 1000 should work.
    wxASSERT((nGaugeValue >= 0) && (nGaugeValue <= 1000));
+
    if (nGaugeValue != mLastValue)
    {
       mGauge->SetValue(nGaugeValue);
+      mGauge->Update();
       mLastValue = nGaugeValue;
    }
 
    // Only update if a full second has passed.
    if (now - mLastUpdate > 1000)
    {
-      wxTimeSpan tsElapsed(0, 0, 0, elapsed);
-      wxTimeSpan tsRemains(0, 0, 0, remains);
+      if (m_bShowElapsedTime) {
+         wxTimeSpan tsElapsed(0, 0, 0, elapsed);
+         mElapsed->SetLabel(tsElapsed.Format(wxT("%H:%M:%S")));
+         mElapsed->Update();
+      }
 
-      mElapsed->SetLabel(tsElapsed.Format(wxT("%H:%M:%S")));
+      wxTimeSpan tsRemains(0, 0, 0, remains);
       mRemaining->SetLabel(tsRemains.Format(wxT("%H:%M:%S")));
+      mRemaining->Update();
 
       mLastUpdate = now;
    }
 
-   wxYieldIfNeeded();
+   // Copied from wx 3.0.2 generic progress dialog
+   //
+   // we have to yield because not only we want to update the display but
+   // also to process the clicks on the cancel and skip buttons
+   // NOTE: using YieldFor() this call shouldn't give re-entrancy problems
+   //       for event handlers not interested to UI/user-input events.
+   //
+   // LL:  Added timer category to prevent extreme delays when processing effects
+   //      (and probably other things).  I do not yet know why this happens and
+   //      I'm not too keen on having timer events processed here, but you do
+   //      what you have to do.
+   wxEventLoopBase::GetActive()->YieldFor(wxEVT_CATEGORY_UI | wxEVT_CATEGORY_USER_INPUT | wxEVT_CATEGORY_TIMER);
 
-   return eProgressSuccess;
+   // MY: Added this after the YieldFor to check we haven't changed the outcome based on buttons pressed...
+   auto iReturn = ProgressResult::Success;
+   if (mCancel)
+   {
+      iReturn = ProgressResult::Cancelled;
+   }
+   else if (mStop)
+   {
+      iReturn = ProgressResult::Stopped;
+   }
+   return iReturn;
 }
